@@ -41,8 +41,10 @@ fn main() -> io::Result<()> {
         \n\
         [Introduction](index.md)\n\
         * [Installation](installation.md)\n\
+        * [Build from source](build.md)\n\
         * [Contributing](contributing.md)\n\
         * [GNU test coverage](test_coverage.md)\n\
+        * [Extensions](extensions.md)\n\
         \n\
         # Reference\n\
         * [Multi-call binary](multicall.md)\n",
@@ -89,6 +91,15 @@ fn main() -> io::Result<()> {
             continue;
         }
         let p = format!("docs/src/utils/{}.md", name);
+
+        let markdown = File::open(format!("src/uu/{name}/{name}.md"))
+            .and_then(|mut f: File| {
+                let mut s = String::new();
+                f.read_to_string(&mut s)?;
+                Ok(s)
+            })
+            .ok();
+
         if let Ok(f) = File::create(&p) {
             MDWriter {
                 w: Box::new(f),
@@ -96,6 +107,7 @@ fn main() -> io::Result<()> {
                 name,
                 tldr_zip: &mut tldr_zip,
                 utils_per_platform: &utils_per_platform,
+                markdown,
             }
             .markdown()?;
             println!("Wrote to '{}'", p);
@@ -109,10 +121,11 @@ fn main() -> io::Result<()> {
 
 struct MDWriter<'a, 'b> {
     w: Box<dyn Write>,
-    command: Command<'a>,
+    command: Command,
     name: &'a str,
     tldr_zip: &'b mut Option<ZipArchive<File>>,
     utils_per_platform: &'b HashMap<&'b str, Vec<String>>,
+    markdown: Option<String>,
 }
 
 impl<'a, 'b> MDWriter<'a, 'b> {
@@ -122,6 +135,7 @@ impl<'a, 'b> MDWriter<'a, 'b> {
         self.usage()?;
         self.description()?;
         self.options()?;
+        self.after_help()?;
         self.examples()
     }
 
@@ -167,18 +181,25 @@ impl<'a, 'b> MDWriter<'a, 'b> {
         let mut usage: String = self
             .command
             .render_usage()
+            .to_string()
             .lines()
-            .skip(1)
+            .map(|l| l.strip_prefix("Usage:").unwrap_or(l))
             .map(|l| l.trim())
             .filter(|l| !l.is_empty())
             .collect::<Vec<_>>()
             .join("\n");
-        usage = usage.replace(uucore::execution_phrase(), self.name);
+        usage = usage
+            .to_string()
+            .replace(uucore::execution_phrase(), self.name);
         writeln!(self.w, "{}", usage)?;
         writeln!(self.w, "```")
     }
 
     fn description(&mut self) -> io::Result<()> {
+        if let Some(after_help) = self.markdown_section("about") {
+            return writeln!(self.w, "\n\n{}", after_help);
+        }
+
         if let Some(about) = self
             .command
             .get_long_about()
@@ -190,6 +211,22 @@ impl<'a, 'b> MDWriter<'a, 'b> {
         }
     }
 
+    fn after_help(&mut self) -> io::Result<()> {
+        if let Some(after_help) = self.markdown_section("after help") {
+            return writeln!(self.w, "\n\n{}", after_help);
+        }
+
+        if let Some(after_help) = self
+            .command
+            .get_after_long_help()
+            .or_else(|| self.command.get_after_help())
+        {
+            writeln!(self.w, "\n\n{}", after_help)
+        } else {
+            Ok(())
+        }
+    }
+
     fn examples(&mut self) -> io::Result<()> {
         if let Some(zip) = self.tldr_zip {
             let content = if let Some(f) =
@@ -282,11 +319,40 @@ impl<'a, 'b> MDWriter<'a, 'b> {
             writeln!(
                 self.w,
                 "<dd>\n\n{}\n\n</dd>",
-                arg.get_help().unwrap_or_default().replace('\n', "<br />")
+                arg.get_help()
+                    .unwrap_or_default()
+                    .to_string()
+                    .replace('\n', "<br />")
             )?;
         }
         writeln!(self.w, "</dl>\n")
     }
+
+    fn markdown_section(&self, section: &str) -> Option<String> {
+        let md = self.markdown.as_ref()?;
+        let section = section.to_lowercase();
+
+        fn is_section_header(line: &str, section: &str) -> bool {
+            line.strip_prefix("##")
+                .map_or(false, |l| l.trim().to_lowercase() == section)
+        }
+
+        let result = md
+            .lines()
+            .skip_while(|&l| !is_section_header(l, &section))
+            .skip(1)
+            .take_while(|l| !l.starts_with("##"))
+            .collect::<Vec<_>>()
+            .join("\n")
+            .trim()
+            .to_string();
+
+        if !result.is_empty() {
+            Some(result)
+        } else {
+            None
+        }
+    }
 }
 
 fn get_zip_content(archive: &mut ZipArchive<impl Read + Seek>, name: &str) -> Option<String> {
