@@ -10,11 +10,9 @@
 
 // spell-checker:ignore (ToDO) nonprint nonblank nonprinting
 
-#[cfg(unix)]
-extern crate unix_socket;
-
 // last synced with: cat (GNU coreutils) 8.13
-use clap::{crate_version, Arg, Command};
+use clap::{crate_version, Arg, ArgAction, Command};
+use is_terminal::IsTerminal;
 use std::fs::{metadata, File};
 use std::io::{self, Read, Write};
 use thiserror::Error;
@@ -35,13 +33,11 @@ use std::net::Shutdown;
 #[cfg(unix)]
 use std::os::unix::fs::FileTypeExt;
 #[cfg(unix)]
-use unix_socket::UnixStream;
-use uucore::format_usage;
+use std::os::unix::net::UnixStream;
+use uucore::{format_usage, help_about, help_usage};
 
-static NAME: &str = "cat";
-static USAGE: &str = "{} [OPTION]... [FILE]...";
-static ABOUT: &str = "Concatenate FILE(s), or standard input, to standard output
-With no FILE, or when FILE is -, read standard input.";
+const USAGE: &str = help_usage!("cat.md");
+const ABOUT: &str = help_about!("cat.md");
 
 #[derive(Error, Debug)]
 enum CatError {
@@ -188,9 +184,9 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
 
     let matches = uu_app().try_get_matches_from(args)?;
 
-    let number_mode = if matches.contains_id(options::NUMBER_NONBLANK) {
+    let number_mode = if matches.get_flag(options::NUMBER_NONBLANK) {
         NumberingMode::NonEmpty
-    } else if matches.contains_id(options::NUMBER) {
+    } else if matches.get_flag(options::NUMBER) {
         NumberingMode::All
     } else {
         NumberingMode::None
@@ -203,7 +199,7 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
         options::SHOW_NONPRINTING.to_owned(),
     ]
     .iter()
-    .any(|v| matches.contains_id(v));
+    .any(|v| matches.get_flag(v));
 
     let show_ends = vec![
         options::SHOW_ENDS.to_owned(),
@@ -211,7 +207,7 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
         options::SHOW_NONPRINTING_ENDS.to_owned(),
     ]
     .iter()
-    .any(|v| matches.contains_id(v));
+    .any(|v| matches.get_flag(v));
 
     let show_tabs = vec![
         options::SHOW_ALL.to_owned(),
@@ -219,9 +215,9 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
         options::SHOW_NONPRINTING_TABS.to_owned(),
     ]
     .iter()
-    .any(|v| matches.contains_id(v));
+    .any(|v| matches.get_flag(v));
 
-    let squeeze_blank = matches.contains_id(options::SQUEEZE_BLANK);
+    let squeeze_blank = matches.get_flag(options::SQUEEZE_BLANK);
     let files: Vec<String> = match matches.get_many::<String>(options::FILE) {
         Some(v) => v.clone().map(|v| v.to_owned()).collect(),
         None => vec!["-".to_owned()],
@@ -237,9 +233,8 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
     cat_files(&files, &options)
 }
 
-pub fn uu_app<'a>() -> Command<'a> {
+pub fn uu_app() -> Command {
     Command::new(uucore::util_name())
-        .name(NAME)
         .version(crate_version!())
         .override_usage(format_usage(USAGE))
         .about(ABOUT)
@@ -247,62 +242,71 @@ pub fn uu_app<'a>() -> Command<'a> {
         .arg(
             Arg::new(options::FILE)
                 .hide(true)
-                .multiple_occurrences(true)
+                .action(clap::ArgAction::Append)
                 .value_hint(clap::ValueHint::FilePath),
         )
         .arg(
             Arg::new(options::SHOW_ALL)
                 .short('A')
                 .long(options::SHOW_ALL)
-                .help("equivalent to -vET"),
+                .help("equivalent to -vET")
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::NUMBER_NONBLANK)
                 .short('b')
                 .long(options::NUMBER_NONBLANK)
                 .help("number nonempty output lines, overrides -n")
-                .overrides_with(options::NUMBER),
+                .overrides_with(options::NUMBER)
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::SHOW_NONPRINTING_ENDS)
                 .short('e')
-                .help("equivalent to -vE"),
+                .help("equivalent to -vE")
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::SHOW_ENDS)
                 .short('E')
                 .long(options::SHOW_ENDS)
-                .help("display $ at end of each line"),
+                .help("display $ at end of each line")
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::NUMBER)
                 .short('n')
                 .long(options::NUMBER)
-                .help("number all output lines"),
+                .help("number all output lines")
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::SQUEEZE_BLANK)
                 .short('s')
                 .long(options::SQUEEZE_BLANK)
-                .help("suppress repeated empty output lines"),
+                .help("suppress repeated empty output lines")
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::SHOW_NONPRINTING_TABS)
                 .short('t')
                 .long(options::SHOW_NONPRINTING_TABS)
-                .help("equivalent to -vT"),
+                .help("equivalent to -vT")
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::SHOW_TABS)
                 .short('T')
                 .long(options::SHOW_TABS)
-                .help("display TAB characters at ^I"),
+                .help("display TAB characters at ^I")
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::SHOW_NONPRINTING)
                 .short('v')
                 .long(options::SHOW_NONPRINTING)
-                .help("use ^ and M- notation, except for LF (\\n) and TAB (\\t)"),
+                .help("use ^ and M- notation, except for LF (\\n) and TAB (\\t)")
+                .action(ArgAction::SetTrue),
         )
 }
 
@@ -329,7 +333,7 @@ fn cat_path(
             let stdin = io::stdin();
             let mut handle = InputHandle {
                 reader: stdin,
-                is_interactive: atty::is(atty::Stream::Stdin),
+                is_interactive: std::io::stdin().is_terminal(),
             };
             cat_handle(&mut handle, options, state)
         }
@@ -420,7 +424,7 @@ fn get_input_type(path: &str) -> CatResult<InputType> {
         ft if ft.is_file() => Ok(InputType::File),
         ft if ft.is_symlink() => Ok(InputType::SymLink),
         _ => Err(CatError::UnknownFiletype {
-            ft_debug: format!("{:?}", ft),
+            ft_debug: format!("{ft:?}"),
         }),
     }
 }
