@@ -9,21 +9,19 @@
 
 use uucore::display::Quotable;
 pub use uucore::entries::{self, Group, Locate, Passwd};
-use uucore::format_usage;
 use uucore::perms::{chown_base, options, IfFrom};
+use uucore::{format_usage, help_about, help_usage};
 
 use uucore::error::{FromIo, UResult, USimpleError};
 
-use clap::{crate_version, Arg, ArgMatches, Command};
+use clap::{crate_version, Arg, ArgAction, ArgMatches, Command};
 
 use std::fs;
 use std::os::unix::fs::MetadataExt;
 
-static ABOUT: &str = "change file owner and group";
+static ABOUT: &str = help_about!("chown.md");
 
-const USAGE: &str = "\
-    {} [OPTION]... [OWNER][:[GROUP]] FILE...
-    {} [OPTION]... --reference=RFILE FILE...";
+const USAGE: &str = help_usage!("chown.md");
 
 fn parse_gid_uid_and_filter(matches: &ArgMatches) -> UResult<(Option<u32>, Option<u32>, IfFrom)> {
     let filter = if let Some(spec) = matches.get_one::<String>(options::FROM) {
@@ -40,7 +38,7 @@ fn parse_gid_uid_and_filter(matches: &ArgMatches) -> UResult<(Option<u32>, Optio
     let dest_uid: Option<u32>;
     let dest_gid: Option<u32>;
     if let Some(file) = matches.get_one::<String>(options::REFERENCE) {
-        let meta = fs::metadata(&file)
+        let meta = fs::metadata(file)
             .map_err_context(|| format!("failed to get attributes of {}", file.quote()))?;
         dest_gid = Some(meta.gid());
         dest_uid = Some(meta.uid());
@@ -63,22 +61,25 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
     )
 }
 
-pub fn uu_app<'a>() -> Command<'a> {
+pub fn uu_app() -> Command {
     Command::new(uucore::util_name())
         .version(crate_version!())
         .about(ABOUT)
         .override_usage(format_usage(USAGE))
         .infer_long_args(true)
+        .disable_help_flag(true)
         .arg(
             Arg::new(options::HELP)
                 .long(options::HELP)
-                .help("Print help information."),
+                .help("Print help information.")
+                .action(ArgAction::Help),
         )
         .arg(
             Arg::new(options::verbosity::CHANGES)
                 .short('c')
                 .long(options::verbosity::CHANGES)
-                .help("like verbose but report only when a change is made"),
+                .help("like verbose but report only when a change is made")
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::dereference::DEREFERENCE)
@@ -86,7 +87,8 @@ pub fn uu_app<'a>() -> Command<'a> {
                 .help(
                     "affect the referent of each symbolic link (this is the default), \
                     rather than the symbolic link itself",
-                ),
+                )
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::dereference::NO_DEREFERENCE)
@@ -95,7 +97,8 @@ pub fn uu_app<'a>() -> Command<'a> {
                 .help(
                     "affect symbolic links instead of any referenced file \
                     (useful only on systems that can change the ownership of a symlink)",
-                ),
+                )
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::FROM)
@@ -111,23 +114,27 @@ pub fn uu_app<'a>() -> Command<'a> {
         .arg(
             Arg::new(options::preserve_root::PRESERVE)
                 .long(options::preserve_root::PRESERVE)
-                .help("fail to operate recursively on '/'"),
+                .help("fail to operate recursively on '/'")
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::preserve_root::NO_PRESERVE)
                 .long(options::preserve_root::NO_PRESERVE)
-                .help("do not treat '/' specially (the default)"),
+                .help("do not treat '/' specially (the default)")
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::verbosity::QUIET)
                 .long(options::verbosity::QUIET)
-                .help("suppress most error messages"),
+                .help("suppress most error messages")
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::RECURSIVE)
                 .short('R')
                 .long(options::RECURSIVE)
-                .help("operate on files and directories recursively"),
+                .help("operate on files and directories recursively")
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::REFERENCE)
@@ -135,36 +142,41 @@ pub fn uu_app<'a>() -> Command<'a> {
                 .help("use RFILE's owner and group rather than specifying OWNER:GROUP values")
                 .value_name("RFILE")
                 .value_hint(clap::ValueHint::FilePath)
-                .min_values(1),
+                .num_args(1..),
         )
         .arg(
             Arg::new(options::verbosity::SILENT)
                 .short('f')
-                .long(options::verbosity::SILENT),
+                .long(options::verbosity::SILENT)
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::traverse::TRAVERSE)
                 .short(options::traverse::TRAVERSE.chars().next().unwrap())
                 .help("if a command line argument is a symbolic link to a directory, traverse it")
-                .overrides_with_all(&[options::traverse::EVERY, options::traverse::NO_TRAVERSE]),
+                .overrides_with_all([options::traverse::EVERY, options::traverse::NO_TRAVERSE])
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::traverse::EVERY)
                 .short(options::traverse::EVERY.chars().next().unwrap())
                 .help("traverse every symbolic link to a directory encountered")
-                .overrides_with_all(&[options::traverse::TRAVERSE, options::traverse::NO_TRAVERSE]),
+                .overrides_with_all([options::traverse::TRAVERSE, options::traverse::NO_TRAVERSE])
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::traverse::NO_TRAVERSE)
                 .short(options::traverse::NO_TRAVERSE.chars().next().unwrap())
                 .help("do not traverse any symbolic links (default)")
-                .overrides_with_all(&[options::traverse::TRAVERSE, options::traverse::EVERY]),
+                .overrides_with_all([options::traverse::TRAVERSE, options::traverse::EVERY])
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::verbosity::VERBOSE)
                 .long(options::verbosity::VERBOSE)
                 .short('v')
-                .help("output a diagnostic for every file processed"),
+                .help("output a diagnostic for every file processed")
+                .action(ArgAction::SetTrue),
         )
 }
 
@@ -186,7 +198,9 @@ fn parse_spec(spec: &str, sep: char) -> UResult<(Option<u32>, Option<u32>)> {
     let user = args.next().unwrap_or("");
     let group = args.next().unwrap_or("");
 
-    let uid = if !user.is_empty() {
+    let uid = if user.is_empty() {
+        None
+    } else {
         Some(match Passwd::locate(user) {
             Ok(u) => u.uid, // We have been able to get the uid
             Err(_) =>
@@ -213,10 +227,10 @@ fn parse_spec(spec: &str, sep: char) -> UResult<(Option<u32>, Option<u32>)> {
                 }
             }
         })
-    } else {
-        None
     };
-    let gid = if !group.is_empty() {
+    let gid = if group.is_empty() {
+        None
+    } else {
         Some(match Group::locate(group) {
             Ok(g) => g.gid,
             Err(_) => match group.parse() {
@@ -229,9 +243,20 @@ fn parse_spec(spec: &str, sep: char) -> UResult<(Option<u32>, Option<u32>)> {
                 }
             },
         })
-    } else {
-        None
     };
+
+    if user.chars().next().map(char::is_numeric).unwrap_or(false)
+        && group.is_empty()
+        && spec != user
+    {
+        // if the arg starts with an id numeric value, the group isn't set but the separator is provided,
+        // we should fail with an error
+        return Err(USimpleError::new(
+            1,
+            format!("invalid spec: {}", spec.quote()),
+        ));
+    }
+
     Ok((uid, gid))
 }
 
