@@ -8,25 +8,23 @@
 
 // spell-checker:ignore (chrono) Datelike Timelike ; (format) DATEFILE MMDDhhmm ; (vars) datetime datetimes
 
+use chrono::format::{Item, StrftimeItems};
 use chrono::{DateTime, FixedOffset, Local, Offset, Utc};
 #[cfg(windows)]
 use chrono::{Datelike, Timelike};
-use clap::{crate_version, Arg, Command};
+use clap::{crate_version, Arg, ArgAction, Command};
 #[cfg(all(unix, not(target_os = "macos"), not(target_os = "redox")))]
 use libc::{clock_settime, timespec, CLOCK_REALTIME};
 use std::fs::File;
 use std::io::{BufRead, BufReader};
 use std::path::PathBuf;
 use uucore::display::Quotable;
-#[cfg(not(any(target_os = "macos", target_os = "redox")))]
+#[cfg(not(any(target_os = "redox")))]
 use uucore::error::FromIo;
 use uucore::error::{UResult, USimpleError};
-use uucore::{format_usage, show_error};
+use uucore::{format_usage, help_about, help_usage, show};
 #[cfg(windows)]
-use winapi::{
-    shared::minwindef::WORD,
-    um::{minwinbase::SYSTEMTIME, sysinfoapi::SetSystemTime},
-};
+use windows_sys::Win32::{Foundation::SYSTEMTIME, System::SystemInformation::SetSystemTime};
 
 // Options
 const DATE: &str = "date";
@@ -38,10 +36,8 @@ const MINUTE: &str = "minute";
 const SECOND: &str = "second";
 const NS: &str = "ns";
 
-const ABOUT: &str = "print or set the system date and time";
-const USAGE: &str = "\
-    {} [OPTION]... [+FORMAT]...
-    {} [OPTION]... [MMDDhhmm[[CC]YY][.ss]]";
+const ABOUT: &str = help_about!("date.md");
+const USAGE: &str = help_usage!("date.md");
 
 const OPT_DATE: &str = "date";
 const OPT_FORMAT: &str = "format";
@@ -118,8 +114,8 @@ impl<'a> From<&'a str> for Iso8601Format {
             SECONDS | SECOND => Self::Seconds,
             NS => Self::Ns,
             DATE => Self::Date,
-            // Should be caught by clap
-            _ => panic!("Invalid format: {}", s),
+            // Note: This is caught by clap via `possible_values`
+            _ => unreachable!(),
         }
     }
 }
@@ -137,7 +133,7 @@ impl<'a> From<&'a str> for Rfc3339Format {
             SECONDS | SECOND => Self::Seconds,
             NS => Self::Ns,
             // Should be caught by clap
-            _ => panic!("Invalid format: {}", s),
+            _ => panic!("Invalid format: {s}"),
         }
     }
 }
@@ -160,7 +156,7 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
         .map(|mut iter| iter.next().unwrap_or(&DATE.to_string()).as_str().into())
     {
         Format::Iso8601(fmt)
-    } else if matches.contains_id(OPT_RFC_EMAIL) {
+    } else if matches.get_flag(OPT_RFC_EMAIL) {
         Format::Rfc5322
     } else if let Some(fmt) = matches
         .get_one::<String>(OPT_RFC_3339)
@@ -191,7 +187,7 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
     };
 
     let settings = Settings {
-        utc: matches.contains_id(OPT_UNIVERSAL),
+        utc: matches.get_flag(OPT_UNIVERSAL),
         format,
         date_source,
         set_to,
@@ -207,9 +203,6 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
 
         return set_system_datetime(date);
     } else {
-        // Declare a file here because it needs to outlive the `dates` iterator.
-        let file: File;
-
         // Get the current time, either in the local time zone or UTC.
         let now: DateTime<FixedOffset> = if settings.utc {
             let now = Utc::now();
@@ -227,9 +220,16 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
                 Box::new(iter)
             }
             DateSource::File(ref path) => {
-                file = File::open(path).unwrap();
+                if path.is_dir() {
+                    return Err(USimpleError::new(
+                        2,
+                        format!("expected file, got directory {}", path.quote()),
+                    ));
+                }
+                let file = File::open(path)
+                    .map_err_context(|| path.as_os_str().to_string_lossy().to_string())?;
                 let lines = BufReader::new(file).lines();
-                let iter = lines.filter_map(Result::ok).map(parse_date);
+                let iter = lines.map_while(Result::ok).map(parse_date);
                 Box::new(iter)
             }
             DateSource::Now => {
@@ -246,10 +246,25 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
                 Ok(date) => {
                     // GNU `date` uses `%N` for nano seconds, however crate::chrono uses `%f`
                     let format_string = &format_string.replace("%N", "%f");
-                    let formatted = date.format(format_string).to_string().replace("%f", "%N");
-                    println!("{}", formatted);
+                    // Hack to work around panic in chrono,
+                    // TODO - remove when a fix for https://github.com/chronotope/chrono/issues/623 is released
+                    let format_items = StrftimeItems::new(format_string);
+                    if format_items.clone().any(|i| i == Item::Error) {
+                        return Err(USimpleError::new(
+                            1,
+                            format!("invalid format {}", format_string.replace("%f", "%N")),
+                        ));
+                    }
+                    let formatted = date
+                        .format_with_items(format_items)
+                        .to_string()
+                        .replace("%f", "%N");
+                    println!("{formatted}");
                 }
-                Err((input, _err)) => show_error!("invalid date {}", input.quote()),
+                Err((input, _err)) => show!(USimpleError::new(
+                    1,
+                    format!("invalid date {}", input.quote())
+                )),
             }
         }
     }
@@ -257,7 +272,7 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
     Ok(())
 }
 
-pub fn uu_app<'a>() -> Command<'a> {
+pub fn uu_app() -> Command {
     Command::new(uucore::util_name())
         .version(crate_version!())
         .about(ABOUT)
@@ -267,7 +282,6 @@ pub fn uu_app<'a>() -> Command<'a> {
             Arg::new(OPT_DATE)
                 .short('d')
                 .long(OPT_DATE)
-                .takes_value(true)
                 .value_name("STRING")
                 .help("display time described by STRING, not 'now'"),
         )
@@ -275,7 +289,6 @@ pub fn uu_app<'a>() -> Command<'a> {
             Arg::new(OPT_FILE)
                 .short('f')
                 .long(OPT_FILE)
-                .takes_value(true)
                 .value_name("DATEFILE")
                 .value_hint(clap::ValueHint::FilePath)
                 .help("like --date; once for each line of DATEFILE"),
@@ -284,33 +297,36 @@ pub fn uu_app<'a>() -> Command<'a> {
             Arg::new(OPT_ISO_8601)
                 .short('I')
                 .long(OPT_ISO_8601)
-                .takes_value(true)
                 .value_name("FMT")
+                .value_parser([DATE, HOUR, HOURS, MINUTE, MINUTES, SECOND, SECONDS, NS])
+                .num_args(0..=1)
+                .default_missing_value(OPT_DATE)
                 .help(ISO_8601_HELP_STRING),
         )
         .arg(
             Arg::new(OPT_RFC_EMAIL)
                 .short('R')
                 .long(OPT_RFC_EMAIL)
-                .help(RFC_5322_HELP_STRING),
+                .help(RFC_5322_HELP_STRING)
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(OPT_RFC_3339)
                 .long(OPT_RFC_3339)
-                .takes_value(true)
                 .value_name("FMT")
+                .value_parser([DATE, SECOND, SECONDS, NS])
                 .help(RFC_3339_HELP_STRING),
         )
         .arg(
             Arg::new(OPT_DEBUG)
                 .long(OPT_DEBUG)
-                .help("annotate the parsed date, and warn about questionable usage to stderr"),
+                .help("annotate the parsed date, and warn about questionable usage to stderr")
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(OPT_REFERENCE)
                 .short('r')
                 .long(OPT_REFERENCE)
-                .takes_value(true)
                 .value_name("FILE")
                 .value_hint(clap::ValueHint::AnyPath)
                 .help("display the last modification time of FILE"),
@@ -319,7 +335,6 @@ pub fn uu_app<'a>() -> Command<'a> {
             Arg::new(OPT_SET)
                 .short('s')
                 .long(OPT_SET)
-                .takes_value(true)
                 .value_name("STRING")
                 .help(OPT_SET_HELP_STRING),
         )
@@ -328,9 +343,10 @@ pub fn uu_app<'a>() -> Command<'a> {
                 .short('u')
                 .long(OPT_UNIVERSAL)
                 .alias(OPT_UNIVERSAL_2)
-                .help("print or set Coordinated Universal Time (UTC)"),
+                .help("print or set Coordinated Universal Time (UTC)")
+                .action(ArgAction::SetTrue),
         )
-        .arg(Arg::new(OPT_FORMAT).multiple_occurrences(false))
+        .arg(Arg::new(OPT_FORMAT))
 }
 
 /// Return the appropriate format string for the given settings.
@@ -398,10 +414,10 @@ fn set_system_datetime(date: DateTime<Utc>) -> UResult<()> {
 
     let result = unsafe { clock_settime(CLOCK_REALTIME, &timespec) };
 
-    if result != 0 {
-        Err(std::io::Error::last_os_error().map_err_context(|| "cannot set date".to_string()))
-    } else {
+    if result == 0 {
         Ok(())
+    } else {
+        Err(std::io::Error::last_os_error().map_err_context(|| "cannot set date".to_string()))
     }
 }
 
@@ -412,16 +428,16 @@ fn set_system_datetime(date: DateTime<Utc>) -> UResult<()> {
 /// https://docs.microsoft.com/en-us/windows/win32/api/minwinbase/ns-minwinbase-systemtime
 fn set_system_datetime(date: DateTime<Utc>) -> UResult<()> {
     let system_time = SYSTEMTIME {
-        wYear: date.year() as WORD,
-        wMonth: date.month() as WORD,
+        wYear: date.year() as u16,
+        wMonth: date.month() as u16,
         // Ignored
         wDayOfWeek: 0,
-        wDay: date.day() as WORD,
-        wHour: date.hour() as WORD,
-        wMinute: date.minute() as WORD,
-        wSecond: date.second() as WORD,
+        wDay: date.day() as u16,
+        wHour: date.hour() as u16,
+        wMinute: date.minute() as u16,
+        wSecond: date.second() as u16,
         // TODO: be careful of leap seconds - valid range is [0, 999] - how to handle?
-        wMilliseconds: ((date.nanosecond() / 1_000_000) % 1000) as WORD,
+        wMilliseconds: ((date.nanosecond() / 1_000_000) % 1000) as u16,
     };
 
     let result = unsafe { SetSystemTime(&system_time) };
