@@ -5,11 +5,9 @@
 //  * For the full copyright and license information, please view the LICENSE
 //  * file that was distributed with this source code.
 
-#[macro_use]
-extern crate uucore;
-
 use chrono::prelude::DateTime;
 use chrono::Local;
+use clap::ArgAction;
 use clap::{crate_version, Arg, ArgMatches, Command};
 use glob::Pattern;
 use std::collections::HashSet;
@@ -36,19 +34,18 @@ use std::{error::Error, fmt::Display};
 use uucore::display::{print_verbatim, Quotable};
 use uucore::error::FromIo;
 use uucore::error::{UError, UResult};
-use uucore::format_usage;
 use uucore::parse_glob;
 use uucore::parse_size::{parse_size, ParseSizeError};
+use uucore::{
+    crash, format_usage, help_about, help_section, help_usage, show, show_error, show_warning,
+};
 #[cfg(windows)]
-use winapi::shared::minwindef::{DWORD, LPVOID};
+use windows_sys::Win32::Foundation::HANDLE;
 #[cfg(windows)]
-use winapi::um::fileapi::{FILE_ID_INFO, FILE_STANDARD_INFO};
-#[cfg(windows)]
-use winapi::um::minwinbase::{FileIdInfo, FileStandardInfo};
-#[cfg(windows)]
-use winapi::um::winbase::GetFileInformationByHandleEx;
-#[cfg(windows)]
-use winapi::um::winnt::FILE_ID_128;
+use windows_sys::Win32::Storage::FileSystem::{
+    FileIdInfo, FileStandardInfo, GetFileInformationByHandleEx, FILE_ID_128, FILE_ID_INFO,
+    FILE_STANDARD_INFO,
+};
 
 mod options {
     pub const HELP: &str = "help";
@@ -78,25 +75,9 @@ mod options {
     pub const FILE: &str = "FILE";
 }
 
-const ABOUT: &str = "estimate file space usage";
-const LONG_HELP: &str = "
-Display values are in units of the first available SIZE from --block-size,
-and the DU_BLOCK_SIZE, BLOCK_SIZE and BLOCKSIZE environment variables.
-Otherwise, units default to 1024 bytes (or 512 if POSIXLY_CORRECT is set).
-
-SIZE is an integer and optional unit (example: 10M is 10*1024*1024).
-Units are K, M, G, T, P, E, Z, Y (powers of 1024) or KB, MB,... (powers
-of 1000).
-
-PATTERN allows some advanced exclusions. For example, the following syntaxes
-are supported:
-? will match only one character
-* will match zero or more characters
-{a,b} will match a or b
-";
-const USAGE: &str = "\
-    {} [OPTION]... [FILE]...
-    {} [OPTION]... --files0-from=F";
+const ABOUT: &str = help_about!("du.md");
+const AFTER_HELP: &str = help_section!("after help", "du.md");
+const USAGE: &str = help_usage!("du.md");
 
 // TODO: Support Z & Y (currently limited by size of u64)
 const UNITS: [(char, u32); 6] = [('E', 6), ('P', 5), ('T', 4), ('G', 3), ('M', 2), ('K', 1)];
@@ -148,7 +129,7 @@ impl Stat {
             path,
             is_dir: metadata.is_dir(),
             size: metadata.len(),
-            blocks: metadata.blocks() as u64,
+            blocks: metadata.blocks(),
             inodes: 1,
             inode: Some(file_info),
             created: birth_u64(&metadata),
@@ -193,7 +174,7 @@ fn birth_u64(meta: &Metadata) -> Option<u64> {
     meta.created()
         .ok()
         .and_then(|t| t.duration_since(UNIX_EPOCH).ok())
-        .map(|e| e.as_secs() as u64)
+        .map(|e| e.as_secs())
 }
 
 #[cfg(windows)]
@@ -207,21 +188,19 @@ fn get_size_on_disk(path: &Path) -> u64 {
         Err(_) => return size_on_disk, // opening directories will fail
     };
 
-    let handle = file.as_raw_handle();
-
     unsafe {
         let mut file_info: FILE_STANDARD_INFO = core::mem::zeroed();
         let file_info_ptr: *mut FILE_STANDARD_INFO = &mut file_info;
 
         let success = GetFileInformationByHandleEx(
-            handle,
+            file.as_raw_handle() as HANDLE,
             FileStandardInfo,
-            file_info_ptr as LPVOID,
-            std::mem::size_of::<FILE_STANDARD_INFO>() as DWORD,
+            file_info_ptr as _,
+            std::mem::size_of::<FILE_STANDARD_INFO>() as u32,
         );
 
         if success != 0 {
-            size_on_disk = *file_info.AllocationSize.QuadPart() as u64;
+            size_on_disk = file_info.AllocationSize as u64;
         }
     }
 
@@ -237,17 +216,15 @@ fn get_file_info(path: &Path) -> Option<FileInfo> {
         Err(_) => return result,
     };
 
-    let handle = file.as_raw_handle();
-
     unsafe {
         let mut file_info: FILE_ID_INFO = core::mem::zeroed();
         let file_info_ptr: *mut FILE_ID_INFO = &mut file_info;
 
         let success = GetFileInformationByHandleEx(
-            handle,
+            file.as_raw_handle() as HANDLE,
             FileIdInfo,
-            file_info_ptr as LPVOID,
-            std::mem::size_of::<FILE_ID_INFO>() as DWORD,
+            file_info_ptr as _,
+            std::mem::size_of::<FILE_ID_INFO>() as u32,
         );
 
         if success != 0 {
@@ -282,9 +259,9 @@ fn read_block_size(s: Option<&str>) -> u64 {
 }
 
 fn choose_size(matches: &ArgMatches, stat: &Stat) -> u64 {
-    if matches.contains_id(options::INODES) {
+    if matches.get_flag(options::INODES) {
         stat.inodes
-    } else if matches.contains_id(options::APPARENT_SIZE) || matches.contains_id(options::BYTES) {
+    } else if matches.get_flag(options::APPARENT_SIZE) || matches.get_flag(options::BYTES) {
         stat.size
     } else {
         // The st_blocks field indicates the number of blocks allocated to the file, 512-byte units.
@@ -398,7 +375,7 @@ fn convert_size_human(size: u64, multiplier: u64, _block_size: u64) -> String {
     if size == 0 {
         return "0".to_string();
     }
-    format!("{}B", size)
+    format!("{size}B")
 }
 
 fn convert_size_b(size: u64, _multiplier: u64, _block_size: u64) -> String {
@@ -457,7 +434,7 @@ Try '{} --help' for more information.",
 'birth' and 'creation' arguments are not supported on this platform.",
                 s.quote()
             ),
-            Self::InvalidGlob(s) => write!(f, "Invalid exclude syntax: {}", s),
+            Self::InvalidGlob(s) => write!(f, "Invalid exclude syntax: {s}"),
         }
     }
 }
@@ -492,7 +469,7 @@ fn build_exclude_patterns(matches: &ArgMatches) -> UResult<Vec<Pattern>> {
     let exclude_from_iterator = matches
         .get_many::<String>(options::EXCLUDE_FROM)
         .unwrap_or_default()
-        .flat_map(|f| file_as_vec(&f));
+        .flat_map(file_as_vec);
 
     let excludes_iterator = matches
         .get_many::<String>(options::EXCLUDE)
@@ -501,7 +478,7 @@ fn build_exclude_patterns(matches: &ArgMatches) -> UResult<Vec<Pattern>> {
 
     let mut exclude_patterns = Vec::new();
     for f in excludes_iterator.chain(exclude_from_iterator) {
-        if matches.contains_id(options::VERBOSE) {
+        if matches.get_flag(options::VERBOSE) {
             println!("adding {:?} to the exclude list ", &f);
         }
         match parse_glob::from_str(&f) {
@@ -519,7 +496,7 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
 
     let matches = uu_app().try_get_matches_from(args)?;
 
-    let summarize = matches.contains_id(options::SUMMARIZE);
+    let summarize = matches.get_flag(options::SUMMARIZE);
 
     let max_depth = parse_depth(
         matches
@@ -529,14 +506,14 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
     )?;
 
     let options = Options {
-        all: matches.contains_id(options::ALL),
+        all: matches.get_flag(options::ALL),
         max_depth,
-        total: matches.contains_id(options::TOTAL),
-        separate_dirs: matches.contains_id(options::SEPARATE_DIRS),
-        one_file_system: matches.contains_id(options::ONE_FILE_SYSTEM),
-        dereference: matches.contains_id(options::DEREFERENCE),
-        inodes: matches.contains_id(options::INODES),
-        verbose: matches.contains_id(options::VERBOSE),
+        total: matches.get_flag(options::TOTAL),
+        separate_dirs: matches.get_flag(options::SEPARATE_DIRS),
+        one_file_system: matches.get_flag(options::ONE_FILE_SYSTEM),
+        dereference: matches.get_flag(options::DEREFERENCE),
+        inodes: matches.get_flag(options::INODES),
+        verbose: matches.get_flag(options::VERBOSE),
     };
 
     let files = match matches.get_one::<String>(options::FILE) {
@@ -549,7 +526,7 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
     };
 
     if options.inodes
-        && (matches.contains_id(options::APPARENT_SIZE) || matches.contains_id(options::BYTES))
+        && (matches.get_flag(options::APPARENT_SIZE) || matches.get_flag(options::BYTES))
     {
         show_warning!("options --apparent-size and -b are ineffective with --inodes");
     }
@@ -565,19 +542,19 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
             .unwrap_or_else(|e| crash!(1, "{}", format_error_message(&e, s, options::THRESHOLD)))
     });
 
-    let multiplier: u64 = if matches.contains_id(options::SI) {
+    let multiplier: u64 = if matches.get_flag(options::SI) {
         1000
     } else {
         1024
     };
     let convert_size_fn = {
-        if matches.contains_id(options::HUMAN_READABLE) || matches.contains_id(options::SI) {
+        if matches.get_flag(options::HUMAN_READABLE) || matches.get_flag(options::SI) {
             convert_size_human
-        } else if matches.contains_id(options::BYTES) {
+        } else if matches.get_flag(options::BYTES) {
             convert_size_b
-        } else if matches.contains_id(options::BLOCK_SIZE_1K) {
+        } else if matches.get_flag(options::BLOCK_SIZE_1K) {
             convert_size_k
-        } else if matches.contains_id(options::BLOCK_SIZE_1M) {
+        } else if matches.get_flag(options::BLOCK_SIZE_1M) {
             convert_size_m
         } else {
             convert_size_other
@@ -594,7 +571,7 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
     let time_format_str =
         parse_time_style(matches.get_one::<String>("time-style").map(|s| s.as_str()))?;
 
-    let line_separator = if matches.contains_id(options::NULL) {
+    let line_separator = if matches.get_flag(options::NULL) {
         "\0"
     } else {
         "\n"
@@ -659,12 +636,12 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
                         let time_str = tm.format(time_format_str).to_string();
                         print!("{}\t{}\t", convert_size(size), time_str);
                         print_verbatim(stat.path).unwrap();
-                        print!("{}", line_separator);
+                        print!("{line_separator}");
                     }
                 } else if !summarize || index == len - 1 {
                     print!("{}\t", convert_size(size));
                     print_verbatim(stat.path).unwrap();
-                    print!("{}", line_separator);
+                    print!("{line_separator}");
                 }
                 if options.total && index == (len - 1) {
                     // The last element will be the total size of the the path under
@@ -683,7 +660,7 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
 
     if options.total {
         print!("{}\ttotal", convert_size(grand_total));
-        print!("{}", line_separator);
+        print!("{line_separator}");
     }
 
     Ok(())
@@ -710,23 +687,26 @@ fn parse_depth(max_depth_str: Option<&str>, summarize: bool) -> UResult<Option<u
     }
 }
 
-pub fn uu_app<'a>() -> Command<'a> {
+pub fn uu_app() -> Command {
     Command::new(uucore::util_name())
         .version(crate_version!())
         .about(ABOUT)
-        .after_help(LONG_HELP)
+        .after_help(AFTER_HELP)
         .override_usage(format_usage(USAGE))
         .infer_long_args(true)
+        .disable_help_flag(true)
         .arg(
             Arg::new(options::HELP)
                 .long(options::HELP)
                 .help("Print help information.")
+                .action(ArgAction::Help)
         )
         .arg(
             Arg::new(options::ALL)
                 .short('a')
                 .long(options::ALL)
-                .help("write counts for all files, not just directories"),
+                .help("write counts for all files, not just directories")
+                .action(ArgAction::SetTrue)
         )
         .arg(
             Arg::new(options::APPARENT_SIZE)
@@ -736,6 +716,7 @@ pub fn uu_app<'a>() -> Command<'a> {
                     although the apparent size is usually smaller, it may be larger due to holes \
                     in ('sparse') files, internal fragmentation, indirect blocks, and the like"
                 )
+                .action(ArgAction::SetTrue)
         )
         .arg(
             Arg::new(options::BLOCK_SIZE)
@@ -752,12 +733,14 @@ pub fn uu_app<'a>() -> Command<'a> {
                 .short('b')
                 .long("bytes")
                 .help("equivalent to '--apparent-size --block-size=1'")
+                .action(ArgAction::SetTrue)
         )
         .arg(
             Arg::new(options::TOTAL)
                 .long("total")
                 .short('c')
                 .help("produce a grand total")
+                .action(ArgAction::SetTrue)
         )
         .arg(
             Arg::new(options::MAX_DEPTH)
@@ -775,6 +758,7 @@ pub fn uu_app<'a>() -> Command<'a> {
                 .long("human-readable")
                 .short('h')
                 .help("print sizes in human readable format (e.g., 1K 234M 2G)")
+                .action(ArgAction::SetTrue)
         )
         .arg(
             Arg::new(options::INODES)
@@ -782,70 +766,81 @@ pub fn uu_app<'a>() -> Command<'a> {
                 .help(
                     "list inode usage information instead of block usage like --block-size=1K"
                 )
+                .action(ArgAction::SetTrue)
         )
         .arg(
             Arg::new(options::BLOCK_SIZE_1K)
                 .short('k')
                 .help("like --block-size=1K")
+                .action(ArgAction::SetTrue)
         )
         .arg(
             Arg::new(options::COUNT_LINKS)
                 .short('l')
                 .long("count-links")
                 .help("count sizes many times if hard linked")
+                .action(ArgAction::SetTrue)
         )
         .arg(
             Arg::new(options::DEREFERENCE)
                 .short('L')
                 .long(options::DEREFERENCE)
                 .help("dereference all symbolic links")
+                .action(ArgAction::SetTrue)
         )
         // .arg(
         //     Arg::new("no-dereference")
         //         .short('P')
         //         .long("no-dereference")
         //         .help("don't follow any symbolic links (this is the default)")
+        //         .action(ArgAction::SetTrue),
         // )
         .arg(
             Arg::new(options::BLOCK_SIZE_1M)
                 .short('m')
                 .help("like --block-size=1M")
+                .action(ArgAction::SetTrue)
         )
         .arg(
             Arg::new(options::NULL)
                 .short('0')
                 .long("null")
                 .help("end each output line with 0 byte rather than newline")
+                .action(ArgAction::SetTrue)
         )
         .arg(
             Arg::new(options::SEPARATE_DIRS)
                 .short('S')
                 .long("separate-dirs")
                 .help("do not include size of subdirectories")
+                .action(ArgAction::SetTrue)
         )
         .arg(
             Arg::new(options::SUMMARIZE)
                 .short('s')
                 .long("summarize")
                 .help("display only a total for each argument")
+                .action(ArgAction::SetTrue)
         )
         .arg(
             Arg::new(options::SI)
                 .long(options::SI)
                 .help("like -h, but use powers of 1000 not 1024")
+                .action(ArgAction::SetTrue)
         )
         .arg(
             Arg::new(options::ONE_FILE_SYSTEM)
                 .short('x')
                 .long(options::ONE_FILE_SYSTEM)
                 .help("skip directories on different file systems")
+                .action(ArgAction::SetTrue)
         )
         .arg(
             Arg::new(options::THRESHOLD)
                 .short('t')
                 .long(options::THRESHOLD)
                 .value_name("SIZE")
-                .number_of_values(1)
+                .num_args(1)
                 .allow_hyphen_values(true)
                 .help("exclude entries smaller than SIZE if positive, \
                           or entries greater than SIZE if negative")
@@ -855,13 +850,14 @@ pub fn uu_app<'a>() -> Command<'a> {
                 .short('v')
                 .long("verbose")
                 .help("verbose mode (option not present in GNU/Coreutils)")
+                .action(ArgAction::SetTrue)
         )
         .arg(
             Arg::new(options::EXCLUDE)
                 .long(options::EXCLUDE)
                 .value_name("PATTERN")
                 .help("exclude files that match PATTERN")
-                .multiple_occurrences(true)
+                .action(ArgAction::Append)
         )
         .arg(
             Arg::new(options::EXCLUDE_FROM)
@@ -870,15 +866,14 @@ pub fn uu_app<'a>() -> Command<'a> {
                 .value_name("FILE")
                 .value_hint(clap::ValueHint::FilePath)
                 .help("exclude files that match any pattern in FILE")
-                .multiple_occurrences(true)
-
+                .action(ArgAction::Append)
         )
         .arg(
             Arg::new(options::TIME)
                 .long(options::TIME)
                 .value_name("WORD")
                 .require_equals(true)
-                .min_values(0)
+                .num_args(0..)
                 .value_parser(["atime", "access", "use", "ctime", "status", "birth", "creation"])
                 .help(
                     "show time of the last modification of any file in the \
@@ -899,7 +894,7 @@ pub fn uu_app<'a>() -> Command<'a> {
             Arg::new(options::FILE)
                 .hide(true)
                 .value_hint(clap::ValueHint::AnyPath)
-                .multiple_occurrences(true)
+                .action(ArgAction::Append)
         )
 }
 
@@ -913,11 +908,16 @@ impl FromStr for Threshold {
     type Err = ParseSizeError;
 
     fn from_str(s: &str) -> std::result::Result<Self, Self::Err> {
-        let offset = if s.starts_with(&['-', '+'][..]) { 1 } else { 0 };
+        let offset = usize::from(s.starts_with(&['-', '+'][..]));
 
         let size = parse_size(&s[offset..])?;
 
         if s.starts_with('-') {
+            // Threshold of '-0' excludes everything besides 0 sized entries
+            // GNU's du treats '-0' as an invalid argument
+            if size == 0 {
+                return Err(ParseSizeError::ParseFailure(s.to_string()));
+            }
             Ok(Self::Upper(size))
         } else {
             Ok(Self::Lower(size))
