@@ -8,16 +8,12 @@
 
 /* last synced with: env (GNU coreutils) 8.13 */
 
-// spell-checker:ignore (ToDO) chdir execvp progname subcommand subcommands unsets setenv putenv spawnp
+// spell-checker:ignore (ToDO) chdir execvp progname subcommand subcommands unsets setenv putenv spawnp SIGSEGV SIGBUS sigaction
 
-#[macro_use]
-extern crate clap;
-
-#[macro_use]
-extern crate uucore;
-
-use clap::{Arg, Command};
+use clap::{crate_name, crate_version, Arg, ArgAction, Command};
 use ini::Ini;
+#[cfg(unix)]
+use nix::sys::signal::{raise, sigaction, SaFlags, SigAction, SigHandler, SigSet, Signal};
 use std::borrow::Cow;
 use std::env;
 use std::io::{self, Write};
@@ -27,15 +23,11 @@ use std::os::unix::process::ExitStatusExt;
 use std::process;
 use uucore::display::Quotable;
 use uucore::error::{UClapError, UResult, USimpleError, UUsageError};
-use uucore::format_usage;
-#[cfg(unix)]
-use uucore::signals::signal_name_by_value;
+use uucore::{format_usage, help_about, help_section, help_usage, show_warning};
 
-const ABOUT: &str = "set each NAME to VALUE in the environment and run COMMAND";
-const USAGE: &str = "{} [OPTION]... [-] [NAME=VALUE]... [COMMAND [ARG]...]";
-const AFTER_HELP: &str = "\
-A mere - implies -i. If no COMMAND, print the resulting environment.
-";
+const ABOUT: &str = help_about!("env.md");
+const USAGE: &str = help_usage!("env.md");
+const AFTER_HELP: &str = help_section!("after help", "env.md");
 
 struct Options<'a> {
     ignore_env: bool,
@@ -126,57 +118,69 @@ fn build_command<'a, 'b>(args: &'a mut Vec<&'b str>) -> (Cow<'b, str>, &'a [&'b
     (progname, &args[..])
 }
 
-pub fn uu_app<'a>() -> Command<'a> {
+pub fn uu_app() -> Command {
     Command::new(crate_name!())
         .version(crate_version!())
         .about(ABOUT)
         .override_usage(format_usage(USAGE))
         .after_help(AFTER_HELP)
-        .allow_external_subcommands(true)
         .infer_long_args(true)
-        .arg(Arg::new("ignore-environment")
-            .short('i')
-            .long("ignore-environment")
-            .help("start with an empty environment"))
-        .arg(Arg::new("chdir")
-            .short('C') // GNU env compatibility
-            .long("chdir")
-            .takes_value(true)
-            .number_of_values(1)
-            .value_name("DIR")
-            .value_hint(clap::ValueHint::DirPath)
-            .help("change working directory to DIR"))
-        .arg(Arg::new("null")
-            .short('0')
-            .long("null")
-            .help("end each output line with a 0 byte rather than a newline (only valid when \
-                    printing the environment)"))
-        .arg(Arg::new("file")
-            .short('f')
-            .long("file")
-            .takes_value(true)
-            .number_of_values(1)
-            .value_name("PATH")
-            .value_hint(clap::ValueHint::FilePath)
-            .multiple_occurrences(true)
-            .help("read and set variables from a \".env\"-style configuration file (prior to any \
-                    unset and/or set)"))
-        .arg(Arg::new("unset")
-            .short('u')
-            .long("unset")
-            .takes_value(true)
-            .number_of_values(1)
-            .value_name("NAME")
-            .multiple_occurrences(true)
-            .help("remove variable from the environment"))
+        .trailing_var_arg(true)
+        .arg(
+            Arg::new("ignore-environment")
+                .short('i')
+                .long("ignore-environment")
+                .help("start with an empty environment")
+                .action(ArgAction::SetTrue),
+        )
+        .arg(
+            Arg::new("chdir")
+                .short('C') // GNU env compatibility
+                .long("chdir")
+                .number_of_values(1)
+                .value_name("DIR")
+                .value_hint(clap::ValueHint::DirPath)
+                .help("change working directory to DIR"),
+        )
+        .arg(
+            Arg::new("null")
+                .short('0')
+                .long("null")
+                .help(
+                    "end each output line with a 0 byte rather than a newline (only \
+                valid when printing the environment)",
+                )
+                .action(ArgAction::SetTrue),
+        )
+        .arg(
+            Arg::new("file")
+                .short('f')
+                .long("file")
+                .value_name("PATH")
+                .value_hint(clap::ValueHint::FilePath)
+                .action(ArgAction::Append)
+                .help(
+                    "read and set variables from a \".env\"-style configuration file \
+                (prior to any unset and/or set)",
+                ),
+        )
+        .arg(
+            Arg::new("unset")
+                .short('u')
+                .long("unset")
+                .value_name("NAME")
+                .action(ArgAction::Append)
+                .help("remove variable from the environment"),
+        )
+        .arg(Arg::new("vars").action(ArgAction::Append))
 }
 
 fn run_env(args: impl uucore::Args) -> UResult<()> {
     let app = uu_app();
     let matches = app.try_get_matches_from(args).with_exit_code(125)?;
 
-    let ignore_env = matches.contains_id("ignore-environment");
-    let null = matches.contains_id("null");
+    let ignore_env = matches.get_flag("ignore-environment");
+    let null = matches.get_flag("null");
     let running_directory = matches.get_one::<String>("chdir").map(|s| s.as_str());
     let files = match matches.get_many::<String>("file") {
         Some(v) => v.map(|s| s.as_str()).collect(),
@@ -204,38 +208,30 @@ fn run_env(args: impl uucore::Args) -> UResult<()> {
             Err(error) => {
                 return Err(USimpleError::new(
                     125,
-                    format!("cannot change directory to \"{}\": {}", d, error),
+                    format!("cannot change directory to \"{d}\": {error}"),
                 ));
             }
         };
     }
 
-    // we handle the name, value pairs and the program to be executed by treating them as external
-    // subcommands in clap
-    if let Some((external, matches)) = matches.subcommand() {
-        let mut begin_prog_opts = false;
-
-        if external == "-" {
-            // "-" implies -i and stop parsing opts
-            opts.ignore_env = true;
-        } else {
-            begin_prog_opts = parse_name_value_opt(&mut opts, external)?;
+    let mut begin_prog_opts = false;
+    if let Some(mut iter) = matches.get_many::<String>("vars") {
+        // read NAME=VALUE arguments (and up to a single program argument)
+        while !begin_prog_opts {
+            if let Some(opt) = iter.next() {
+                if opt == "-" {
+                    opts.ignore_env = true;
+                } else {
+                    begin_prog_opts = parse_name_value_opt(&mut opts, opt)?;
+                }
+            } else {
+                break;
+            }
         }
 
-        if let Some(mut iter) = matches.get_many::<String>("") {
-            // read NAME=VALUE arguments (and up to a single program argument)
-            while !begin_prog_opts {
-                if let Some(opt) = iter.next() {
-                    begin_prog_opts = parse_name_value_opt(&mut opts, opt)?;
-                } else {
-                    break;
-                }
-            }
-
-            // read any leftover program arguments
-            for opt in iter {
-                parse_program_opt(&mut opts, opt)?;
-            }
+        // read any leftover program arguments
+        for opt in iter {
+            parse_program_opt(&mut opts, opt)?;
         }
     }
 
@@ -303,7 +299,10 @@ fn run_env(args: impl uucore::Args) -> UResult<()> {
         env::set_var(name, val);
     }
 
-    if !opts.program.is_empty() {
+    if opts.program.is_empty() {
+        // no program provided, so just dump all env vars to stdout
+        print_env(opts.null);
+    } else {
         // we need to execute a command
         let (prog, args) = build_command(&mut opts.program);
 
@@ -323,24 +322,23 @@ fn run_env(args: impl uucore::Args) -> UResult<()> {
                     // See std::os::unix::process::ExitStatusExt for more information. This prints out
                     // the interrupted process and the signal it received.
                     let signal_code = exit.signal().unwrap();
-                    eprintln!(
-                        "\"{}\" terminated by signal {}",
-                        {
-                            let mut command = uucore::util_name().to_owned();
-                            command.push(' ');
-                            command.push_str(&opts.program.join(" "));
-                            command
-                        },
-                        signal_name_by_value(signal_code as usize).map_or_else(
-                            || String::from("UNKNOWN"),
-                            |signal| {
-                                let mut full_signal_name = String::from("SIG");
-                                full_signal_name.push_str(signal);
-                                full_signal_name
-                            }
+                    let signal = Signal::try_from(signal_code).unwrap();
+
+                    // We have to disable any handler that's installed by default.
+                    // This ensures that we exit on this signal.
+                    // For example, `SIGSEGV` and `SIGBUS` have default handlers installed in Rust.
+                    // We ignore the errors because there is not much we can do if that fails anyway.
+                    // SAFETY: The function is unsafe because installing functions is unsafe, but we are
+                    // just defaulting to default behavior and not installing a function. Hence, the call
+                    // is safe.
+                    let _ = unsafe {
+                        sigaction(
+                            signal,
+                            &SigAction::new(SigHandler::SigDfl, SaFlags::empty(), SigSet::all()),
                         )
-                    );
-                    return Err((128 + signal_code).into());
+                    };
+
+                    let _ = raise(signal);
                 }
                 #[cfg(not(unix))]
                 return Err(exit.code().unwrap().into());
@@ -349,9 +347,6 @@ fn run_env(args: impl uucore::Args) -> UResult<()> {
             Err(_) => return Err(126.into()),
             Ok(_) => (),
         }
-    } else {
-        // no program provided, so just dump all env vars to stdout
-        print_env(opts.null);
     }
 
     Ok(())
