@@ -7,20 +7,18 @@
 
 // spell-checker:ignore (ToDOs) ncount routput
 
-use clap::{crate_version, Arg, Command};
+use clap::{crate_version, Arg, ArgAction, Command};
 use std::fs::File;
 use std::io::{stdin, BufRead, BufReader, Read};
 use std::path::Path;
 use uucore::display::Quotable;
 use uucore::error::{FromIo, UResult, USimpleError};
-use uucore::format_usage;
+use uucore::{format_usage, help_about, help_usage};
 
 const TAB_WIDTH: usize = 8;
 
-static NAME: &str = "fold";
-static USAGE: &str = "{} [OPTION]... [FILE]...";
-static ABOUT: &str = "Writes each file (or standard input if no files are given)
- to standard output whilst breaking long lines";
+const USAGE: &str = help_usage!("fold.md");
+const ABOUT: &str = help_about!("fold.md");
 
 mod options {
     pub const BYTES: &str = "bytes";
@@ -34,11 +32,11 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
     let args = args.collect_lossy();
 
     let (args, obs_width) = handle_obsolete(&args[..]);
-    let matches = uu_app().get_matches_from(args);
+    let matches = uu_app().try_get_matches_from(args)?;
 
-    let bytes = matches.contains_id(options::BYTES);
-    let spaces = matches.contains_id(options::SPACES);
-    let poss_width = match matches.value_of(options::WIDTH) {
+    let bytes = matches.get_flag(options::BYTES);
+    let spaces = matches.get_flag(options::SPACES);
+    let poss_width = match matches.get_one::<String>(options::WIDTH) {
         Some(v) => Some(v.to_owned()),
         None => obs_width,
     };
@@ -61,9 +59,8 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
     fold(&files, bytes, spaces, width)
 }
 
-pub fn uu_app<'a>() -> Command<'a> {
+pub fn uu_app() -> Command {
     Command::new(uucore::util_name())
-        .name(NAME)
         .version(crate_version!())
         .override_usage(format_usage(USAGE))
         .about(ABOUT)
@@ -76,14 +73,14 @@ pub fn uu_app<'a>() -> Command<'a> {
                     "count using bytes rather than columns (meaning control characters \
                      such as newline are not treated specially)",
                 )
-                .takes_value(false),
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::SPACES)
                 .long(options::SPACES)
                 .short('s')
                 .help("break lines at word boundaries rather than a hard cut-off")
-                .takes_value(false),
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::WIDTH)
@@ -91,13 +88,12 @@ pub fn uu_app<'a>() -> Command<'a> {
                 .short('w')
                 .help("set WIDTH as the maximum line width rather than 80")
                 .value_name("WIDTH")
-                .allow_hyphen_values(true)
-                .takes_value(true),
+                .allow_hyphen_values(true),
         )
         .arg(
             Arg::new(options::FILE)
                 .hide(true)
-                .multiple_occurrences(true)
+                .action(ArgAction::Append)
                 .value_hint(clap::ValueHint::FilePath),
         )
 }
@@ -191,9 +187,9 @@ fn fold_file_bytewise<T: Read>(mut file: BufReader<T>, spaces: bool, width: usiz
             let at_eol = i >= len;
 
             if at_eol {
-                print!("{}", slice);
+                print!("{slice}");
             } else {
-                println!("{}", slice);
+                println!("{slice}");
             }
         }
 
@@ -275,9 +271,7 @@ fn fold_file<T: Read>(mut file: BufReader<T>, spaces: bool, width: usize) -> URe
                     last_space = if spaces { Some(output.len()) } else { None };
                 }
                 '\x08' => {
-                    if col_count > 0 {
-                        col_count -= 1;
-                    }
+                    col_count = col_count.saturating_sub(1);
                 }
                 _ if spaces && ch.is_whitespace() => {
                     last_space = Some(output.len());
@@ -290,7 +284,7 @@ fn fold_file<T: Read>(mut file: BufReader<T>, spaces: bool, width: usize) -> URe
         }
 
         if !output.is_empty() {
-            print!("{}", output);
+            print!("{output}");
             output.truncate(0);
         }
 
