@@ -9,24 +9,12 @@
 
 // spell-checker:ignore (ToDO) algo, algoname, regexes, nread, nonames
 
-#[macro_use]
-extern crate clap;
-
-#[macro_use]
-extern crate uucore;
-
-mod digest;
-
-use self::digest::Digest;
-use self::digest::DigestWriter;
-
+use clap::builder::ValueParser;
+use clap::crate_version;
+use clap::ArgAction;
 use clap::{Arg, ArgMatches, Command};
 use hex::encode;
-use md5::Md5;
 use regex::Regex;
-use sha1::Sha1;
-use sha2::{Sha224, Sha256, Sha384, Sha512};
-use sha3::{Sha3_224, Sha3_256, Sha3_384, Sha3_512, Shake128, Shake256};
 use std::cmp::Ordering;
 use std::error::Error;
 use std::ffi::{OsStr, OsString};
@@ -35,10 +23,17 @@ use std::io::{self, stdin, BufRead, BufReader, Read};
 use std::iter;
 use std::num::ParseIntError;
 use std::path::Path;
-use uucore::display::Quotable;
 use uucore::error::{FromIo, UError, UResult};
+use uucore::sum::{
+    Blake2b, Blake3, Digest, DigestWriter, Md5, Sha1, Sha224, Sha256, Sha384, Sha3_224, Sha3_256,
+    Sha3_384, Sha3_512, Sha512, Shake128, Shake256,
+};
+use uucore::{crash, display::Quotable, show_warning};
+use uucore::{format_usage, help_about, help_usage};
 
 const NAME: &str = "hashsum";
+const ABOUT: &str = help_about!("hashsum.md");
+const USAGE: &str = help_usage!("hashsum.md");
 
 struct Options {
     algoname: &'static str,
@@ -52,6 +47,7 @@ struct Options {
     strict: bool,
     warn: bool,
     output_bits: usize,
+    zero: bool,
 }
 
 #[allow(clippy::cognitive_complexity)]
@@ -69,44 +65,33 @@ fn detect_algo(
         "sha256sum" => ("SHA256", Box::new(Sha256::new()) as Box<dyn Digest>, 256),
         "sha384sum" => ("SHA384", Box::new(Sha384::new()) as Box<dyn Digest>, 384),
         "sha512sum" => ("SHA512", Box::new(Sha512::new()) as Box<dyn Digest>, 512),
-        "b2sum" => (
-            "BLAKE2",
-            Box::new(blake2b_simd::State::new()) as Box<dyn Digest>,
-            512,
-        ),
-        "b3sum" => (
-            "BLAKE3",
-            Box::new(blake3::Hasher::new()) as Box<dyn Digest>,
-            256,
-        ),
-        "sha3sum" => match matches.value_of("bits") {
-            Some(bits_str) => match (bits_str).parse::<usize>() {
-                Ok(224) => (
-                    "SHA3-224",
-                    Box::new(Sha3_224::new()) as Box<dyn Digest>,
-                    224,
-                ),
-                Ok(256) => (
-                    "SHA3-256",
-                    Box::new(Sha3_256::new()) as Box<dyn Digest>,
-                    256,
-                ),
-                Ok(384) => (
-                    "SHA3-384",
-                    Box::new(Sha3_384::new()) as Box<dyn Digest>,
-                    384,
-                ),
-                Ok(512) => (
-                    "SHA3-512",
-                    Box::new(Sha3_512::new()) as Box<dyn Digest>,
-                    512,
-                ),
-                Ok(_) => crash!(
-                    1,
-                    "Invalid output size for SHA3 (expected 224, 256, 384, or 512)"
-                ),
-                Err(err) => crash!(1, "{}", err),
-            },
+        "b2sum" => ("BLAKE2", Box::new(Blake2b::new()) as Box<dyn Digest>, 512),
+        "b3sum" => ("BLAKE3", Box::new(Blake3::new()) as Box<dyn Digest>, 256),
+        "sha3sum" => match matches.get_one::<usize>("bits") {
+            Some(224) => (
+                "SHA3-224",
+                Box::new(Sha3_224::new()) as Box<dyn Digest>,
+                224,
+            ),
+            Some(256) => (
+                "SHA3-256",
+                Box::new(Sha3_256::new()) as Box<dyn Digest>,
+                256,
+            ),
+            Some(384) => (
+                "SHA3-384",
+                Box::new(Sha3_384::new()) as Box<dyn Digest>,
+                384,
+            ),
+            Some(512) => (
+                "SHA3-512",
+                Box::new(Sha3_512::new()) as Box<dyn Digest>,
+                512,
+            ),
+            Some(_) => crash!(
+                1,
+                "Invalid output size for SHA3 (expected 224, 256, 384, or 512)"
+            ),
             None => crash!(1, "--bits required for SHA3"),
         },
         "sha3-224sum" => (
@@ -129,26 +114,20 @@ fn detect_algo(
             Box::new(Sha3_512::new()) as Box<dyn Digest>,
             512,
         ),
-        "shake128sum" => match matches.value_of("bits") {
-            Some(bits_str) => match (bits_str).parse::<usize>() {
-                Ok(bits) => (
-                    "SHAKE128",
-                    Box::new(Shake128::new()) as Box<dyn Digest>,
-                    bits,
-                ),
-                Err(err) => crash!(1, "{}", err),
-            },
+        "shake128sum" => match matches.get_one::<usize>("bits") {
+            Some(bits) => (
+                "SHAKE128",
+                Box::new(Shake128::new()) as Box<dyn Digest>,
+                *bits,
+            ),
             None => crash!(1, "--bits required for SHAKE-128"),
         },
-        "shake256sum" => match matches.value_of("bits") {
-            Some(bits_str) => match (bits_str).parse::<usize>() {
-                Ok(bits) => (
-                    "SHAKE256",
-                    Box::new(Shake256::new()) as Box<dyn Digest>,
-                    bits,
-                ),
-                Err(err) => crash!(1, "{}", err),
-            },
+        "shake256sum" => match matches.get_one::<usize>("bits") {
+            Some(bits) => (
+                "SHAKE256",
+                Box::new(Shake256::new()) as Box<dyn Digest>,
+                *bits,
+            ),
             None => crash!(1, "--bits required for SHAKE-256"),
         },
         _ => {
@@ -161,89 +140,80 @@ fn detect_algo(
                     alg = Some(val);
                     output_bits = bits;
                 };
-                if matches.contains_id("md5") {
+                if matches.get_flag("md5") {
                     set_or_crash("MD5", Box::new(Md5::new()), 128);
                 }
-                if matches.contains_id("sha1") {
+                if matches.get_flag("sha1") {
                     set_or_crash("SHA1", Box::new(Sha1::new()), 160);
                 }
-                if matches.contains_id("sha224") {
+                if matches.get_flag("sha224") {
                     set_or_crash("SHA224", Box::new(Sha224::new()), 224);
                 }
-                if matches.contains_id("sha256") {
+                if matches.get_flag("sha256") {
                     set_or_crash("SHA256", Box::new(Sha256::new()), 256);
                 }
-                if matches.contains_id("sha384") {
+                if matches.get_flag("sha384") {
                     set_or_crash("SHA384", Box::new(Sha384::new()), 384);
                 }
-                if matches.contains_id("sha512") {
+                if matches.get_flag("sha512") {
                     set_or_crash("SHA512", Box::new(Sha512::new()), 512);
                 }
-                if matches.contains_id("b2sum") {
-                    set_or_crash("BLAKE2", Box::new(blake2b_simd::State::new()), 512);
+                if matches.get_flag("b2sum") {
+                    set_or_crash("BLAKE2", Box::new(Blake2b::new()), 512);
                 }
-                if matches.contains_id("b3sum") {
-                    set_or_crash("BLAKE3", Box::new(blake3::Hasher::new()), 256);
+                if matches.get_flag("b3sum") {
+                    set_or_crash("BLAKE3", Box::new(Blake3::new()), 256);
                 }
-                if matches.contains_id("sha3") {
-                    match matches.value_of("bits") {
-                        Some(bits_str) => match (bits_str).parse::<usize>() {
-                            Ok(224) => set_or_crash(
-                                "SHA3-224",
-                                Box::new(Sha3_224::new()) as Box<dyn Digest>,
-                                224,
-                            ),
-                            Ok(256) => set_or_crash(
-                                "SHA3-256",
-                                Box::new(Sha3_256::new()) as Box<dyn Digest>,
-                                256,
-                            ),
-                            Ok(384) => set_or_crash(
-                                "SHA3-384",
-                                Box::new(Sha3_384::new()) as Box<dyn Digest>,
-                                384,
-                            ),
-                            Ok(512) => set_or_crash(
-                                "SHA3-512",
-                                Box::new(Sha3_512::new()) as Box<dyn Digest>,
-                                512,
-                            ),
-                            Ok(_) => crash!(
-                                1,
-                                "Invalid output size for SHA3 (expected 224, 256, 384, or 512)"
-                            ),
-                            Err(err) => crash!(1, "{}", err),
-                        },
+                if matches.get_flag("sha3") {
+                    match matches.get_one::<usize>("bits") {
+                        Some(224) => set_or_crash(
+                            "SHA3-224",
+                            Box::new(Sha3_224::new()) as Box<dyn Digest>,
+                            224,
+                        ),
+                        Some(256) => set_or_crash(
+                            "SHA3-256",
+                            Box::new(Sha3_256::new()) as Box<dyn Digest>,
+                            256,
+                        ),
+                        Some(384) => set_or_crash(
+                            "SHA3-384",
+                            Box::new(Sha3_384::new()) as Box<dyn Digest>,
+                            384,
+                        ),
+                        Some(512) => set_or_crash(
+                            "SHA3-512",
+                            Box::new(Sha3_512::new()) as Box<dyn Digest>,
+                            512,
+                        ),
+                        Some(_) => crash!(
+                            1,
+                            "Invalid output size for SHA3 (expected 224, 256, 384, or 512)"
+                        ),
                         None => crash!(1, "--bits required for SHA3"),
                     }
                 }
-                if matches.contains_id("sha3-224") {
+                if matches.get_flag("sha3-224") {
                     set_or_crash("SHA3-224", Box::new(Sha3_224::new()), 224);
                 }
-                if matches.contains_id("sha3-256") {
+                if matches.get_flag("sha3-256") {
                     set_or_crash("SHA3-256", Box::new(Sha3_256::new()), 256);
                 }
-                if matches.contains_id("sha3-384") {
+                if matches.get_flag("sha3-384") {
                     set_or_crash("SHA3-384", Box::new(Sha3_384::new()), 384);
                 }
-                if matches.contains_id("sha3-512") {
+                if matches.get_flag("sha3-512") {
                     set_or_crash("SHA3-512", Box::new(Sha3_512::new()), 512);
                 }
-                if matches.contains_id("shake128") {
-                    match matches.value_of("bits") {
-                        Some(bits_str) => match (bits_str).parse::<usize>() {
-                            Ok(bits) => set_or_crash("SHAKE128", Box::new(Shake128::new()), bits),
-                            Err(err) => crash!(1, "{}", err),
-                        },
+                if matches.get_flag("shake128") {
+                    match matches.get_one::<usize>("bits") {
+                        Some(bits) => set_or_crash("SHAKE128", Box::new(Shake128::new()), *bits),
                         None => crash!(1, "--bits required for SHAKE-128"),
                     }
                 }
-                if matches.contains_id("shake256") {
-                    match matches.value_of("bits") {
-                        Some(bits_str) => match (bits_str).parse::<usize>() {
-                            Ok(bits) => set_or_crash("SHAKE256", Box::new(Shake256::new()), bits),
-                            Err(err) => crash!(1, "{}", err),
-                        },
+                if matches.get_flag("shake256") {
+                    match matches.get_one::<usize>("bits") {
+                        Some(bits) => set_or_crash("SHAKE256", Box::new(Shake256::new()), *bits),
                         None => crash!(1, "--bits required for SHAKE-256"),
                     }
                 }
@@ -259,10 +229,6 @@ fn parse_bit_num(arg: &str) -> Result<usize, ParseIntError> {
     arg.parse()
 }
 
-fn is_valid_bit_num(arg: &str) -> Result<(), String> {
-    parse_bit_num(arg).map(|_| ()).map_err(|e| format!("{}", e))
-}
-
 #[uucore::main]
 pub fn uumain(mut args: impl uucore::Args) -> UResult<()> {
     // if there is no program name for some reason, default to "hashsum"
@@ -283,28 +249,28 @@ pub fn uumain(mut args: impl uucore::Args) -> UResult<()> {
     //        causes "error: " to be printed twice (once from crash!() and once from clap).  With
     //        the current setup, the name of the utility is not printed, but I think this is at
     //        least somewhat better from a user's perspective.
-    let matches = command.get_matches_from(args);
+    let matches = command.try_get_matches_from(args)?;
 
     let (name, algo, bits) = detect_algo(&binary_name, &matches);
 
-    let binary = if matches.contains_id("binary") {
+    let binary = if matches.get_flag("binary") {
         true
-    } else if matches.contains_id("text") {
+    } else if matches.get_flag("text") {
         false
     } else {
         binary_flag_default
     };
-    let check = matches.contains_id("check");
-    let tag = matches.contains_id("tag");
-    let nonames = if binary_name == "b3sum" {
-        matches.contains_id("no-names")
-    } else {
-        false
-    };
-    let status = matches.contains_id("status");
-    let quiet = matches.contains_id("quiet") || status;
-    let strict = matches.contains_id("strict");
-    let warn = matches.contains_id("warn") && !status;
+    let check = matches.get_flag("check");
+    let tag = matches.get_flag("tag");
+    let nonames = *matches
+        .try_get_one("no-names")
+        .unwrap_or(None)
+        .unwrap_or(&false);
+    let status = matches.get_flag("status");
+    let quiet = matches.get_flag("quiet") || status;
+    let strict = matches.get_flag("strict");
+    let warn = matches.get_flag("warn") && !status;
+    let zero = matches.get_flag("zero");
 
     let opts = Options {
         algoname: name,
@@ -318,6 +284,7 @@ pub fn uumain(mut args: impl uucore::Args) -> UResult<()> {
         quiet,
         strict,
         warn,
+        zero,
     };
 
     match matches.get_many::<OsString>("FILE") {
@@ -326,7 +293,7 @@ pub fn uumain(mut args: impl uucore::Args) -> UResult<()> {
     }
 }
 
-pub fn uu_app_common<'a>() -> Command<'a> {
+pub fn uu_app_common() -> Command {
     #[cfg(windows)]
     const BINARY_HELP: &str = "read in binary mode (default)";
     #[cfg(not(windows))]
@@ -337,66 +304,82 @@ pub fn uu_app_common<'a>() -> Command<'a> {
     const TEXT_HELP: &str = "read in text mode (default)";
     Command::new(uucore::util_name())
         .version(crate_version!())
-        .about("Compute and check message digests.")
+        .about(ABOUT)
+        .override_usage(format_usage(USAGE))
         .infer_long_args(true)
         .arg(
             Arg::new("binary")
                 .short('b')
                 .long("binary")
-                .help(BINARY_HELP),
+                .help(BINARY_HELP)
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new("check")
                 .short('c')
                 .long("check")
-                .help("read hashsums from the FILEs and check them"),
+                .help("read hashsums from the FILEs and check them")
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new("tag")
                 .long("tag")
-                .help("create a BSD-style checksum"),
+                .help("create a BSD-style checksum")
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new("text")
                 .short('t')
                 .long("text")
                 .help(TEXT_HELP)
-                .conflicts_with("binary"),
+                .conflicts_with("binary")
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new("quiet")
                 .short('q')
                 .long("quiet")
-                .help("don't print OK for each successfully verified file"),
+                .help("don't print OK for each successfully verified file")
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new("status")
                 .short('s')
                 .long("status")
-                .help("don't output anything, status code shows success"),
+                .help("don't output anything, status code shows success")
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new("strict")
                 .long("strict")
-                .help("exit non-zero for improperly formatted checksum lines"),
+                .help("exit non-zero for improperly formatted checksum lines")
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new("warn")
                 .short('w')
                 .long("warn")
-                .help("warn about improperly formatted checksum lines"),
+                .help("warn about improperly formatted checksum lines")
+                .action(ArgAction::SetTrue),
+        )
+        .arg(
+            Arg::new("zero")
+                .short('z')
+                .long("zero")
+                .help("end each output line with NUL, not newline")
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new("FILE")
                 .index(1)
-                .multiple_occurrences(true)
+                .action(ArgAction::Append)
                 .value_name("FILE")
                 .value_hint(clap::ValueHint::FilePath)
-                .allow_invalid_utf8(true),
+                .value_parser(ValueParser::os_string()),
         )
 }
 
-pub fn uu_app_b3sum<'a>() -> Command<'a> {
+pub fn uu_app_b3sum() -> Command {
     uu_app_b3sum_opts(uu_app_common())
 }
 
@@ -404,11 +387,12 @@ fn uu_app_b3sum_opts(command: Command) -> Command {
     command.arg(
         Arg::new("no-names")
             .long("no-names")
-            .help("Omits filenames in the output (option not present in GNU/Coreutils)"),
+            .help("Omits filenames in the output (option not present in GNU/Coreutils)")
+            .action(ArgAction::SetTrue),
     )
 }
 
-pub fn uu_app_bits<'a>() -> Command<'a> {
+pub fn uu_app_bits() -> Command {
     uu_app_opt_bits(uu_app_common())
 }
 
@@ -418,14 +402,13 @@ fn uu_app_opt_bits(command: Command) -> Command {
         Arg::new("bits")
             .long("bits")
             .help("set the size of the output (only for SHAKE)")
-            .takes_value(true)
             .value_name("BITS")
             // XXX: should we actually use validators?  they're not particularly efficient
-            .validator(is_valid_bit_num),
+            .value_parser(parse_bit_num),
     )
 }
 
-pub fn uu_app_custom<'a>() -> Command<'a> {
+pub fn uu_app_custom() -> Command {
     let mut command = uu_app_b3sum_opts(uu_app_opt_bits(uu_app_common()));
     let algorithms = &[
         ("md5", "work with MD5"),
@@ -452,14 +435,19 @@ pub fn uu_app_custom<'a>() -> Command<'a> {
     ];
 
     for (name, desc) in algorithms {
-        command = command.arg(Arg::new(*name).long(name).help(*desc));
+        command = command.arg(
+            Arg::new(*name)
+                .long(name)
+                .help(*desc)
+                .action(ArgAction::SetTrue),
+        );
     }
     command
 }
 
 // hashsum is handled differently in build.rs, therefore this is not the same
 // as in other utilities.
-fn uu_app<'a>(binary_name: &str) -> Command<'a> {
+fn uu_app(binary_name: &str) -> Command {
     match binary_name {
         // These all support the same options.
         "md5sum" | "sha1sum" | "sha224sum" | "sha256sum" | "sha384sum" | "sha512sum" => {
@@ -505,7 +493,8 @@ where
     I: Iterator<Item = &'a OsStr>,
 {
     let mut bad_format = 0;
-    let mut failed = 0;
+    let mut failed_cksum = 0;
+    let mut failed_open_file = 0;
     let binary_marker = if options.binary { "*" } else { " " };
     for filename in files {
         let filename = Path::new(filename);
@@ -530,13 +519,12 @@ where
             // where `n` is the number of bytes.
             let bytes = options.digest.output_bits() / 4;
             let modifier = if bytes > 0 {
-                format!("{{{}}}", bytes)
+                format!("{{{bytes}}}")
             } else {
                 "+".to_string()
             };
             let gnu_re = Regex::new(&format!(
-                r"^(?P<digest>[a-fA-F0-9]{}) (?P<binary>[ \*])(?P<fileName>.*)",
-                modifier,
+                r"^(?P<digest>[a-fA-F0-9]{modifier}) (?P<binary>[ \*])(?P<fileName>.*)",
             ))
             .map_err(|_| HashsumError::InvalidRegex)?;
             let bsd_re = Regex::new(&format!(
@@ -581,8 +569,19 @@ where
                         }
                     },
                 };
-                let f = File::open(ck_filename)
-                    .map_err_context(|| "failed to open file".to_string())?;
+                let f = match File::open(ck_filename) {
+                    Err(_) => {
+                        failed_open_file += 1;
+                        println!(
+                            "{}: {}: No such file or directory",
+                            uucore::util_name(),
+                            ck_filename
+                        );
+                        println!("{ck_filename}: FAILED open or read");
+                        continue;
+                    }
+                    Ok(file) => file,
+                };
                 let mut ckf = BufReader::new(Box::new(f) as Box<dyn Read>);
                 let real_sum = digest_reader(
                     &mut options.digest,
@@ -603,13 +602,13 @@ where
                 // easier (and more important) on Unix than on Windows.
                 if sum == real_sum {
                     if !options.quiet {
-                        println!("{}: OK", ck_filename);
+                        println!("{ck_filename}: OK");
                     }
                 } else {
                     if !options.status {
-                        println!("{}: FAILED", ck_filename);
+                        println!("{ck_filename}: FAILED");
                     }
-                    failed += 1;
+                    failed_cksum += 1;
                 }
             }
         } else {
@@ -623,7 +622,9 @@ where
             if options.tag {
                 println!("{} ({}) = {}", options.algoname, filename.display(), sum);
             } else if options.nonames {
-                println!("{}", sum);
+                println!("{sum}");
+            } else if options.zero {
+                print!("{} {}{}\0", sum, binary_marker, filename.display());
             } else {
                 println!("{} {}{}", sum, binary_marker, filename.display());
             }
@@ -633,10 +634,17 @@ where
         match bad_format.cmp(&1) {
             Ordering::Equal => show_warning!("{} line is improperly formatted", bad_format),
             Ordering::Greater => show_warning!("{} lines are improperly formatted", bad_format),
-            _ => {}
+            Ordering::Less => {}
         };
-        if failed > 0 {
-            show_warning!("{} computed checksum did NOT match", failed);
+        if failed_cksum > 0 {
+            show_warning!("{} computed checksum did NOT match", failed_cksum);
+        }
+        match failed_open_file.cmp(&1) {
+            Ordering::Equal => show_warning!("{} listed file could not be read", failed_open_file),
+            Ordering::Greater => {
+                show_warning!("{} listed files could not be read", failed_open_file);
+            }
+            Ordering::Less => {}
         }
     }
 
@@ -673,7 +681,7 @@ fn digest_reader<T: Read>(
         // Assume it's SHAKE.  result_str() doesn't work with shake (as of 8/30/2016)
         let mut bytes = Vec::new();
         bytes.resize((output_bits + 7) / 8, 0);
-        digest.result(&mut bytes);
+        digest.hash_finalize(&mut bytes);
         Ok(encode(bytes))
     }
 }
