@@ -9,20 +9,17 @@
 
 mod mode;
 
-#[macro_use]
-extern crate uucore;
-
-use clap::{crate_version, Arg, ArgMatches, Command};
+use clap::{crate_version, Arg, ArgAction, ArgMatches, Command};
 use file_diff::diff;
 use filetime::{set_file_times, FileTime};
 use uucore::backup_control::{self, BackupMode};
 use uucore::display::Quotable;
 use uucore::entries::{grp2gid, usr2uid};
 use uucore::error::{FromIo, UError, UIoError, UResult, UUsageError};
-use uucore::format_usage;
 use uucore::fs::dir_strip_dot_for_creation;
 use uucore::mode::get_umask;
 use uucore::perms::{wrap_chown, Verbosity, VerbosityLevel};
+use uucore::{format_usage, help_about, help_usage, show, show_error, show_if_err, uio_error};
 
 use libc::{getegid, geteuid};
 use std::error::Error;
@@ -30,7 +27,9 @@ use std::fmt::{Debug, Display};
 use std::fs;
 use std::fs::File;
 use std::os::unix::fs::MetadataExt;
-use std::path::{Path, PathBuf};
+#[cfg(unix)]
+use std::os::unix::prelude::OsStrExt;
+use std::path::{Path, PathBuf, MAIN_SEPARATOR};
 use std::process;
 
 const DEFAULT_MODE: u32 = 0o755;
@@ -42,8 +41,8 @@ pub struct Behavior {
     specified_mode: Option<u32>,
     backup_mode: BackupMode,
     suffix: String,
-    owner: String,
-    group: String,
+    owner_id: Option<u32>,
+    group_id: Option<u32>,
     verbose: bool,
     preserve_timestamps: bool,
     compare: bool,
@@ -59,14 +58,15 @@ enum InstallError {
     DirNeedsArg(),
     CreateDirFailed(PathBuf, std::io::Error),
     ChmodFailed(PathBuf),
+    ChownFailed(PathBuf, String),
     InvalidTarget(PathBuf),
     TargetDirIsntDir(PathBuf),
     BackupFailed(PathBuf, PathBuf, std::io::Error),
     InstallFailed(PathBuf, PathBuf, std::io::Error),
     StripProgramFailed(String),
     MetadataFailed(std::io::Error),
-    NoSuchUser(String),
-    NoSuchGroup(String),
+    InvalidUser(String),
+    InvalidGroup(String),
     OmittingDirectory(PathBuf),
 }
 
@@ -88,7 +88,7 @@ impl Error for InstallError {}
 impl Display for InstallError {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         match self {
-            Self::Unimplemented(opt) => write!(f, "Unimplemented feature: {}", opt),
+            Self::Unimplemented(opt) => write!(f, "Unimplemented feature: {opt}"),
             Self::DirNeedsArg() => {
                 write!(
                     f,
@@ -100,6 +100,7 @@ impl Display for InstallError {
                 Display::fmt(&uio_error!(e, "failed to create {}", dir.quote()), f)
             }
             Self::ChmodFailed(file) => write!(f, "failed to chmod {}", file.quote()),
+            Self::ChownFailed(file, msg) => write!(f, "failed to chown {}: {}", file.quote(), msg),
             Self::InvalidTarget(target) => write!(
                 f,
                 "invalid target {}: No such file or directory",
@@ -116,10 +117,10 @@ impl Display for InstallError {
                 &uio_error!(e, "cannot install {} to {}", from.quote(), to.quote()),
                 f,
             ),
-            Self::StripProgramFailed(msg) => write!(f, "strip program failed: {}", msg),
+            Self::StripProgramFailed(msg) => write!(f, "strip program failed: {msg}"),
             Self::MetadataFailed(e) => Display::fmt(&uio_error!(e, ""), f),
-            Self::NoSuchUser(user) => write!(f, "no such user: {}", user.maybe_quote()),
-            Self::NoSuchGroup(group) => write!(f, "no such group: {}", group.maybe_quote()),
+            Self::InvalidUser(user) => write!(f, "invalid user: {}", user.quote()),
+            Self::InvalidGroup(group) => write!(f, "invalid group: {}", group.quote()),
             Self::OmittingDirectory(dir) => write!(f, "omitting directory {}", dir.quote()),
         }
     }
@@ -143,9 +144,8 @@ impl Behavior {
     }
 }
 
-static ABOUT: &str = "Copy SOURCE to DEST or multiple SOURCE(s) to the existing
- DIRECTORY, while setting permission modes and owner/group";
-const USAGE: &str = "{} [OPTION]... [FILE]...";
+const ABOUT: &str = help_about!("install.md");
+const USAGE: &str = help_usage!("install.md");
 
 static OPT_COMPARE: &str = "compare";
 static OPT_DIRECTORY: &str = "directory";
@@ -188,57 +188,63 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
     }
 }
 
-pub fn uu_app<'a>() -> Command<'a> {
+pub fn uu_app() -> Command {
     Command::new(uucore::util_name())
         .version(crate_version!())
         .about(ABOUT)
         .override_usage(format_usage(USAGE))
         .infer_long_args(true)
-        .arg(
-            backup_control::arguments::backup()
-        )
-        .arg(
-            backup_control::arguments::backup_no_args()
-        )
+        .arg(backup_control::arguments::backup())
+        .arg(backup_control::arguments::backup_no_args())
         .arg(
             Arg::new(OPT_IGNORED)
-            .short('c')
-            .help("ignored")
+                .short('c')
+                .help("ignored")
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(OPT_COMPARE)
-            .short('C')
-            .long(OPT_COMPARE)
-            .help("compare each pair of source and destination files, and in some cases, do not modify the destination at all")
+                .short('C')
+                .long(OPT_COMPARE)
+                .help(
+                    "compare each pair of source and destination files, and in some cases, \
+                    do not modify the destination at all",
+                )
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(OPT_DIRECTORY)
                 .short('d')
                 .long(OPT_DIRECTORY)
-                .help("treat all arguments as directory names. create all components of the specified directories")
+                .help(
+                    "treat all arguments as directory names. create all components of \
+                        the specified directories",
+                )
+                .action(ArgAction::SetTrue),
         )
-
         .arg(
             // TODO implement flag
             Arg::new(OPT_CREATE_LEADING)
                 .short('D')
-                .help("create all leading components of DEST except the last, then copy SOURCE to DEST")
+                .help(
+                    "create all leading components of DEST except the last, then copy \
+                        SOURCE to DEST",
+                )
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(OPT_GROUP)
                 .short('g')
                 .long(OPT_GROUP)
                 .help("set group ownership, instead of process's current group")
-                .value_name("GROUP")
-                .takes_value(true)
+                .value_name("GROUP"),
         )
         .arg(
             Arg::new(OPT_MODE)
                 .short('m')
                 .long(OPT_MODE)
                 .help("set permission mode (as in chmod), instead of rwxr-xr-x")
-                .value_name("MODE")
-                .takes_value(true)
+                .value_name("MODE"),
         )
         .arg(
             Arg::new(OPT_OWNER)
@@ -246,31 +252,33 @@ pub fn uu_app<'a>() -> Command<'a> {
                 .long(OPT_OWNER)
                 .help("set ownership (super-user only)")
                 .value_name("OWNER")
-                .takes_value(true)
-                .value_hint(clap::ValueHint::Username)
+                .value_hint(clap::ValueHint::Username),
         )
         .arg(
             Arg::new(OPT_PRESERVE_TIMESTAMPS)
                 .short('p')
                 .long(OPT_PRESERVE_TIMESTAMPS)
-                .help("apply access/modification times of SOURCE files to corresponding destination files")
+                .help(
+                    "apply access/modification times of SOURCE files to \
+                    corresponding destination files",
+                )
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(OPT_STRIP)
                 .short('s')
                 .long(OPT_STRIP)
                 .help("strip symbol tables (no action Windows)")
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(OPT_STRIP_PROGRAM)
                 .long(OPT_STRIP_PROGRAM)
                 .help("program used to strip binaries (no action Windows)")
                 .value_name("PROGRAM")
-                .value_hint(clap::ValueHint::CommandName)
-        )
-        .arg(
-            backup_control::arguments::suffix()
+                .value_hint(clap::ValueHint::CommandName),
         )
+        .arg(backup_control::arguments::suffix())
         .arg(
             // TODO implement flag
             Arg::new(OPT_TARGET_DIRECTORY)
@@ -278,7 +286,7 @@ pub fn uu_app<'a>() -> Command<'a> {
                 .long(OPT_TARGET_DIRECTORY)
                 .help("move all SOURCE arguments into DIRECTORY")
                 .value_name("DIRECTORY")
-                .value_hint(clap::ValueHint::DirPath)
+                .value_hint(clap::ValueHint::DirPath),
         )
         .arg(
             // TODO implement flag
@@ -286,13 +294,14 @@ pub fn uu_app<'a>() -> Command<'a> {
                 .short('T')
                 .long(OPT_NO_TARGET_DIRECTORY)
                 .help("(unimplemented) treat DEST as a normal file")
-
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(OPT_VERBOSE)
-            .short('v')
-            .long(OPT_VERBOSE)
-            .help("explain what is being done")
+                .short('v')
+                .long(OPT_VERBOSE)
+                .help("explain what is being done")
+                .action(ArgAction::SetTrue),
         )
         .arg(
             // TODO implement flag
@@ -300,6 +309,7 @@ pub fn uu_app<'a>() -> Command<'a> {
                 .short('P')
                 .long(OPT_PRESERVE_CONTEXT)
                 .help("(unimplemented) preserve security context")
+                .action(ArgAction::SetTrue),
         )
         .arg(
             // TODO implement flag
@@ -308,13 +318,13 @@ pub fn uu_app<'a>() -> Command<'a> {
                 .long(OPT_CONTEXT)
                 .help("(unimplemented) set security context of files and directories")
                 .value_name("CONTEXT")
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(ARG_FILES)
-                .multiple_occurrences(true)
-                .takes_value(true)
-                .min_values(1)
-                .value_hint(clap::ValueHint::AnyPath)
+                .action(ArgAction::Append)
+                .num_args(1..)
+                .value_hint(clap::ValueHint::AnyPath),
         )
 }
 
@@ -328,11 +338,11 @@ pub fn uu_app<'a>() -> Command<'a> {
 ///
 ///
 fn check_unimplemented(matches: &ArgMatches) -> UResult<()> {
-    if matches.contains_id(OPT_NO_TARGET_DIRECTORY) {
+    if matches.get_flag(OPT_NO_TARGET_DIRECTORY) {
         Err(InstallError::Unimplemented(String::from("--no-target-directory, -T")).into())
-    } else if matches.contains_id(OPT_PRESERVE_CONTEXT) {
+    } else if matches.get_flag(OPT_PRESERVE_CONTEXT) {
         Err(InstallError::Unimplemented(String::from("--preserve-context, -P")).into())
-    } else if matches.contains_id(OPT_CONTEXT) {
+    } else if matches.get_flag(OPT_CONTEXT) {
         Err(InstallError::Unimplemented(String::from("--context, -Z")).into())
     } else {
         Ok(())
@@ -348,7 +358,7 @@ fn check_unimplemented(matches: &ArgMatches) -> UResult<()> {
 /// In event of failure, returns an integer intended as a program return code.
 ///
 fn behavior(matches: &ArgMatches) -> UResult<Behavior> {
-    let main_function = if matches.contains_id(OPT_DIRECTORY) {
+    let main_function = if matches.get_flag(OPT_DIRECTORY) {
         MainFunction::Directory
     } else {
         MainFunction::Standard
@@ -371,9 +381,9 @@ fn behavior(matches: &ArgMatches) -> UResult<Behavior> {
         .get_one::<String>(OPT_TARGET_DIRECTORY)
         .map(|d| d.to_owned());
 
-    let preserve_timestamps = matches.contains_id(OPT_PRESERVE_TIMESTAMPS);
-    let compare = matches.contains_id(OPT_COMPARE);
-    let strip = matches.contains_id(OPT_STRIP);
+    let preserve_timestamps = matches.get_flag(OPT_PRESERVE_TIMESTAMPS);
+    let compare = matches.get_flag(OPT_COMPARE);
+    let strip = matches.get_flag(OPT_STRIP);
     if preserve_timestamps && compare {
         show_error!("Options --compare and --preserve-timestamps are mutually exclusive");
         return Err(1.into());
@@ -382,22 +392,45 @@ fn behavior(matches: &ArgMatches) -> UResult<Behavior> {
         show_error!("Options --compare and --strip are mutually exclusive");
         return Err(1.into());
     }
+
+    let owner = matches
+        .get_one::<String>(OPT_OWNER)
+        .map(|s| s.as_str())
+        .unwrap_or("")
+        .to_string();
+
+    let owner_id = if owner.is_empty() {
+        None
+    } else {
+        match usr2uid(&owner) {
+            Ok(u) => Some(u),
+            Err(_) => return Err(InstallError::InvalidUser(owner.clone()).into()),
+        }
+    };
+
+    let group = matches
+        .get_one::<String>(OPT_GROUP)
+        .map(|s| s.as_str())
+        .unwrap_or("")
+        .to_string();
+
+    let group_id = if group.is_empty() {
+        None
+    } else {
+        match grp2gid(&group) {
+            Ok(g) => Some(g),
+            Err(_) => return Err(InstallError::InvalidGroup(group.clone()).into()),
+        }
+    };
+
     Ok(Behavior {
         main_function,
         specified_mode,
         backup_mode,
         suffix: backup_control::determine_backup_suffix(matches),
-        owner: matches
-            .get_one::<String>(OPT_OWNER)
-            .map(|s| s.as_str())
-            .unwrap_or("")
-            .to_string(),
-        group: matches
-            .get_one::<String>(OPT_GROUP)
-            .map(|s| s.as_str())
-            .unwrap_or("")
-            .to_string(),
-        verbose: matches.contains_id(OPT_VERBOSE),
+        owner_id,
+        group_id,
+        verbose: matches.get_flag(OPT_VERBOSE),
         preserve_timestamps,
         compare,
         strip,
@@ -407,7 +440,7 @@ fn behavior(matches: &ArgMatches) -> UResult<Behavior> {
                 .map(|s| s.as_str())
                 .unwrap_or(DEFAULT_STRIP_PROGRAM),
         ),
-        create_leading: matches.contains_id(OPT_CREATE_LEADING),
+        create_leading: matches.get_flag(OPT_CREATE_LEADING),
         target_dir,
     })
 }
@@ -457,6 +490,8 @@ fn directory(paths: &[String], b: &Behavior) -> UResult<()> {
                 uucore::error::set_exit_code(1);
                 continue;
             }
+
+            show_if_err!(chown_optional_user_group(path, b));
         }
         // If the exit code was set, or show! has been called at least once
         // (which sets the exit code as well), function execution will end after
@@ -473,40 +508,73 @@ fn is_new_file_path(path: &Path) -> bool {
             || path.parent().unwrap().as_os_str().is_empty()) // In case of a simple file
 }
 
+/// Test if the path is an existing directory or ends with a trailing separator.
+///
+/// Returns true, if one of the conditions above is met; else false.
+///
+#[cfg(unix)]
+fn is_potential_directory_path(path: &Path) -> bool {
+    let separator = MAIN_SEPARATOR as u8;
+    path.as_os_str().as_bytes().last() == Some(&separator) || path.is_dir()
+}
+
+#[cfg(not(unix))]
+fn is_potential_directory_path(path: &Path) -> bool {
+    let path_str = path.to_string_lossy();
+    path_str.ends_with(MAIN_SEPARATOR) || path_str.ends_with('/') || path.is_dir()
+}
+
 /// Perform an install, given a list of paths and behavior.
 ///
 /// Returns a Result type with the Err variant containing the error message.
 ///
 fn standard(mut paths: Vec<String>, b: &Behavior) -> UResult<()> {
+    // first check that paths contains at least one element
+    if paths.is_empty() {
+        return Err(UUsageError::new(1, "missing file operand"));
+    }
+
+    // get the target from either "-t foo" param or from the last given paths argument
     let target: PathBuf = if let Some(path) = &b.target_dir {
         path.into()
     } else {
-        paths
-            .pop()
-            .ok_or_else(|| UUsageError::new(1, "missing file operand"))?
-            .into()
+        let last_path: PathBuf = paths.pop().unwrap().into();
+
+        // paths has to contain more elements
+        if paths.is_empty() {
+            return Err(UUsageError::new(
+                1,
+                format!(
+                    "missing destination file operand after '{}'",
+                    last_path.to_str().unwrap()
+                ),
+            ));
+        }
+
+        last_path
     };
 
     let sources = &paths.iter().map(PathBuf::from).collect::<Vec<_>>();
 
-    if sources.len() > 1 || (target.exists() && target.is_dir()) {
-        copy_files_into_dir(sources, &target, b)
-    } else {
+    if b.create_leading {
         // if -t is used in combination with -D, create whole target because it does not include filename
-        let to_create: Option<&Path> = if b.target_dir.is_some() && b.create_leading {
+        let to_create: Option<&Path> = if b.target_dir.is_some() {
             Some(target.as_path())
-        } else {
+        // if source and target are filenames used in combination with -D, create target's parent
+        } else if !(sources.len() > 1 || is_potential_directory_path(&target)) {
             target.parent()
+        } else {
+            None
         };
 
         if let Some(to_create) = to_create {
-            if !to_create.exists() && b.create_leading {
+            if !to_create.exists() {
                 if b.verbose {
                     let mut result = PathBuf::new();
                     // When creating directories with -Dv, show directory creations step by step
                     for part in to_create.components() {
                         result.push(part.as_os_str());
-                        if !Path::new(part.as_os_str()).is_dir() {
+                        if !result.is_dir() {
                             // Don't display when the directory already exists
                             println!("install: creating directory {}", result.quote());
                         }
@@ -516,33 +584,18 @@ fn standard(mut paths: Vec<String>, b: &Behavior) -> UResult<()> {
                 if let Err(e) = fs::create_dir_all(to_create) {
                     return Err(InstallError::CreateDirFailed(to_create.to_path_buf(), e).into());
                 }
-
-                // Silent the warning as we want to the error message
-                #[allow(clippy::question_mark)]
-                if mode::chmod(to_create, b.mode()).is_err() {
-                    return Err(InstallError::ChmodFailed(to_create.to_path_buf()).into());
-                }
             }
         }
+    }
 
-        let source = sources.first().ok_or_else(|| {
-            UUsageError::new(
-                1,
-                format!(
-                    "missing destination file operand after '{}'",
-                    target.to_str().unwrap()
-                ),
-            )
-        })?;
+    if sources.len() > 1 || is_potential_directory_path(&target) {
+        copy_files_into_dir(sources, &target, b)
+    } else {
+        let source = sources.first().unwrap();
 
-        // If the -D flag was passed (target does not include filename),
-        // we need to add the source name to the target_dir
-        // because `copy` expects `to` to be a file, not a directory
-        let target = if target.is_dir() && b.create_leading {
-            target.join(source)
-        } else {
-            target // already includes dest filename
-        };
+        if source.is_dir() {
+            return Err(InstallError::OmittingDirectory(source.to_path_buf()).into());
+        }
 
         if target.is_file() || is_new_file_path(&target) {
             copy(source, &target, b)
@@ -593,6 +646,42 @@ fn copy_files_into_dir(files: &[PathBuf], target_dir: &Path, b: &Behavior) -> UR
     Ok(())
 }
 
+/// Handle incomplete user/group parings for chown.
+///
+/// Returns a Result type with the Err variant containing the error message.
+///
+/// # Parameters
+///
+/// _path_ must exist.
+///
+/// # Errors
+///
+/// If the owner or group are invalid or copy system call fails, we print a verbose error and
+/// return an empty error value.
+///
+fn chown_optional_user_group(path: &Path, b: &Behavior) -> UResult<()> {
+    if b.owner_id.is_some() || b.group_id.is_some() {
+        let meta = match fs::metadata(path) {
+            Ok(meta) => meta,
+            Err(e) => return Err(InstallError::MetadataFailed(e).into()),
+        };
+
+        // GNU coreutils doesn't print chown operations during install with verbose flag.
+        let verbosity = Verbosity {
+            groups_only: b.owner_id.is_none(),
+            level: VerbosityLevel::Normal,
+        };
+
+        match wrap_chown(path, &meta, b.owner_id, b.group_id, false, verbosity) {
+            Ok(msg) if b.verbose && !msg.is_empty() => println!("chown: {msg}"),
+            Ok(_) => {}
+            Err(e) => return Err(InstallError::ChownFailed(path.to_path_buf(), e).into()),
+        }
+    }
+
+    Ok(())
+}
+
 /// Copy one file to a new location, changing metadata.
 ///
 /// Returns a Result type with the Err variant containing the error message.
@@ -675,66 +764,7 @@ fn copy(from: &Path, to: &Path, b: &Behavior) -> UResult<()> {
         return Err(InstallError::ChmodFailed(to.to_path_buf()).into());
     }
 
-    if !b.owner.is_empty() {
-        let meta = match fs::metadata(to) {
-            Ok(meta) => meta,
-            Err(e) => return Err(InstallError::MetadataFailed(e).into()),
-        };
-
-        let owner_id = match usr2uid(&b.owner) {
-            Ok(g) => g,
-            _ => return Err(InstallError::NoSuchUser(b.owner.clone()).into()),
-        };
-        let gid = meta.gid();
-        match wrap_chown(
-            to,
-            &meta,
-            Some(owner_id),
-            Some(gid),
-            false,
-            Verbosity {
-                groups_only: false,
-                level: VerbosityLevel::Normal,
-            },
-        ) {
-            Ok(n) => {
-                if !n.is_empty() {
-                    show_error!("{}", n);
-                }
-            }
-            Err(e) => show_error!("{}", e),
-        }
-    }
-
-    if !b.group.is_empty() {
-        let meta = match fs::metadata(to) {
-            Ok(meta) => meta,
-            Err(e) => return Err(InstallError::MetadataFailed(e).into()),
-        };
-
-        let group_id = match grp2gid(&b.group) {
-            Ok(g) => g,
-            _ => return Err(InstallError::NoSuchGroup(b.group.clone()).into()),
-        };
-        match wrap_chown(
-            to,
-            &meta,
-            Some(group_id),
-            None,
-            false,
-            Verbosity {
-                groups_only: true,
-                level: VerbosityLevel::Normal,
-            },
-        ) {
-            Ok(n) => {
-                if !n.is_empty() {
-                    show_error!("{}", n);
-                }
-            }
-            Err(e) => show_error!("{}", e),
-        }
-    }
+    chown_optional_user_group(to, b)?;
 
     if b.preserve_timestamps {
         let meta = match fs::metadata(from) {
@@ -814,19 +844,11 @@ fn need_copy(from: &Path, to: &Path, b: &Behavior) -> UResult<bool> {
 
     // TODO: if -P (#1809) and from/to contexts mismatch, return true.
 
-    if !b.owner.is_empty() {
-        let owner_id = match usr2uid(&b.owner) {
-            Ok(id) => id,
-            _ => return Err(InstallError::NoSuchUser(b.owner.clone()).into()),
-        };
+    if let Some(owner_id) = b.owner_id {
         if owner_id != to_meta.uid() {
             return Ok(true);
         }
-    } else if !b.group.is_empty() {
-        let group_id = match grp2gid(&b.group) {
-            Ok(id) => id,
-            _ => return Err(InstallError::NoSuchGroup(b.group.clone()).into()),
-        };
+    } else if let Some(group_id) = b.group_id {
         if group_id != to_meta.gid() {
             return Ok(true);
         }
