@@ -7,10 +7,8 @@
 
 // spell-checker:ignore (ToDO) autoformat FILENUM whitespaces pairable unpairable nocheck
 
-#[macro_use]
-extern crate uucore;
-
-use clap::{crate_version, Arg, Command};
+use clap::builder::ValueParser;
+use clap::{crate_version, Arg, ArgAction, Command};
 use memchr::{memchr3_iter, memchr_iter};
 use std::cmp::Ordering;
 use std::convert::From;
@@ -24,8 +22,10 @@ use std::num::IntErrorKind;
 use std::os::unix::ffi::OsStrExt;
 use uucore::display::Quotable;
 use uucore::error::{set_exit_code, UError, UResult, USimpleError};
+use uucore::{crash, crash_if_err, format_usage, help_about, help_usage};
 
-static NAME: &str = "join";
+const ABOUT: &str = help_about!("join.md");
+const USAGE: &str = help_usage!("join.md");
 
 #[derive(Debug)]
 enum JoinError {
@@ -44,7 +44,7 @@ impl Error for JoinError {}
 impl Display for JoinError {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         match self {
-            Self::IOError(e) => write!(f, "io error: {}", e),
+            Self::IOError(e) => write!(f, "io error: {e}"),
             Self::UnorderedInput(e) => f.write_str(e),
         }
     }
@@ -601,13 +601,13 @@ impl<'a> State<'a> {
 
 #[uucore::main]
 pub fn uumain(args: impl uucore::Args) -> UResult<()> {
-    let matches = uu_app().get_matches_from(args);
+    let matches = uu_app().try_get_matches_from(args)?;
 
-    let keys = parse_field_number_option(matches.value_of("j"))?;
-    let key1 = parse_field_number_option(matches.value_of("1"))?;
-    let key2 = parse_field_number_option(matches.value_of("2"))?;
+    let keys = parse_field_number_option(matches.get_one::<String>("j").map(|s| s.as_str()))?;
+    let key1 = parse_field_number_option(matches.get_one::<String>("1").map(|s| s.as_str()))?;
+    let key2 = parse_field_number_option(matches.get_one::<String>("2").map(|s| s.as_str()))?;
 
-    let mut settings: Settings = Default::default();
+    let mut settings = Settings::default();
 
     let v_values = matches.get_many::<String>("v");
     if v_values.is_some() {
@@ -624,7 +624,7 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
         }
     }
 
-    settings.ignore_case = matches.contains_id("i");
+    settings.ignore_case = matches.get_flag("i");
     settings.key1 = get_field_number(keys, key1)?;
     settings.key2 = get_field_number(keys, key2)?;
 
@@ -654,7 +654,7 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
         };
     }
 
-    if let Some(format) = matches.value_of("o") {
+    if let Some(format) = matches.get_one::<String>("o") {
         if format == "auto" {
             settings.autoformat = true;
         } else {
@@ -666,28 +666,28 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
         }
     }
 
-    if let Some(empty) = matches.value_of("e") {
+    if let Some(empty) = matches.get_one::<String>("e") {
         settings.empty = empty.as_bytes().to_vec();
     }
 
-    if matches.contains_id("nocheck-order") {
+    if matches.get_flag("nocheck-order") {
         settings.check_order = CheckOrder::Disabled;
     }
 
-    if matches.contains_id("check-order") {
+    if matches.get_flag("check-order") {
         settings.check_order = CheckOrder::Enabled;
     }
 
-    if matches.contains_id("header") {
+    if matches.get_flag("header") {
         settings.headers = true;
     }
 
-    if matches.contains_id("z") {
+    if matches.get_flag("z") {
         settings.line_ending = LineEnding::Nul;
     }
 
-    let file1 = matches.value_of("file1").unwrap();
-    let file2 = matches.value_of("file2").unwrap();
+    let file1 = matches.get_one::<String>("file1").unwrap();
+    let file2 = matches.get_one::<String>("file2").unwrap();
 
     if file1 == "-" && file2 == "-" {
         return Err(USimpleError::new(1, "both files cannot be standard input"));
@@ -695,25 +695,21 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
 
     match exec(file1, file2, settings) {
         Ok(_) => Ok(()),
-        Err(e) => Err(USimpleError::new(1, format!("{}", e))),
+        Err(e) => Err(USimpleError::new(1, format!("{e}"))),
     }
 }
 
-pub fn uu_app<'a>() -> Command<'a> {
-    Command::new(NAME)
+pub fn uu_app() -> Command {
+    Command::new(uucore::util_name())
         .version(crate_version!())
-        .about(
-            "For each pair of input lines with identical join fields, write a line to
-standard output. The default join field is the first, delimited by blanks.
-
-When FILE1 or FILE2 (not both) is -, read standard input.",
-        )
+        .about(ABOUT)
+        .override_usage(format_usage(USAGE))
         .infer_long_args(true)
         .arg(
             Arg::new("a")
                 .short('a')
-                .multiple_occurrences(true)
-                .number_of_values(1)
+                .action(ArgAction::Append)
+                .num_args(1)
                 .value_parser(["1", "2"])
                 .value_name("FILENUM")
                 .help(
@@ -724,8 +720,8 @@ FILENUM is 1 or 2, corresponding to FILE1 or FILE2",
         .arg(
             Arg::new("v")
                 .short('v')
-                .multiple_occurrences(true)
-                .number_of_values(1)
+                .action(ArgAction::Append)
+                .num_args(1)
                 .value_parser(["1", "2"])
                 .value_name("FILENUM")
                 .help("like -a FILENUM, but suppress joined output lines"),
@@ -733,7 +729,6 @@ FILENUM is 1 or 2, corresponding to FILE1 or FILE2",
         .arg(
             Arg::new("e")
                 .short('e')
-                .takes_value(true)
                 .value_name("EMPTY")
                 .help("replace missing input fields with EMPTY"),
         )
@@ -741,62 +736,70 @@ FILENUM is 1 or 2, corresponding to FILE1 or FILE2",
             Arg::new("i")
                 .short('i')
                 .long("ignore-case")
-                .help("ignore differences in case when comparing fields"),
+                .help("ignore differences in case when comparing fields")
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new("j")
                 .short('j')
-                .takes_value(true)
                 .value_name("FIELD")
                 .help("equivalent to '-1 FIELD -2 FIELD'"),
         )
         .arg(
             Arg::new("o")
                 .short('o')
-                .takes_value(true)
                 .value_name("FORMAT")
                 .help("obey FORMAT while constructing output line"),
         )
         .arg(
             Arg::new("t")
                 .short('t')
-                .takes_value(true)
                 .value_name("CHAR")
-                .allow_invalid_utf8(true)
+                .value_parser(ValueParser::os_string())
                 .help("use CHAR as input and output field separator"),
         )
         .arg(
             Arg::new("1")
                 .short('1')
-                .takes_value(true)
                 .value_name("FIELD")
                 .help("join on this FIELD of file 1"),
         )
         .arg(
             Arg::new("2")
                 .short('2')
-                .takes_value(true)
                 .value_name("FIELD")
                 .help("join on this FIELD of file 2"),
         )
-        .arg(Arg::new("check-order").long("check-order").help(
-            "check that the input is correctly sorted, \
+        .arg(
+            Arg::new("check-order")
+                .long("check-order")
+                .help(
+                    "check that the input is correctly sorted, \
              even if all input lines are pairable",
-        ))
+                )
+                .action(ArgAction::SetTrue),
+        )
         .arg(
             Arg::new("nocheck-order")
                 .long("nocheck-order")
-                .help("do not check that the input is correctly sorted"),
+                .help("do not check that the input is correctly sorted")
+                .action(ArgAction::SetTrue),
         )
-        .arg(Arg::new("header").long("header").help(
-            "treat the first line in each file as field headers, \
+        .arg(
+            Arg::new("header")
+                .long("header")
+                .help(
+                    "treat the first line in each file as field headers, \
              print them without trying to pair them",
-        ))
+                )
+                .action(ArgAction::SetTrue),
+        )
         .arg(
             Arg::new("z")
                 .short('z')
                 .long("zero-terminated")
-                .help("line delimiter is NUL, not newline"),
+                .help("line delimiter is NUL, not newline")
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new("file1")
