@@ -22,22 +22,19 @@ impl UError for MvError {}
 impl Display for MvError {
     fn fmt(&self, f: &mut Formatter) -> Result {
         match self {
-            MvError::NoSuchFile(s) => write!(f, "cannot stat {}: No such file or directory", s),
-            MvError::SameFile(s, t) => write!(f, "{} and {} are the same file", s, t),
-            MvError::SelfSubdirectory(s) => write!(
+            Self::NoSuchFile(s) => write!(f, "cannot stat {s}: No such file or directory"),
+            Self::SameFile(s, t) => write!(f, "{s} and {t} are the same file"),
+            Self::SelfSubdirectory(s) => write!(
                 f,
-                "cannot move '{s}' to a subdirectory of itself, '{s}/{s}'",
-                s = s
+                "cannot move '{s}' to a subdirectory of itself, '{s}/{s}'"
             ),
-            MvError::DirectoryToNonDirectory(t) => {
-                write!(f, "cannot overwrite directory {} with non-directory", t)
+            Self::DirectoryToNonDirectory(t) => {
+                write!(f, "cannot overwrite directory {t} with non-directory")
             }
-            MvError::NonDirectoryToDirectory(s, t) => write!(
-                f,
-                "cannot overwrite non-directory {} with directory {}",
-                t, s
-            ),
-            MvError::NotADirectory(t) => write!(f, "target {} is not a directory", t),
+            Self::NonDirectoryToDirectory(s, t) => {
+                write!(f, "cannot overwrite non-directory {t} with directory {s}")
+            }
+            Self::NotADirectory(t) => write!(f, "target {t} is not a directory"),
         }
     }
 }
