@@ -10,15 +10,13 @@
 
 mod error;
 
-#[macro_use]
-extern crate uucore;
-
 use clap::builder::ValueParser;
-use clap::{crate_version, Arg, ArgMatches, Command, ErrorKind};
+use clap::{crate_version, error::ErrorKind, Arg, ArgAction, ArgMatches, Command};
+use indicatif::{MultiProgress, ProgressBar, ProgressStyle};
 use std::env;
 use std::ffi::OsString;
 use std::fs;
-use std::io::{self, stdin};
+use std::io;
 #[cfg(unix)]
 use std::os::unix;
 #[cfg(windows)]
@@ -27,9 +25,12 @@ use std::path::{Path, PathBuf};
 use uucore::backup_control::{self, BackupMode};
 use uucore::display::Quotable;
 use uucore::error::{FromIo, UError, UResult, USimpleError, UUsageError};
-use uucore::format_usage;
+use uucore::{format_usage, help_about, help_usage, prompt_yes, show};
 
-use fs_extra::dir::{move_dir, CopyOptions as DirCopyOptions};
+use fs_extra::dir::{
+    get_size as dir_get_size, move_dir, move_dir_with_progress, CopyOptions as DirCopyOptions,
+    TransitProcess, TransitProcessResult,
+};
 
 use crate::error::MvError;
 
@@ -42,6 +43,7 @@ pub struct Behavior {
     no_target_dir: bool,
     verbose: bool,
     strip_slashes: bool,
+    progress_bar: bool,
 }
 
 #[derive(Clone, Eq, PartialEq)]
@@ -51,12 +53,8 @@ pub enum OverwriteMode {
     Force,
 }
 
-static ABOUT: &str = "Move SOURCE to DEST, or multiple SOURCE(s) to DIRECTORY.";
-static LONG_HELP: &str = "";
-const USAGE: &str = "\
-    {} [OPTION]... [-T] SOURCE DEST
-    {} [OPTION]... SOURCE... DIRECTORY
-    {} [OPTION]... -t DIRECTORY SOURCE...";
+const ABOUT: &str = help_about!("mv.md");
+const USAGE: &str = help_usage!("mv.md");
 
 static OPT_FORCE: &str = "force";
 static OPT_INTERACTIVE: &str = "interactive";
@@ -66,17 +64,12 @@ static OPT_TARGET_DIRECTORY: &str = "target-directory";
 static OPT_NO_TARGET_DIRECTORY: &str = "no-target-directory";
 static OPT_UPDATE: &str = "update";
 static OPT_VERBOSE: &str = "verbose";
-
+static OPT_PROGRESS: &str = "progress";
 static ARG_FILES: &str = "files";
 
 #[uucore::main]
 pub fn uumain(args: impl uucore::Args) -> UResult<()> {
-    let help = format!(
-        "{}\n{}",
-        LONG_HELP,
-        backup_control::BACKUP_CONTROL_LONG_HELP
-    );
-    let mut app = uu_app().after_help(&*help);
+    let mut app = uu_app().after_help(backup_control::BACKUP_CONTROL_LONG_HELP);
     let matches = app.try_get_matches_from_mut(args)?;
 
     if !matches.contains_id(OPT_TARGET_DIRECTORY)
@@ -89,8 +82,7 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
         app.error(
             ErrorKind::TooFewValues,
             format!(
-                "The argument '<{}>...' requires at least 2 values, but only 1 was provided",
-                ARG_FILES
+                "The argument '<{ARG_FILES}>...' requires at least 2 values, but only 1 was provided"
             ),
         )
         .exit();
@@ -118,19 +110,20 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
         overwrite: overwrite_mode,
         backup: backup_mode,
         suffix: backup_suffix,
-        update: matches.contains_id(OPT_UPDATE),
+        update: matches.get_flag(OPT_UPDATE),
         target_dir: matches
             .get_one::<OsString>(OPT_TARGET_DIRECTORY)
             .map(OsString::from),
-        no_target_dir: matches.contains_id(OPT_NO_TARGET_DIRECTORY),
-        verbose: matches.contains_id(OPT_VERBOSE),
-        strip_slashes: matches.contains_id(OPT_STRIP_TRAILING_SLASHES),
+        no_target_dir: matches.get_flag(OPT_NO_TARGET_DIRECTORY),
+        verbose: matches.get_flag(OPT_VERBOSE),
+        strip_slashes: matches.get_flag(OPT_STRIP_TRAILING_SLASHES),
+        progress_bar: matches.get_flag(OPT_PROGRESS),
     };
 
     exec(&files[..], &behavior)
 }
 
-pub fn uu_app<'a>() -> Command<'a> {
+pub fn uu_app() -> Command {
     Command::new(uucore::util_name())
         .version(crate_version!())
         .about(ABOUT)
@@ -142,24 +135,28 @@ pub fn uu_app<'a>() -> Command<'a> {
             Arg::new(OPT_FORCE)
                 .short('f')
                 .long(OPT_FORCE)
-                .help("do not prompt before overwriting"),
+                .help("do not prompt before overwriting")
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(OPT_INTERACTIVE)
                 .short('i')
                 .long(OPT_INTERACTIVE)
-                .help("prompt before override"),
+                .help("prompt before override")
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(OPT_NO_CLOBBER)
                 .short('n')
                 .long(OPT_NO_CLOBBER)
-                .help("do not overwrite an existing file"),
+                .help("do not overwrite an existing file")
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(OPT_STRIP_TRAILING_SLASHES)
                 .long(OPT_STRIP_TRAILING_SLASHES)
-                .help("remove any trailing slashes from each SOURCE argument"),
+                .help("remove any trailing slashes from each SOURCE argument")
+                .action(ArgAction::SetTrue),
         )
         .arg(backup_control::arguments::suffix())
         .arg(
@@ -167,7 +164,6 @@ pub fn uu_app<'a>() -> Command<'a> {
                 .short('t')
                 .long(OPT_TARGET_DIRECTORY)
                 .help("move all SOURCE arguments into DIRECTORY")
-                .takes_value(true)
                 .value_name("DIRECTORY")
                 .value_hint(clap::ValueHint::DirPath)
                 .conflicts_with(OPT_NO_TARGET_DIRECTORY)
@@ -177,23 +173,40 @@ pub fn uu_app<'a>() -> Command<'a> {
             Arg::new(OPT_NO_TARGET_DIRECTORY)
                 .short('T')
                 .long(OPT_NO_TARGET_DIRECTORY)
-                .help("treat DEST as a normal file"),
+                .help("treat DEST as a normal file")
+                .action(ArgAction::SetTrue),
         )
-        .arg(Arg::new(OPT_UPDATE).short('u').long(OPT_UPDATE).help(
-            "move only when the SOURCE file is newer than the destination file \
+        .arg(
+            Arg::new(OPT_UPDATE)
+                .short('u')
+                .long(OPT_UPDATE)
+                .help(
+                    "move only when the SOURCE file is newer than the destination file \
                        or when the destination file is missing",
-        ))
+                )
+                .action(ArgAction::SetTrue),
+        )
         .arg(
             Arg::new(OPT_VERBOSE)
                 .short('v')
                 .long(OPT_VERBOSE)
-                .help("explain what is being done"),
+                .help("explain what is being done")
+                .action(ArgAction::SetTrue),
+        )
+        .arg(
+            Arg::new(OPT_PROGRESS)
+                .short('g')
+                .long(OPT_PROGRESS)
+                .help(
+                    "Display a progress bar. \n\
+                Note: this feature is not supported by GNU coreutils.",
+                )
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(ARG_FILES)
-                .multiple_occurrences(true)
-                .takes_value(true)
-                .min_values(1)
+                .action(ArgAction::Append)
+                .num_args(1..)
                 .required(true)
                 .value_parser(ValueParser::os_string())
                 .value_hint(clap::ValueHint::AnyPath),
@@ -206,9 +219,9 @@ fn determine_overwrite_mode(matches: &ArgMatches) -> OverwriteMode {
     // overwrite options are supplied, only the last takes effect.
     // To default to no-clobber in that situation seems safer:
     //
-    if matches.contains_id(OPT_NO_CLOBBER) {
+    if matches.get_flag(OPT_NO_CLOBBER) {
         OverwriteMode::NoClobber
-    } else if matches.contains_id(OPT_INTERACTIVE) {
+    } else if matches.get_flag(OPT_INTERACTIVE) {
         OverwriteMode::Interactive
     } else {
         OverwriteMode::Force
@@ -261,12 +274,12 @@ fn exec(files: &[OsString], b: &Behavior) -> UResult<()> {
 
             if target.is_dir() {
                 if b.no_target_dir {
-                    if !source.is_dir() {
-                        Err(MvError::DirectoryToNonDirectory(target.quote().to_string()).into())
-                    } else {
-                        rename(source, target, b).map_err_context(|| {
+                    if source.is_dir() {
+                        rename(source, target, b, None).map_err_context(|| {
                             format!("cannot move {} to {}", source.quote(), target.quote())
                         })
+                    } else {
+                        Err(MvError::DirectoryToNonDirectory(target.quote().to_string()).into())
                     }
                 } else {
                     move_files_into_dir(&[source.clone()], target, b)
@@ -275,8 +288,7 @@ fn exec(files: &[OsString], b: &Behavior) -> UResult<()> {
                 match b.overwrite {
                     OverwriteMode::NoClobber => return Ok(()),
                     OverwriteMode::Interactive => {
-                        println!("{}: overwrite {}? ", uucore::util_name(), target.quote());
-                        if !read_yes() {
+                        if !prompt_yes!("overwrite {}? ", target.quote()) {
                             return Ok(());
                         }
                     }
@@ -288,7 +300,7 @@ fn exec(files: &[OsString], b: &Behavior) -> UResult<()> {
                 )
                 .into())
             } else {
-                rename(source, target, b).map_err(|e| USimpleError::new(1, format!("{}", e)))
+                rename(source, target, b, None).map_err(|e| USimpleError::new(1, format!("{e}")))
             }
         }
         _ => {
@@ -315,7 +327,27 @@ fn move_files_into_dir(files: &[PathBuf], target_dir: &Path, b: &Behavior) -> UR
         .canonicalize()
         .unwrap_or_else(|_| target_dir.to_path_buf());
 
+    let multi_progress = b.progress_bar.then(MultiProgress::new);
+
+    let count_progress = if let Some(ref multi_progress) = multi_progress {
+        if files.len() > 1 {
+            Some(multi_progress.add(
+                ProgressBar::new(files.len().try_into().unwrap()).with_style(
+                    ProgressStyle::with_template("moving {msg} {wide_bar} {pos}/{len}").unwrap(),
+                ),
+            ))
+        } else {
+            None
+        }
+    } else {
+        None
+    };
+
     for sourcepath in files.iter() {
+        if let Some(ref pb) = count_progress {
+            pb.set_message(sourcepath.to_string_lossy().to_string());
+        }
+
         let targetpath = match sourcepath.file_name() {
             Some(name) => target_dir.join(name),
             None => {
@@ -346,27 +378,49 @@ fn move_files_into_dir(files: &[PathBuf], target_dir: &Path, b: &Behavior) -> UR
             }
         }
 
-        show_if_err!(
-            rename(sourcepath, &targetpath, b).map_err_context(|| format!(
-                "cannot move {} to {}",
-                sourcepath.quote(),
-                targetpath.quote()
-            ))
-        );
+        let rename_result = rename(sourcepath, &targetpath, b, multi_progress.as_ref())
+            .map_err_context(|| {
+                format!(
+                    "cannot move {} to {}",
+                    sourcepath.quote(),
+                    targetpath.quote()
+                )
+            });
+
+        if let Err(e) = rename_result {
+            match multi_progress {
+                Some(ref pb) => pb.suspend(|| show!(e)),
+                None => show!(e),
+            };
+        };
+
+        if let Some(ref pb) = count_progress {
+            pb.inc(1);
+        }
     }
     Ok(())
 }
 
-fn rename(from: &Path, to: &Path, b: &Behavior) -> io::Result<()> {
+fn rename(
+    from: &Path,
+    to: &Path,
+    b: &Behavior,
+    multi_progress: Option<&MultiProgress>,
+) -> io::Result<()> {
     let mut backup_path = None;
 
     if to.exists() {
+        if b.update && b.overwrite == OverwriteMode::Interactive {
+            // `mv -i --update old new` when `new` exists doesn't move anything
+            // and exit with 0
+            return Ok(());
+        }
+
         match b.overwrite {
             OverwriteMode::NoClobber => return Ok(()),
             OverwriteMode::Interactive => {
-                println!("{}: overwrite {}? ", uucore::util_name(), to.quote());
-                if !read_yes() {
-                    return Ok(());
+                if !prompt_yes!("overwrite {}?", to.quote()) {
+                    return Err(io::Error::new(io::ErrorKind::Other, ""));
                 }
             }
             OverwriteMode::Force => {}
@@ -374,7 +428,7 @@ fn rename(from: &Path, to: &Path, b: &Behavior) -> io::Result<()> {
 
         backup_path = backup_control::get_backup_path(b.backup, to, &b.suffix);
         if let Some(ref backup_path) = backup_path {
-            rename_with_fallback(to, backup_path)?;
+            rename_with_fallback(to, backup_path, multi_progress)?;
         }
 
         if b.update && fs::metadata(from)?.modified()? <= fs::metadata(to)?.modified()? {
@@ -394,21 +448,36 @@ fn rename(from: &Path, to: &Path, b: &Behavior) -> io::Result<()> {
         }
     }
 
-    rename_with_fallback(from, to)?;
+    rename_with_fallback(from, to, multi_progress)?;
 
     if b.verbose {
-        print!("{} -> {}", from.quote(), to.quote());
-        match backup_path {
-            Some(path) => println!(" (backup: {})", path.quote()),
-            None => println!(),
-        }
+        let message = match backup_path {
+            Some(path) => format!(
+                "{} -> {} (backup: {})",
+                from.quote(),
+                to.quote(),
+                path.quote()
+            ),
+            None => format!("{} -> {}", from.quote(), to.quote()),
+        };
+
+        match multi_progress {
+            Some(pb) => pb.suspend(|| {
+                println!("{message}");
+            }),
+            None => println!("{message}"),
+        };
     }
     Ok(())
 }
 
 /// A wrapper around `fs::rename`, so that if it fails, we try falling back on
 /// copying and removing.
-fn rename_with_fallback(from: &Path, to: &Path) -> io::Result<()> {
+fn rename_with_fallback(
+    from: &Path,
+    to: &Path,
+    multi_progress: Option<&MultiProgress>,
+) -> io::Result<()> {
     if fs::rename(from, to).is_err() {
         // Get metadata without following symlinks
         let metadata = from.symlink_metadata()?;
@@ -430,13 +499,45 @@ fn rename_with_fallback(from: &Path, to: &Path) -> io::Result<()> {
                 copy_inside: true,
                 ..DirCopyOptions::new()
             };
-            if let Err(err) = move_dir(from, to, &options) {
+
+            // Calculate total size of directory
+            // Silently degrades:
+            //    If finding the total size fails for whatever reason,
+            //    the progress bar wont be shown for this file / dir.
+            //    (Move will probably fail due to permission error later?)
+            let total_size = dir_get_size(from).ok();
+
+            let progress_bar =
+                if let (Some(multi_progress), Some(total_size)) = (multi_progress, total_size) {
+                    let bar = ProgressBar::new(total_size).with_style(
+                        ProgressStyle::with_template(
+                            "{msg}: [{elapsed_precise}] {wide_bar} {bytes:>7}/{total_bytes:7}",
+                        )
+                        .unwrap(),
+                    );
+
+                    Some(multi_progress.add(bar))
+                } else {
+                    None
+                };
+
+            let result = if let Some(ref pb) = progress_bar {
+                move_dir_with_progress(from, to, &options, |process_info: TransitProcess| {
+                    pb.set_position(process_info.copied_bytes);
+                    pb.set_message(process_info.file_name);
+                    TransitProcessResult::ContinueOrAbort
+                })
+            } else {
+                move_dir(from, to, &options)
+            };
+
+            if let Err(err) = result {
                 return match err.kind {
                     fs_extra::error::ErrorKind::PermissionDenied => Err(io::Error::new(
                         io::ErrorKind::PermissionDenied,
                         "Permission denied",
                     )),
-                    _ => Err(io::Error::new(io::ErrorKind::Other, format!("{:?}", err))),
+                    _ => Err(io::Error::new(io::ErrorKind::Other, format!("{err:?}"))),
                 };
             }
         } else {
@@ -453,17 +554,17 @@ fn rename_symlink_fallback(from: &Path, to: &Path) -> io::Result<()> {
     let path_symlink_points_to = fs::read_link(from)?;
     #[cfg(unix)]
     {
-        unix::fs::symlink(&path_symlink_points_to, &to).and_then(|_| fs::remove_file(&from))?;
+        unix::fs::symlink(path_symlink_points_to, to).and_then(|_| fs::remove_file(from))?;
     }
     #[cfg(windows)]
     {
         if path_symlink_points_to.exists() {
             if path_symlink_points_to.is_dir() {
-                windows::fs::symlink_dir(&path_symlink_points_to, &to)?;
+                windows::fs::symlink_dir(&path_symlink_points_to, to)?;
             } else {
-                windows::fs::symlink_file(&path_symlink_points_to, &to)?;
+                windows::fs::symlink_file(&path_symlink_points_to, to)?;
             }
-            fs::remove_file(&from)?;
+            fs::remove_file(from)?;
         } else {
             return Err(io::Error::new(
                 io::ErrorKind::NotFound,
@@ -481,17 +582,6 @@ fn rename_symlink_fallback(from: &Path, to: &Path) -> io::Result<()> {
     Ok(())
 }
 
-fn read_yes() -> bool {
-    let mut s = String::new();
-    match stdin().read_line(&mut s) {
-        Ok(_) => match s.chars().next() {
-            Some(x) => x == 'y' || x == 'Y',
-            _ => false,
-        },
-        _ => false,
-    }
-}
-
 fn is_empty_dir(path: &Path) -> bool {
     match fs::read_dir(path) {
         Ok(contents) => contents.peekable().peek().is_none(),
