@@ -5,19 +5,20 @@
 //  * For the full copyright and license information, please view the LICENSE
 //  * file that was distributed with this source code.
 
+use clap::ArgAction;
 use clap::{crate_version, Arg, Command};
 use std::env;
 use std::io;
 use std::path::PathBuf;
-use uucore::format_usage;
+use uucore::{format_usage, help_about, help_usage};
 
 use uucore::display::println_verbatim;
 use uucore::error::{FromIo, UResult};
 
-static ABOUT: &str = "Display the full filename of the current working directory.";
-const USAGE: &str = "{} [OPTION]... FILE...";
-static OPT_LOGICAL: &str = "logical";
-static OPT_PHYSICAL: &str = "physical";
+const ABOUT: &str = help_about!("pwd.md");
+const USAGE: &str = help_usage!("pwd.md");
+const OPT_LOGICAL: &str = "logical";
+const OPT_PHYSICAL: &str = "physical";
 
 fn physical_path() -> io::Result<PathBuf> {
     // std::env::current_dir() is a thin wrapper around libc::getcwd().
@@ -83,36 +84,22 @@ fn logical_path() -> io::Result<PathBuf> {
             {
                 use std::fs::metadata;
                 use std::os::unix::fs::MetadataExt;
-                let path_info = match metadata(path) {
-                    Ok(info) => info,
-                    Err(_) => return false,
-                };
-                let real_info = match metadata(".") {
-                    Ok(info) => info,
-                    Err(_) => return false,
-                };
-                if path_info.dev() != real_info.dev() || path_info.ino() != real_info.ino() {
-                    return false;
+                match (metadata(path), metadata(".")) {
+                    (Ok(info1), Ok(info2)) => {
+                        info1.dev() == info2.dev() && info1.ino() == info2.ino()
+                    }
+                    _ => false,
                 }
             }
 
             #[cfg(not(unix))]
             {
                 use std::fs::canonicalize;
-                let canon_path = match canonicalize(path) {
-                    Ok(path) => path,
-                    Err(_) => return false,
-                };
-                let real_path = match canonicalize(".") {
-                    Ok(path) => path,
-                    Err(_) => return false,
-                };
-                if canon_path != real_path {
-                    return false;
+                match (canonicalize(path), canonicalize(".")) {
+                    (Ok(path1), Ok(path2)) => path1 == path2,
+                    _ => false,
                 }
             }
-
-            true
         }
 
         match env::var_os("PWD").map(PathBuf::from) {
@@ -124,8 +111,11 @@ fn logical_path() -> io::Result<PathBuf> {
 
 #[uucore::main]
 pub fn uumain(args: impl uucore::Args) -> UResult<()> {
-    let matches = uu_app().get_matches_from(args);
-    let cwd = if matches.contains_id(OPT_LOGICAL) {
+    let matches = uu_app().try_get_matches_from(args)?;
+    // if POSIXLY_CORRECT is set, we want to a logical resolution.
+    // This produces a different output when doing mkdir -p a/b && ln -s a/b c && cd c && pwd
+    // We should get c in this case instead of a/b at the end of the path
+    let cwd = if matches.get_flag(OPT_LOGICAL) || env::var("POSIXLY_CORRECT").is_ok() {
         logical_path()
     } else {
         physical_path()
@@ -143,12 +133,12 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
         .map(Into::into)
         .unwrap_or(cwd);
 
-    println_verbatim(&cwd).map_err_context(|| "failed to print current directory".to_owned())?;
+    println_verbatim(cwd).map_err_context(|| "failed to print current directory".to_owned())?;
 
     Ok(())
 }
 
-pub fn uu_app<'a>() -> Command<'a> {
+pub fn uu_app() -> Command {
     Command::new(uucore::util_name())
         .version(crate_version!())
         .about(ABOUT)
@@ -158,13 +148,15 @@ pub fn uu_app<'a>() -> Command<'a> {
             Arg::new(OPT_LOGICAL)
                 .short('L')
                 .long(OPT_LOGICAL)
-                .help("use PWD from environment, even if it contains symlinks"),
+                .help("use PWD from environment, even if it contains symlinks")
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(OPT_PHYSICAL)
                 .short('P')
                 .long(OPT_PHYSICAL)
                 .overrides_with(OPT_LOGICAL)
-                .help("avoid all symlinks"),
+                .help("avoid all symlinks")
+                .action(ArgAction::SetTrue),
         )
 }
