@@ -7,10 +7,8 @@
 
 // spell-checker:ignore (ToDO) ENOTDIR
 
-#[macro_use]
-extern crate uucore;
-
-use clap::{crate_version, Arg, Command};
+use clap::builder::ValueParser;
+use clap::{crate_version, Arg, ArgAction, Command};
 use std::ffi::OsString;
 use std::fs::{read_dir, remove_dir};
 use std::io;
@@ -18,12 +16,10 @@ use std::path::Path;
 use uucore::display::Quotable;
 use uucore::error::{set_exit_code, strip_errno, UResult};
 
-#[cfg(unix)]
-use uucore::fs::is_symlink;
-use uucore::{format_usage, util_name};
+use uucore::{format_usage, help_about, help_usage, show_error, util_name};
 
-static ABOUT: &str = "Remove the DIRECTORY(ies), if they are empty.";
-const USAGE: &str = "{} [OPTION]... DIRECTORY...";
+static ABOUT: &str = help_about!("rmdir.md");
+const USAGE: &str = help_usage!("rmdir.md");
 static OPT_IGNORE_FAIL_NON_EMPTY: &str = "ignore-fail-on-non-empty";
 static OPT_PARENTS: &str = "parents";
 static OPT_VERBOSE: &str = "verbose";
@@ -32,12 +28,12 @@ static ARG_DIRS: &str = "dirs";
 
 #[uucore::main]
 pub fn uumain(args: impl uucore::Args) -> UResult<()> {
-    let matches = uu_app().get_matches_from(args);
+    let matches = uu_app().try_get_matches_from(args)?;
 
     let opts = Opts {
-        ignore: matches.contains_id(OPT_IGNORE_FAIL_NON_EMPTY),
-        parents: matches.contains_id(OPT_PARENTS),
-        verbose: matches.contains_id(OPT_VERBOSE),
+        ignore: matches.get_flag(OPT_IGNORE_FAIL_NON_EMPTY),
+        parents: matches.get_flag(OPT_PARENTS),
+        verbose: matches.get_flag(OPT_VERBOSE),
     };
 
     for path in matches
@@ -77,7 +73,7 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
                 if error.raw_os_error() == Some(libc::ENOTDIR) && bytes.ends_with(b"/") {
                     // Strip the trailing slash or .symlink_metadata() will follow the symlink
                     let no_slash: &Path = OsStr::from_bytes(&bytes[..bytes.len() - 1]).as_ref();
-                    if is_symlink(no_slash) && points_to_directory(no_slash).unwrap_or(true) {
+                    if no_slash.is_symlink() && points_to_directory(no_slash).unwrap_or(true) {
                         show_error!(
                             "failed to remove {}: Symbolic link not followed",
                             path.quote()
@@ -168,7 +164,7 @@ struct Opts {
     verbose: bool,
 }
 
-pub fn uu_app<'a>() -> Command<'a> {
+pub fn uu_app() -> Command {
     Command::new(uucore::util_name())
         .version(crate_version!())
         .about(ABOUT)
@@ -177,25 +173,31 @@ pub fn uu_app<'a>() -> Command<'a> {
         .arg(
             Arg::new(OPT_IGNORE_FAIL_NON_EMPTY)
                 .long(OPT_IGNORE_FAIL_NON_EMPTY)
-                .help("ignore each failure that is solely because a directory is non-empty"),
+                .help("ignore each failure that is solely because a directory is non-empty")
+                .action(ArgAction::SetTrue),
         )
-        .arg(Arg::new(OPT_PARENTS).short('p').long(OPT_PARENTS).help(
-            "remove DIRECTORY and its ancestors; e.g.,
+        .arg(
+            Arg::new(OPT_PARENTS)
+                .short('p')
+                .long(OPT_PARENTS)
+                .help(
+                    "remove DIRECTORY and its ancestors; e.g.,
                   'rmdir -p a/b/c' is similar to rmdir a/b/c a/b a",
-        ))
+                )
+                .action(ArgAction::SetTrue),
+        )
         .arg(
             Arg::new(OPT_VERBOSE)
                 .short('v')
                 .long(OPT_VERBOSE)
-                .help("output a diagnostic for every directory processed"),
+                .help("output a diagnostic for every directory processed")
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(ARG_DIRS)
-                .multiple_occurrences(true)
-                .takes_value(true)
-                .min_values(1)
-                .required(true)
-                .allow_invalid_utf8(true)
+                .action(ArgAction::Append)
+                .num_args(1..)
+                .value_parser(ValueParser::os_string())
                 .value_hint(clap::ValueHint::DirPath),
         )
 }
