@@ -1,8 +1,9 @@
 // spell-checker:ignore (vars) RFILE
 
+use clap::builder::ValueParser;
 use uucore::error::{UResult, UUsageError};
 
-use clap::{Arg, Command};
+use clap::{crate_version, Arg, ArgAction, Command};
 use selinux::{OpaqueSecurityContext, SecurityClass, SecurityContext};
 use uucore::format_usage;
 
@@ -17,7 +18,6 @@ mod errors;
 use errors::error_exit_status;
 use errors::{Error, Result, RunconError};
 
-const VERSION: &str = env!("CARGO_PKG_VERSION");
 const ABOUT: &str = "Run command with specified security context.";
 const USAGE: &str = "\
     {} [CONTEXT COMMAND [ARG...]]
@@ -49,8 +49,9 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
         Err(r) => {
             if let Error::CommandLine(ref r) = r {
                 match r.kind() {
-                    clap::ErrorKind::DisplayHelp | clap::ErrorKind::DisplayVersion => {
-                        println!("{}", r);
+                    clap::error::ErrorKind::DisplayHelp
+                    | clap::error::ErrorKind::DisplayVersion => {
+                        println!("{r}");
                         return Ok(());
                     }
                     _ => {}
@@ -58,7 +59,7 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
             }
             return Err(UUsageError::new(
                 error_exit_status::ANOTHER_ERROR,
-                format!("{}", r),
+                format!("{r}"),
             ));
         }
     };
@@ -103,9 +104,9 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
     }
 }
 
-pub fn uu_app<'a>() -> Command<'a> {
+pub fn uu_app() -> Command {
     Command::new(uucore::util_name())
-        .version(VERSION)
+        .version(crate_version!())
         .about(ABOUT)
         .after_help(DESCRIPTION)
         .override_usage(format_usage(USAGE))
@@ -114,49 +115,45 @@ pub fn uu_app<'a>() -> Command<'a> {
             Arg::new(options::COMPUTE)
                 .short('c')
                 .long(options::COMPUTE)
-                .takes_value(false)
-                .help("Compute process transition context before modifying."),
+                .help("Compute process transition context before modifying.")
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::USER)
                 .short('u')
                 .long(options::USER)
-                .takes_value(true)
                 .value_name("USER")
                 .help("Set user USER in the target security context.")
-                .allow_invalid_utf8(true),
+                .value_parser(ValueParser::os_string()),
         )
         .arg(
             Arg::new(options::ROLE)
                 .short('r')
                 .long(options::ROLE)
-                .takes_value(true)
                 .value_name("ROLE")
                 .help("Set role ROLE in the target security context.")
-                .allow_invalid_utf8(true),
+                .value_parser(ValueParser::os_string()),
         )
         .arg(
             Arg::new(options::TYPE)
                 .short('t')
                 .long(options::TYPE)
-                .takes_value(true)
                 .value_name("TYPE")
                 .help("Set type TYPE in the target security context.")
-                .allow_invalid_utf8(true),
+                .value_parser(ValueParser::os_string()),
         )
         .arg(
             Arg::new(options::RANGE)
                 .short('l')
                 .long(options::RANGE)
-                .takes_value(true)
                 .value_name("RANGE")
                 .help("Set range RANGE in the target security context.")
-                .allow_invalid_utf8(true),
+                .value_parser(ValueParser::os_string()),
         )
         .arg(
             Arg::new("ARG")
-                .multiple_occurrences(true)
-                .allow_invalid_utf8(true)
+                .action(ArgAction::Append)
+                .value_parser(ValueParser::os_string())
                 .value_hint(clap::ValueHint::CommandName),
         )
         // Once "ARG" is parsed, everything after that belongs to it.
@@ -209,7 +206,7 @@ struct Options {
 fn parse_command_line(config: Command, args: impl uucore::Args) -> Result<Options> {
     let matches = config.try_get_matches_from(args)?;
 
-    let compute_transition_context = matches.contains_id(options::COMPUTE);
+    let compute_transition_context = matches.get_flag(options::COMPUTE);
 
     let mut args = matches
         .get_many::<OsString>("ARG")
@@ -266,7 +263,7 @@ fn print_current_context() -> Result<()> {
 
     if let Some(context) = context {
         let context = context.as_ref().to_str()?;
-        println!("{}", context);
+        println!("{context}");
     } else {
         println!();
     }
