@@ -37,21 +37,21 @@ pub enum BadSequence {
 impl Display for BadSequence {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         match self {
-            Self::MissingCharClassName => writeln!(f, "missing character class name '[::]'"),
+            Self::MissingCharClassName => write!(f, "missing character class name '[::]'"),
             Self::MissingEquivalentClassChar => {
-                writeln!(f, "missing equivalence class character '[==]'")
+                write!(f, "missing equivalence class character '[==]'")
             }
             Self::MultipleCharRepeatInSet2 => {
-                writeln!(f, "only one [c*] repeat construct may appear in string2")
+                write!(f, "only one [c*] repeat construct may appear in string2")
             }
             Self::CharRepeatInSet1 => {
-                writeln!(f, "the [c*] repeat construct may not appear in string1")
+                write!(f, "the [c*] repeat construct may not appear in string1")
             }
             Self::InvalidRepeatCount(count) => {
-                writeln!(f, "invalid repeat count '{}' in [c*n] construct", count)
+                write!(f, "invalid repeat count '{count}' in [c*n] construct")
             }
             Self::EmptySet2WhenNotTruncatingSet1 => {
-                writeln!(f, "when not truncating set1, string2 must be non-empty")
+                write!(f, "when not truncating set1, string2 must be non-empty")
             }
         }
     }
@@ -143,13 +143,13 @@ impl Sequence {
         let set1 = Self::from_str(set1_str)?;
         let set2 = Self::from_str(set2_str)?;
 
-        let is_char_star = |s: &&Self| -> bool { matches!(s, Sequence::CharStar(_)) };
+        let is_char_star = |s: &&Self| -> bool { matches!(s, Self::CharStar(_)) };
         let set1_star_count = set1.iter().filter(is_char_star).count();
         if set1_star_count == 0 {
             let set2_star_count = set2.iter().filter(is_char_star).count();
             if set2_star_count < 2 {
                 let char_star = set2.iter().find_map(|s| match s {
-                    Sequence::CharStar(c) => Some(c),
+                    Self::CharStar(c) => Some(c),
                     _ => None,
                 });
                 let mut partition = set2.as_slice().split(|s| matches!(s, Self::CharStar(_)));
@@ -157,7 +157,7 @@ impl Sequence {
                 let set2_len = set2
                     .iter()
                     .filter_map(|s| match s {
-                        Sequence::CharStar(_) => None,
+                        Self::CharStar(_) => None,
                         r => Some(r),
                     })
                     .flat_map(Self::flatten)
