@@ -9,9 +9,7 @@
 
 // spell-checker:ignore (ToDO) nums aflag uflag scol prevtab amode ctype cwidth nbytes lastcol pctype Preprocess
 
-#[macro_use]
-extern crate uucore;
-use clap::{crate_version, Arg, Command};
+use clap::{crate_version, Arg, ArgAction, Command};
 use std::error::Error;
 use std::fmt;
 use std::fs::File;
@@ -21,12 +19,11 @@ use std::str::from_utf8;
 use unicode_width::UnicodeWidthChar;
 use uucore::display::Quotable;
 use uucore::error::{FromIo, UError, UResult};
-use uucore::format_usage;
+use uucore::{crash, crash_if_err, format_usage};
 
-static NAME: &str = "unexpand";
 static USAGE: &str = "{} [OPTION]... [FILE]...";
-static ABOUT: &str = r#"Convert blanks in each FILE to tabs, writing to standard output.
-                        With no FILE, or when FILE is -, read standard input."#;
+static ABOUT: &str = "Convert blanks in each FILE to tabs, writing to standard output.\n\n\
+                      With no FILE, or when FILE is -, read standard input.";
 
 const DEFAULT_TABSTOP: usize = 8;
 
@@ -109,9 +106,9 @@ impl Options {
             Some(s) => tabstops_parse(&s.map(|s| s.as_str()).collect::<Vec<_>>().join(","))?,
         };
 
-        let aflag = (matches.contains_id(options::ALL) || matches.contains_id(options::TABS))
-            && !matches.contains_id(options::FIRST_ONLY);
-        let uflag = !matches.contains_id(options::NO_UTF8);
+        let aflag = (matches.get_flag(options::ALL) || matches.contains_id(options::TABS))
+            && !matches.get_flag(options::FIRST_ONLY);
+        let uflag = !matches.get_flag(options::NO_UTF8);
 
         let files = match matches.get_one::<String>(options::FILE) {
             Some(v) => vec![v.to_string()],
@@ -145,7 +142,7 @@ fn expand_shortcuts(args: &[String]) -> Vec<String> {
             arg[1..]
                 .split(',')
                 .filter(|s| !s.is_empty())
-                .for_each(|s| processed_args.push(format!("--tabs={}", s)));
+                .for_each(|s| processed_args.push(format!("--tabs={s}")));
             has_shortcuts = true;
         } else {
             processed_args.push(arg.to_string());
@@ -172,9 +169,8 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
     unexpand(&Options::new(&matches)?).map_err_context(String::new)
 }
 
-pub fn uu_app<'a>() -> Command<'a> {
+pub fn uu_app() -> Command {
     Command::new(uucore::util_name())
-        .name(NAME)
         .version(crate_version!())
         .override_usage(format_usage(USAGE))
         .about(ABOUT)
@@ -182,37 +178,40 @@ pub fn uu_app<'a>() -> Command<'a> {
         .arg(
             Arg::new(options::FILE)
                 .hide(true)
-                .multiple_occurrences(true)
-                .value_hint(clap::ValueHint::FilePath)
+                .action(ArgAction::Append)
+                .value_hint(clap::ValueHint::FilePath),
         )
         .arg(
             Arg::new(options::ALL)
                 .short('a')
                 .long(options::ALL)
                 .help("convert all blanks, instead of just initial blanks")
-                .takes_value(false),
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::FIRST_ONLY)
                 .long(options::FIRST_ONLY)
                 .help("convert only leading sequences of blanks (overrides -a)")
-                .takes_value(false),
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::TABS)
                 .short('t')
                 .long(options::TABS)
-                .help("use comma separated LIST of tab positions or have tabs N characters apart instead of 8 (enables -a)")
-                .takes_value(true)
-                .multiple_occurrences(true)
-                .value_name("N, LIST")
+                .help(
+                    "use comma separated LIST of tab positions or have tabs N characters \
+                apart instead of 8 (enables -a)",
+                )
+                .action(ArgAction::Append)
+                .value_name("N, LIST"),
         )
         .arg(
             Arg::new(options::NO_UTF8)
                 .short('U')
                 .long(options::NO_UTF8)
-                .takes_value(false)
-                .help("interpret input file as 8-bit ASCII rather than UTF-8"))
+                .help("interpret input file as 8-bit ASCII rather than UTF-8")
+                .action(ArgAction::SetTrue),
+        )
 }
 
 fn open(path: &str) -> BufReader<Box<dyn Read + 'static>> {
@@ -220,7 +219,7 @@ fn open(path: &str) -> BufReader<Box<dyn Read + 'static>> {
     if path == "-" {
         BufReader::new(Box::new(stdin()) as Box<dyn Read>)
     } else {
-        file_buf = match File::open(&path) {
+        file_buf = match File::open(path) {
             Ok(a) => a,
             Err(e) => crash!(1, "{}: {}", path.maybe_quote(), e),
         };
