@@ -15,7 +15,7 @@ use count_fast::{count_bytes_chars_and_lines_fast, count_bytes_fast};
 use countable::WordCountable;
 use unicode_width::UnicodeWidthChar;
 use utf8::{BufReadDecoder, BufReadDecoderError};
-use uucore::{format_usage, show};
+use uucore::{format_usage, help_about, help_usage, show};
 use word_count::{TitledWordCount, WordCount};
 
 use clap::{crate_version, Arg, ArgAction, ArgMatches, Command};
@@ -42,6 +42,7 @@ struct Settings {
     show_max_line_length: bool,
     files0_from_stdin_mode: bool,
     title_quoting_style: QuotingStyle,
+    total_when: TotalWhen,
 }
 
 impl Settings {
@@ -65,6 +66,7 @@ impl Settings {
             show_max_line_length: matches.get_flag(options::MAX_LINE_LENGTH),
             files0_from_stdin_mode,
             title_quoting_style,
+            total_when: matches.get_one::<String>(options::TOTAL).unwrap().into(),
         };
 
         if settings.show_bytes
@@ -84,6 +86,7 @@ impl Settings {
             show_max_line_length: false,
             files0_from_stdin_mode,
             title_quoting_style: settings.title_quoting_style,
+            total_when: settings.total_when,
         }
     }
 
@@ -98,9 +101,8 @@ impl Settings {
     }
 }
 
-static ABOUT: &str = "Display newline, word, and byte counts for each FILE, and a total line if
-more than one FILE is specified. With no FILE, or when FILE is -, read standard input.";
-const USAGE: &str = "{} [OPTION]... [FILE]...";
+const ABOUT: &str = help_about!("wc.md");
+const USAGE: &str = help_usage!("wc.md");
 
 pub mod options {
     pub static BYTES: &str = "bytes";
@@ -108,6 +110,7 @@ pub mod options {
     pub static FILES0_FROM: &str = "files0-from";
     pub static LINES: &str = "lines";
     pub static MAX_LINE_LENGTH: &str = "max-line-length";
+    pub static TOTAL: &str = "total";
     pub static WORDS: &str = "words";
 }
 
@@ -161,6 +164,37 @@ impl Input {
     }
 }
 
+/// When to show the "total" line
+#[derive(PartialEq)]
+enum TotalWhen {
+    Auto,
+    Always,
+    Only,
+    Never,
+}
+
+impl From<&String> for TotalWhen {
+    fn from(s: &String) -> Self {
+        match s.as_ref() {
+            "auto" => Self::Auto,
+            "always" => Self::Always,
+            "only" => Self::Only,
+            "never" => Self::Never,
+            _ => unreachable!("Should have been caught by clap"),
+        }
+    }
+}
+
+impl TotalWhen {
+    fn is_total_row_visible(&self, num_inputs: usize) -> bool {
+        match self {
+            Self::Auto => num_inputs > 1,
+            Self::Always | Self::Only => true,
+            Self::Never => false,
+        }
+    }
+}
+
 #[derive(Debug)]
 enum WcError {
     FilesDisabled(String),
@@ -185,7 +219,7 @@ impl Display for WcError {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         match self {
             Self::FilesDisabled(message) | Self::StdinReprNotAllowed(message) => {
-                write!(f, "{}", message)
+                write!(f, "{message}")
             }
         }
     }
@@ -247,6 +281,15 @@ pub fn uu_app() -> Command {
                 .help("print the length of the longest line")
                 .action(ArgAction::SetTrue),
         )
+        .arg(
+            Arg::new(options::TOTAL)
+                .long(options::TOTAL)
+                .value_parser(["auto", "always", "only", "never"])
+                .default_value("auto")
+                .hide_default_value(true)
+                .value_name("WHEN")
+                .help("when to print a line with total counts"),
+        )
         .arg(
             Arg::new(options::WORDS)
                 .short('w')
@@ -562,11 +605,19 @@ fn compute_number_width(inputs: &[Input], settings: &Settings) -> usize {
 }
 
 fn wc(inputs: &[Input], settings: &Settings) -> UResult<()> {
-    let number_width = compute_number_width(inputs, settings);
-
     let mut total_word_count = WordCount::default();
 
-    let num_inputs = inputs.len();
+    let (number_width, are_stats_visible, total_row_title) =
+        if settings.total_when == TotalWhen::Only {
+            (1, false, None)
+        } else {
+            let number_width = compute_number_width(inputs, settings);
+            let title = Some(String::from("total"));
+
+            (number_width, true, title)
+        };
+
+    let is_total_row_visible = settings.total_when.is_total_row_visible(inputs.len());
 
     for input in inputs {
         let word_count = match word_count_from_input(input, settings) {
@@ -596,24 +647,27 @@ fn wc(inputs: &[Input], settings: &Settings) -> UResult<()> {
         };
         total_word_count += word_count;
         let result = word_count.with_title(input.to_title(&settings.title_quoting_style));
-        if let Err(err) = print_stats(settings, &result, number_width) {
-            show!(USimpleError::new(
-                1,
-                format!(
-                    "failed to print result for {}: {}",
-                    &result.title.unwrap_or_else(|| String::from("<stdin>")),
-                    err,
-                ),
-            ));
+
+        if are_stats_visible {
+            if let Err(err) = print_stats(settings, &result, number_width) {
+                show!(USimpleError::new(
+                    1,
+                    format!(
+                        "failed to print result for {}: {}",
+                        &result.title.unwrap_or_else(|| String::from("<stdin>")),
+                        err,
+                    ),
+                ));
+            }
         }
     }
 
-    if num_inputs > 1 {
-        let total_result = total_word_count.with_title(Some(String::from("total")));
+    if is_total_row_visible {
+        let total_result = total_word_count.with_title(total_row_title);
         if let Err(err) = print_stats(settings, &total_result, number_width) {
             show!(USimpleError::new(
                 1,
-                format!("failed to print total: {}", err)
+                format!("failed to print total: {err}")
             ));
         }
     }
