@@ -81,9 +81,10 @@ impl FileInformation {
             let mut open_options = OpenOptions::new();
             let mut custom_flags = 0;
             if !dereference {
-                custom_flags |= winapi::um::winbase::FILE_FLAG_OPEN_REPARSE_POINT;
+                custom_flags |=
+                    windows_sys::Win32::Storage::FileSystem::FILE_FLAG_OPEN_REPARSE_POINT;
             }
-            custom_flags |= winapi::um::winbase::FILE_FLAG_BACKUP_SEMANTICS;
+            custom_flags |= windows_sys::Win32::Storage::FileSystem::FILE_FLAG_BACKUP_SEMANTICS;
             open_options.custom_flags(custom_flags);
             let file = open_options.read(true).open(path.as_ref())?;
             Self::from_file(&file)
@@ -108,15 +109,38 @@ impl FileInformation {
     }
 
     pub fn number_of_links(&self) -> u64 {
-        #[cfg(unix)]
-        return self.0.st_nlink as u64;
+        #[cfg(all(
+            unix,
+            not(target_vendor = "apple"),
+            not(target_os = "android"),
+            not(target_os = "freebsd"),
+            not(target_arch = "aarch64"),
+            not(target_arch = "riscv64"),
+            target_pointer_width = "64"
+        ))]
+        return self.0.st_nlink;
+        #[cfg(all(
+            unix,
+            any(
+                target_vendor = "apple",
+                target_os = "android",
+                target_os = "freebsd",
+                target_arch = "aarch64",
+                target_arch = "riscv64",
+                not(target_pointer_width = "64")
+            )
+        ))]
+        return self.0.st_nlink.into();
         #[cfg(windows)]
-        return self.0.number_of_links() as u64;
+        return self.0.number_of_links();
     }
 
     #[cfg(unix)]
     pub fn inode(&self) -> u64 {
-        self.0.st_ino as u64
+        #[cfg(all(not(target_os = "freebsd"), target_pointer_width = "64"))]
+        return self.0.st_ino;
+        #[cfg(any(target_os = "freebsd", not(target_pointer_width = "64")))]
+        return self.0.st_ino.into();
     }
 }
 
@@ -355,7 +379,7 @@ pub fn canonicalize<P: AsRef<Path>>(
                     followed_symlinks += 1;
                 } else {
                     let file_info =
-                        FileInformation::from_path(&result.parent().unwrap(), false).unwrap();
+                        FileInformation::from_path(result.parent().unwrap(), false).unwrap();
                     let mut path_to_follow = PathBuf::new();
                     for part in &parts {
                         path_to_follow.push(part.as_os_str());
@@ -395,18 +419,32 @@ pub fn canonicalize<P: AsRef<Path>>(
                 read_dir(parent)?;
             }
         }
-        _ => {}
+        MissingHandling::Missing => {}
     }
     Ok(result)
 }
 
 #[cfg(not(unix))]
-#[allow(unused_variables)]
 pub fn display_permissions(metadata: &fs::Metadata, display_file_type: bool) -> String {
+    let write = if metadata.permissions().readonly() {
+        '-'
+    } else {
+        'w'
+    };
+
     if display_file_type {
-        return String::from("----------");
+        let file_type = if metadata.is_symlink() {
+            'l'
+        } else if metadata.is_dir() {
+            'd'
+        } else {
+            '-'
+        };
+
+        format!("{file_type}r{write}xr{write}xr{write}x")
+    } else {
+        format!("r{write}xr{write}xr{write}x")
     }
-    String::from("---------")
 }
 
 #[cfg(unix)]
@@ -417,6 +455,8 @@ pub fn display_permissions(metadata: &fs::Metadata, display_file_type: bool) ->
     display_permissions_unix(mode, display_file_type)
 }
 
+// The logic below is more readable written this way.
+#[allow(clippy::if_not_else)]
 #[cfg(unix)]
 /// Display the permissions of a file on a unix like system
 pub fn display_permissions_unix(mode: mode_t, display_file_type: bool) -> String {
