@@ -32,7 +32,6 @@
 //! ```
 
 pub extern crate time;
-use self::time::{Timespec, Tm};
 
 use std::ffi::CString;
 use std::io::Result as IOResult;
@@ -52,6 +51,9 @@ pub use libc::getutxent;
 pub use libc::setutxent;
 #[cfg(any(target_vendor = "apple", target_os = "linux", target_os = "netbsd"))]
 pub use libc::utmpxname;
+
+/// # Safety
+/// Just fixed the clippy warning. Please add description here.
 #[cfg(target_os = "freebsd")]
 pub unsafe extern "C" fn utmpxname(_file: *const libc::c_char) -> libc::c_int {
     0
@@ -166,11 +168,11 @@ pub struct Utmpx {
 impl Utmpx {
     /// A.K.A. ut.ut_type
     pub fn record_type(&self) -> i16 {
-        self.inner.ut_type as i16
+        self.inner.ut_type
     }
     /// A.K.A. ut.ut_pid
     pub fn pid(&self) -> i32 {
-        self.inner.ut_pid as i32
+        self.inner.ut_pid
     }
     /// A.K.A. ut.ut_id
     pub fn terminal_suffix(&self) -> String {
@@ -189,23 +191,27 @@ impl Utmpx {
         chars2string!(self.inner.ut_line)
     }
     /// A.K.A. ut.ut_tv
-    pub fn login_time(&self) -> Tm {
-        time::at(Timespec::new(
-            self.inner.ut_tv.tv_sec as i64,
-            self.inner.ut_tv.tv_usec as i32,
-        ))
+    pub fn login_time(&self) -> time::OffsetDateTime {
+        #[allow(clippy::unnecessary_cast)]
+        let ts_nanos: i128 = (1_000_000_000_i64 * self.inner.ut_tv.tv_sec as i64
+            + 1_000_i64 * self.inner.ut_tv.tv_usec as i64)
+            .into();
+        let local_offset = time::OffsetDateTime::now_local().unwrap().offset();
+        time::OffsetDateTime::from_unix_timestamp_nanos(ts_nanos)
+            .unwrap()
+            .to_offset(local_offset)
     }
     /// A.K.A. ut.ut_exit
     ///
     /// Return (e_termination, e_exit)
-    #[cfg(any(target_os = "linux", target_os = "android"))]
+    #[cfg(target_os = "linux")]
     pub fn exit_status(&self) -> (i16, i16) {
         (self.inner.ut_exit.e_termination, self.inner.ut_exit.e_exit)
     }
     /// A.K.A. ut.ut_exit
     ///
     /// Return (0, 0) on Non-Linux platform
-    #[cfg(not(any(target_os = "linux", target_os = "android")))]
+    #[cfg(not(target_os = "linux"))]
     pub fn exit_status(&self) -> (i16, i16) {
         (0, 0)
     }
@@ -221,11 +227,7 @@ impl Utmpx {
     pub fn canon_host(&self) -> IOResult<String> {
         let host = self.host();
 
-        // TODO: change to use `split_once` when MSRV hits 1.52.0
-        // let (hostname, display) = host.split_once(':').unwrap_or((&host, ""));
-        let mut h = host.split(':');
-        let hostname = h.next().unwrap_or(&host);
-        let display = h.next().unwrap_or("");
+        let (hostname, display) = host.split_once(':').unwrap_or((&host, ""));
 
         if !hostname.is_empty() {
             extern crate dns_lookup;
@@ -243,7 +245,7 @@ impl Utmpx {
                         return Ok(if display.is_empty() {
                             ai_canonname
                         } else {
-                            format!("{}:{}", ai_canonname, display)
+                            format!("{ai_canonname}:{display}")
                         });
                     }
                 }
@@ -334,7 +336,9 @@ impl Iterator for UtmpxIter {
     fn next(&mut self) -> Option<Self::Item> {
         unsafe {
             let res = getutxent();
-            if !res.is_null() {
+            if res.is_null() {
+                None
+            } else {
                 // The data behind this pointer will be replaced by the next
                 // call to getutxent(), so we have to read it now.
                 // All the strings live inline in the struct as arrays, which
@@ -342,8 +346,6 @@ impl Iterator for UtmpxIter {
                 Some(Utmpx {
                     inner: ptr::read(res as *const _),
                 })
-            } else {
-                None
             }
         }
     }
