@@ -1,6 +1,6 @@
 // spell-checker:ignore (words) agroupthatdoesntexist auserthatdoesntexist cuuser groupname notexisting passgrp
 
-use crate::common::util::*;
+use crate::common::util::{is_ci, run_ucmd_as_root, CmdResult, TestScenario};
 #[cfg(any(target_os = "linux", target_os = "android"))]
 use rust_users::get_effective_uid;
 
@@ -107,7 +107,7 @@ fn test_chown_only_owner() {
         .arg("--verbose")
         .arg(file1)
         .run();
-    result.stderr_contains(&"retained as");
+    result.stderr_contains("retained as");
 
     // try to change to another existing user, e.g. 'root'
     scene
@@ -116,7 +116,7 @@ fn test_chown_only_owner() {
         .arg("--verbose")
         .arg(file1)
         .fails()
-        .stderr_contains(&"failed to change");
+        .stderr_contains("failed to change");
 }
 
 #[test]
@@ -138,19 +138,19 @@ fn test_chown_only_owner_colon() {
 
     scene
         .ucmd()
-        .arg(format!("{}:", user_name))
+        .arg(format!("{user_name}:"))
         .arg("--verbose")
         .arg(file1)
         .succeeds()
-        .stderr_contains(&"retained as");
+        .stderr_contains("retained as");
 
     scene
         .ucmd()
-        .arg(format!("{}.", user_name))
+        .arg(format!("{user_name}."))
         .arg("--verbose")
         .arg(file1)
         .succeeds()
-        .stderr_contains(&"retained as");
+        .stderr_contains("retained as");
 
     scene
         .ucmd()
@@ -158,7 +158,7 @@ fn test_chown_only_owner_colon() {
         .arg("--verbose")
         .arg(file1)
         .fails()
-        .stderr_contains(&"failed to change");
+        .stderr_contains("failed to change");
 }
 
 #[test]
@@ -179,7 +179,7 @@ fn test_chown_only_colon() {
     if skipping_test_is_okay(&result, "No such id") {
         return;
     }
-    result.stderr_contains(&"retained as"); // TODO: verbose is not printed to stderr in GNU chown
+    result.stderr_contains("retained as"); // TODO: verbose is not printed to stderr in GNU chown
 
     // test chown : file.txt
     // expected:
@@ -192,7 +192,7 @@ fn test_chown_only_colon() {
         .arg("--verbose")
         .arg(file1)
         .fails()
-        .stderr_contains(&"invalid group: '::'");
+        .stderr_contains("invalid group: '::'");
 
     scene
         .ucmd()
@@ -200,7 +200,7 @@ fn test_chown_only_colon() {
         .arg("--verbose")
         .arg(file1)
         .fails()
-        .stderr_contains(&"invalid group: '..'");
+        .stderr_contains("invalid group: '..'");
 }
 
 #[test]
@@ -244,14 +244,14 @@ fn test_chown_owner_group() {
 
     let result = scene
         .ucmd()
-        .arg(format!("{}:{}", user_name, group_name))
+        .arg(format!("{user_name}:{group_name}"))
         .arg("--verbose")
         .arg(file1)
         .run();
     if skipping_test_is_okay(&result, "chown: invalid group:") {
         return;
     }
-    result.stderr_contains(&"retained as");
+    result.stderr_contains("retained as");
 
     scene
         .ucmd()
@@ -259,7 +259,7 @@ fn test_chown_owner_group() {
         .arg("--verbose")
         .arg(file1)
         .fails()
-        .stderr_contains(&"invalid group");
+        .stderr_contains("invalid group");
 
     scene
         .ucmd()
@@ -267,7 +267,7 @@ fn test_chown_owner_group() {
         .arg("--verbose")
         .arg(file1)
         .fails()
-        .stderr_contains(&"invalid group");
+        .stderr_contains("invalid group");
 
     // TODO: on macos group name is not recognized correctly: "chown: invalid group: 'root:root'
     #[cfg(any(windows, all(unix, not(target_os = "macos"))))]
@@ -277,7 +277,7 @@ fn test_chown_owner_group() {
         .arg("--verbose")
         .arg(file1)
         .fails()
-        .stderr_contains(&"failed to change");
+        .stderr_contains("failed to change");
 }
 
 #[test]
@@ -309,26 +309,26 @@ fn test_chown_various_input() {
 
     let result = scene
         .ucmd()
-        .arg(format!("{}:{}", user_name, group_name))
+        .arg(format!("{user_name}:{group_name}"))
         .arg("--verbose")
         .arg(file1)
         .run();
     if skipping_test_is_okay(&result, "chown: invalid group:") {
         return;
     }
-    result.stderr_contains(&"retained as");
+    result.stderr_contains("retained as");
 
     // check that username.groupname is understood
     let result = scene
         .ucmd()
-        .arg(format!("{}.{}", user_name, group_name))
+        .arg(format!("{user_name}.{group_name}"))
         .arg("--verbose")
         .arg(file1)
         .run();
     if skipping_test_is_okay(&result, "chown: invalid group:") {
         return;
     }
-    result.stderr_contains(&"retained as");
+    result.stderr_contains("retained as");
 
     // Fails as user.name doesn't exist in the CI
     // but it is valid
@@ -338,7 +338,7 @@ fn test_chown_various_input() {
         .arg("--verbose")
         .arg(file1)
         .fails()
-        .stderr_contains(&"chown: invalid user: 'user.name:groupname'");
+        .stderr_contains("chown: invalid user: 'user.name:groupname'");
 }
 
 #[test]
@@ -365,7 +365,7 @@ fn test_chown_only_group() {
 
     let result = scene
         .ucmd()
-        .arg(format!(":{}", user_name))
+        .arg(format!(":{user_name}"))
         .arg("--verbose")
         .arg(file1)
         .run();
@@ -377,7 +377,7 @@ fn test_chown_only_group() {
         // With mac into the CI, we can get this answer
         return;
     }
-    result.stderr_contains(&"retained as");
+    result.stderr_contains("retained as");
     result.success();
 
     scene
@@ -386,7 +386,7 @@ fn test_chown_only_group() {
         .arg("--verbose")
         .arg(file1)
         .fails()
-        .stderr_contains(&"failed to change");
+        .stderr_contains("failed to change");
 }
 
 #[test]
@@ -396,7 +396,7 @@ fn test_chown_only_user_id() {
     let scene = TestScenario::new(util_name!());
     let at = &scene.fixtures;
 
-    let result = scene.cmd_keepenv("id").arg("-u").run();
+    let result = scene.cmd("id").arg("-u").run();
     if skipping_test_is_okay(&result, "id: cannot find name for group ID") {
         return;
     }
@@ -412,7 +412,7 @@ fn test_chown_only_user_id() {
         // stderr: "chown: invalid user: '1001'
         return;
     }
-    result.stderr_contains(&"retained as");
+    result.stderr_contains("retained as");
 
     scene
         .ucmd()
@@ -420,7 +420,39 @@ fn test_chown_only_user_id() {
         .arg("--verbose")
         .arg(file1)
         .fails()
-        .stderr_contains(&"failed to change");
+        .stderr_contains("failed to change");
+}
+
+#[test]
+fn test_chown_fail_id() {
+    // test chown 1111. file.txt
+
+    let scene = TestScenario::new(util_name!());
+    let at = &scene.fixtures;
+
+    let result = scene.cmd("id").arg("-u").run();
+    if skipping_test_is_okay(&result, "id: cannot find name for group ID") {
+        return;
+    }
+    let user_id = String::from(result.stdout_str().trim());
+    assert!(!user_id.is_empty());
+
+    let file1 = "test_chown_file1";
+    at.touch(file1);
+
+    scene
+        .ucmd()
+        .arg(format!("{user_id}:"))
+        .arg(file1)
+        .fails()
+        .stderr_contains("invalid spec");
+
+    scene
+        .ucmd()
+        .arg(format!("{user_id}."))
+        .arg(file1)
+        .fails()
+        .stderr_contains("invalid spec");
 }
 
 /// Test for setting the owner to a user ID for a user that does not exist.
@@ -455,7 +487,7 @@ fn test_chown_only_group_id() {
     let scene = TestScenario::new(util_name!());
     let at = &scene.fixtures;
 
-    let result = scene.cmd_keepenv("id").arg("-g").run();
+    let result = scene.cmd("id").arg("-g").run();
     if skipping_test_is_okay(&result, "id: cannot find name for group ID") {
         return;
     }
@@ -467,7 +499,7 @@ fn test_chown_only_group_id() {
 
     let result = scene
         .ucmd()
-        .arg(format!(":{}", group_id))
+        .arg(format!(":{group_id}"))
         .arg("--verbose")
         .arg(file1)
         .run();
@@ -475,7 +507,7 @@ fn test_chown_only_group_id() {
         // With mac into the CI, we can get this answer
         return;
     }
-    result.stderr_contains(&"retained as");
+    result.stderr_contains("retained as");
 
     // Apparently on CI "macos-latest, x86_64-apple-darwin, feat_os_macos"
     // the process has the rights to change from runner:staff to runner:wheel
@@ -486,7 +518,7 @@ fn test_chown_only_group_id() {
         .arg("--verbose")
         .arg(file1)
         .fails()
-        .stderr_contains(&"failed to change");
+        .stderr_contains("failed to change");
 }
 
 /// Test for setting the group to a group ID for a group that does not exist.
@@ -519,14 +551,14 @@ fn test_chown_owner_group_id() {
     let scene = TestScenario::new(util_name!());
     let at = &scene.fixtures;
 
-    let result = scene.cmd_keepenv("id").arg("-u").run();
+    let result = scene.cmd("id").arg("-u").run();
     if skipping_test_is_okay(&result, "id: cannot find name for group ID") {
         return;
     }
     let user_id = String::from(result.stdout_str().trim());
     assert!(!user_id.is_empty());
 
-    let result = scene.cmd_keepenv("id").arg("-g").run();
+    let result = scene.cmd("id").arg("-g").run();
     if skipping_test_is_okay(&result, "id: cannot find name for group ID") {
         return;
     }
@@ -538,7 +570,7 @@ fn test_chown_owner_group_id() {
 
     let result = scene
         .ucmd()
-        .arg(format!("{}:{}", user_id, group_id))
+        .arg(format!("{user_id}:{group_id}"))
         .arg("--verbose")
         .arg(file1)
         .run();
@@ -547,11 +579,11 @@ fn test_chown_owner_group_id() {
         // stderr: "chown: invalid user: '1001:116'
         return;
     }
-    result.stderr_contains(&"retained as");
+    result.stderr_contains("retained as");
 
     let result = scene
         .ucmd()
-        .arg(format!("{}.{}", user_id, group_id))
+        .arg(format!("{user_id}.{group_id}"))
         .arg("--verbose")
         .arg(file1)
         .run();
@@ -560,7 +592,7 @@ fn test_chown_owner_group_id() {
         // stderr: "chown: invalid user: '1001.116'
         return;
     }
-    result.stderr_contains(&"retained as");
+    result.stderr_contains("retained as");
 
     scene
         .ucmd()
@@ -568,7 +600,7 @@ fn test_chown_owner_group_id() {
         .arg("--verbose")
         .arg(file1)
         .fails()
-        .stderr_contains(&"failed to change");
+        .stderr_contains("failed to change");
 }
 
 #[test]
@@ -580,14 +612,14 @@ fn test_chown_owner_group_mix() {
     let scene = TestScenario::new(util_name!());
     let at = &scene.fixtures;
 
-    let result = scene.cmd_keepenv("id").arg("-u").run();
+    let result = scene.cmd("id").arg("-u").run();
     if skipping_test_is_okay(&result, "id: cannot find name for group ID") {
         return;
     }
     let user_id = String::from(result.stdout_str().trim());
     assert!(!user_id.is_empty());
 
-    let result = scene.cmd_keepenv("id").arg("-gn").run();
+    let result = scene.cmd("id").arg("-gn").run();
     if skipping_test_is_okay(&result, "id: cannot find name for group ID") {
         return;
     }
@@ -599,11 +631,11 @@ fn test_chown_owner_group_mix() {
 
     let result = scene
         .ucmd()
-        .arg(format!("{}:{}", user_id, group_name))
+        .arg(format!("{user_id}:{group_name}"))
         .arg("--verbose")
         .arg(file1)
         .run();
-    result.stderr_contains(&"retained as");
+    result.stderr_contains("retained as");
 
     // TODO: on macos group name is not recognized correctly: "chown: invalid group: '0:root'
     #[cfg(any(windows, all(unix, not(target_os = "macos"))))]
@@ -613,7 +645,7 @@ fn test_chown_owner_group_mix() {
         .arg("--verbose")
         .arg(file1)
         .fails()
-        .stderr_contains(&"failed to change");
+        .stderr_contains("failed to change");
 }
 
 #[test]
@@ -630,10 +662,10 @@ fn test_chown_recursive() {
 
     at.mkdir_all("a/b/c");
     at.mkdir("z");
-    at.touch(&at.plus_as_string("a/a"));
-    at.touch(&at.plus_as_string("a/b/b"));
-    at.touch(&at.plus_as_string("a/b/c/c"));
-    at.touch(&at.plus_as_string("z/y"));
+    at.touch(at.plus_as_string("a/a"));
+    at.touch(at.plus_as_string("a/b/b"));
+    at.touch(at.plus_as_string("a/b/c/c"));
+    at.touch(at.plus_as_string("z/y"));
 
     let result = scene
         .ucmd()
@@ -643,8 +675,8 @@ fn test_chown_recursive() {
         .arg("a")
         .arg("z")
         .run();
-    result.stderr_contains(&"ownership of 'a/a' retained as");
-    result.stderr_contains(&"ownership of 'z/y' retained as");
+    result.stderr_contains("ownership of 'a/a' retained as");
+    result.stderr_contains("ownership of 'z/y' retained as");
 }
 
 #[test]
@@ -665,7 +697,7 @@ fn test_root_preserve() {
         .arg(user_name)
         .arg("/")
         .fails();
-    result.stderr_contains(&"chown: it is dangerous to operate recursively");
+    result.stderr_contains("chown: it is dangerous to operate recursively");
 }
 
 #[cfg(any(target_os = "linux", target_os = "android"))]
@@ -708,7 +740,7 @@ fn test_chown_file_notexisting() {
         .fails();
 
     // TODO: uncomment once "failed to change ownership of '{}' to {}" added to stdout
-    // result.stderr_contains(&"retained as");
+    // result.stderr_contains("retained as");
     // TODO: uncomment once message changed from "cannot dereference" to "cannot access"
-    // result.stderr_contains(&"cannot access 'not_existing': No such file or directory");
+    // result.stderr_contains("cannot access 'not_existing': No such file or directory");
 }
