@@ -3,11 +3,9 @@ extern crate libc;
 extern crate regex;
 #[cfg(not(windows))]
 extern crate tempfile;
-#[cfg(unix)]
-extern crate unix_socket;
 
 use self::regex::Regex;
-use crate::common::util::*;
+use crate::common::util::TestScenario;
 
 /*
  * As dir use the same functions than ls, we don't have to retest them here.
@@ -51,5 +49,5 @@ fn test_long_output() {
         .ucmd()
         .arg("-l")
         .succeeds()
-        .stdout_matches(&Regex::new("[rwx-]{10}.*some-file1$").unwrap());
+        .stdout_matches(&Regex::new("[rwx-]{10}.*some-file1\n$").unwrap());
 }
