@@ -1,9 +1,14 @@
 // spell-checker:ignore overridable
-use crate::common::util::*;
+use crate::common::util::TestScenario;
 
 extern crate dircolors;
 use self::dircolors::{guess_syntax, OutputFmt, StrUtils};
 
+#[test]
+fn test_invalid_arg() {
+    new_ucmd!().arg("--definitely-invalid").fails().code_is(1);
+}
+
 #[test]
 fn test_shell_syntax() {
     use std::env;
@@ -160,18 +165,59 @@ fn test_extra_operand() {
         .no_stdout();
 }
 
+#[test]
+fn test_term_matching() {
+    fn check(term_pattern: &str, term: &str, expectation: &str) {
+        let theme = format!(
+            "
+TERM {term_pattern}
+
+.term_matching    00;38;5;61
+"
+        );
+
+        new_ucmd!()
+            .env("TERM", term)
+            .pipe_in(theme)
+            .args(&["-b", "-"])
+            .succeeds()
+            .stdout_is(expectation)
+            .no_stderr();
+    }
+
+    let expectation_if_match = r#"
+LS_COLORS='*.term_matching=00;38;5;61:';
+export LS_COLORS
+"#
+    .trim_start();
+    let expectation_if_no_match = r#"
+LS_COLORS='';
+export LS_COLORS
+"#
+    .trim_start();
+
+    // sanity checks
+    check("matches", "matches", expectation_if_match);
+    check("matches", "no_match", expectation_if_no_match);
+    // character set negation should treat ^ like !
+    check("[!a]_negation", "a_negation", expectation_if_no_match);
+    check("[!a]_negation", "b_negation", expectation_if_match);
+    check("[^a]_negation", "a_negation", expectation_if_no_match);
+    check("[^a]_negation", "b_negation", expectation_if_match);
+}
+
 fn test_helper(file_name: &str, term: &str) {
     new_ucmd!()
         .env("TERM", term)
         .arg("-c")
-        .arg(format!("{}.txt", file_name))
+        .arg(format!("{file_name}.txt"))
         .run()
-        .stdout_is_fixture(format!("{}.csh.expected", file_name));
+        .stdout_is_fixture(format!("{file_name}.csh.expected"));
 
     new_ucmd!()
         .env("TERM", term)
         .arg("-b")
-        .arg(format!("{}.txt", file_name))
+        .arg(format!("{file_name}.txt"))
         .run()
-        .stdout_is_fixture(format!("{}.sh.expected", file_name));
+        .stdout_is_fixture(format!("{file_name}.sh.expected"));
 }
