@@ -5,10 +5,15 @@
 
 // spell-checker:ignore (words) bogusfile emptyfile abcdefghijklmnopqrstuvwxyz abcdefghijklmnopqrstu
 
-use crate::common::util::*;
+use crate::common::util::TestScenario;
 
 static INPUT: &str = "lorem_ipsum.txt";
 
+#[test]
+fn test_invalid_arg() {
+    new_ucmd!().arg("--definitely-invalid").fails().code_is(1);
+}
+
 #[test]
 fn test_stdin_default() {
     new_ucmd!()
@@ -283,21 +288,21 @@ fn test_head_invalid_num() {
     new_ucmd!()
         .args(&["-c", "1024R", "emptyfile.txt"])
         .fails()
-        .stderr_is("head: invalid number of bytes: '1024R'");
+        .stderr_is("head: invalid number of bytes: '1024R'\n");
     new_ucmd!()
         .args(&["-n", "1024R", "emptyfile.txt"])
         .fails()
-        .stderr_is("head: invalid number of lines: '1024R'");
+        .stderr_is("head: invalid number of lines: '1024R'\n");
     #[cfg(not(target_pointer_width = "128"))]
     new_ucmd!()
         .args(&["-c", "1Y", "emptyfile.txt"])
         .fails()
-        .stderr_is("head: invalid number of bytes: '1Y': Value too large for defined data type");
+        .stderr_is("head: invalid number of bytes: '1Y': Value too large for defined data type\n");
     #[cfg(not(target_pointer_width = "128"))]
     new_ucmd!()
         .args(&["-n", "1Y", "emptyfile.txt"])
         .fails()
-        .stderr_is("head: invalid number of lines: '1Y': Value too large for defined data type");
+        .stderr_is("head: invalid number of lines: '1Y': Value too large for defined data type\n");
     #[cfg(target_pointer_width = "32")]
     {
         let sizes = ["1000G", "10T"];
@@ -312,13 +317,13 @@ fn test_head_invalid_num() {
             new_ucmd!()
                 .args(&["-c", size])
                 .fails()
-                .stderr_is("head: out of range integral type conversion attempted: number of bytes is too large");
+                .stderr_is("head: out of range integral type conversion attempted: number of bytes is too large\n");
         }
     }
     new_ucmd!()
         .args(&["-c", "-³"])
         .fails()
-        .stderr_is("head: invalid number of bytes: '³'");
+        .stderr_is("head: invalid number of bytes: '³'\n");
 }
 
 #[test]
