@@ -1,8 +1,8 @@
-// spell-checker:ignore (words) helloworld objdump n'source
+// spell-checker:ignore (words) helloworld nodir objdump n'source
 
-use crate::common::util::*;
+use crate::common::util::{is_ci, TestScenario};
 use filetime::FileTime;
-use rust_users::*;
+use rust_users::{get_effective_gid, get_effective_uid};
 use std::os::unix::fs::PermissionsExt;
 #[cfg(not(any(windows, target_os = "freebsd")))]
 use std::process::Command;
@@ -28,8 +28,8 @@ fn test_install_basic() {
 
     assert!(at.file_exists(file1));
     assert!(at.file_exists(file2));
-    assert!(at.file_exists(&format!("{}/{}", dir, file1)));
-    assert!(at.file_exists(&format!("{}/{}", dir, file2)));
+    assert!(at.file_exists(format!("{dir}/{file1}")));
+    assert!(at.file_exists(format!("{dir}/{file2}")));
 }
 
 #[test]
@@ -76,7 +76,7 @@ fn test_install_unimplemented_arg() {
         .fails()
         .stderr_contains("Unimplemented");
 
-    assert!(!at.file_exists(&format!("{}/{}", dir, file)));
+    assert!(!at.file_exists(format!("{dir}/{file}")));
 }
 
 #[test]
@@ -124,6 +124,37 @@ fn test_install_ancestors_mode_directories() {
     assert_eq!(0o40_200_u32, at.metadata(target_dir).permissions().mode());
 }
 
+#[test]
+fn test_install_ancestors_mode_directories_with_file() {
+    let (at, mut ucmd) = at_and_ucmd!();
+    let ancestor1 = "ancestor1";
+    let ancestor2 = "ancestor1/ancestor2";
+    let target_file = "ancestor1/ancestor2/target_file";
+    let directories_arg = "-D";
+    let mode_arg = "--mode=200";
+    let file = "file";
+    let probe = "probe";
+
+    at.mkdir(probe);
+    let default_perms = at.metadata(probe).permissions().mode();
+
+    at.touch(file);
+
+    ucmd.args(&[mode_arg, directories_arg, file, target_file])
+        .succeeds()
+        .no_stderr();
+
+    assert!(at.dir_exists(ancestor1));
+    assert!(at.dir_exists(ancestor2));
+    assert!(at.file_exists(target_file));
+
+    assert_eq!(default_perms, at.metadata(ancestor1).permissions().mode());
+    assert_eq!(default_perms, at.metadata(ancestor2).permissions().mode());
+
+    // Expected mode only on the target_file.
+    assert_eq!(0o100_200_u32, at.metadata(target_file).permissions().mode());
+}
+
 #[test]
 fn test_install_parent_directories() {
     let (at, mut ucmd) = at_and_ucmd!();
@@ -181,7 +212,7 @@ fn test_install_mode_numeric() {
         .succeeds()
         .no_stderr();
 
-    let dest_file = &format!("{}/{}", dir, file);
+    let dest_file = &format!("{dir}/{file}");
     assert!(at.file_exists(file));
     assert!(at.file_exists(dest_file));
     let permissions = at.metadata(dest_file).permissions();
@@ -192,7 +223,7 @@ fn test_install_mode_numeric() {
 
     scene.ucmd().arg(mode_arg).arg(file).arg(dir2).succeeds();
 
-    let dest_file = &format!("{}/{}", dir2, file);
+    let dest_file = &format!("{dir2}/{file}");
     assert!(at.file_exists(file));
     assert!(at.file_exists(dest_file));
     let permissions = at.metadata(dest_file).permissions();
@@ -210,7 +241,7 @@ fn test_install_mode_symbolic() {
     at.mkdir(dir);
     ucmd.arg(file).arg(dir).arg(mode_arg).succeeds().no_stderr();
 
-    let dest_file = &format!("{}/{}", dir, file);
+    let dest_file = &format!("{dir}/{file}");
     assert!(at.file_exists(file));
     assert!(at.file_exists(dest_file));
     let permissions = at.metadata(dest_file).permissions();
@@ -232,7 +263,7 @@ fn test_install_mode_failing() {
         .fails()
         .stderr_contains("Invalid mode string: invalid digit found in string");
 
-    let dest_file = &format!("{}/{}", dir, file);
+    let dest_file = &format!("{dir}/{file}");
     assert!(at.file_exists(file));
     assert!(!at.file_exists(dest_file));
 }
@@ -278,12 +309,12 @@ fn test_install_target_new_file() {
     at.touch(file);
     at.mkdir(dir);
     ucmd.arg(file)
-        .arg(format!("{}/{}", dir, file))
+        .arg(format!("{dir}/{file}"))
         .succeeds()
         .no_stderr();
 
     assert!(at.file_exists(file));
-    assert!(at.file_exists(&format!("{}/{}", dir, file)));
+    assert!(at.file_exists(format!("{dir}/{file}")));
 }
 
 #[test]
@@ -299,7 +330,7 @@ fn test_install_target_new_file_with_group() {
         .arg(file)
         .arg("--group")
         .arg(gid.to_string())
-        .arg(format!("{}/{}", dir, file))
+        .arg(format!("{dir}/{file}"))
         .run();
 
     if is_ci() && result.stderr_str().contains("no such group:") {
@@ -310,7 +341,7 @@ fn test_install_target_new_file_with_group() {
 
     result.success();
     assert!(at.file_exists(file));
-    assert!(at.file_exists(&format!("{}/{}", dir, file)));
+    assert!(at.file_exists(format!("{dir}/{file}")));
 }
 
 #[test]
@@ -326,7 +357,7 @@ fn test_install_target_new_file_with_owner() {
         .arg(file)
         .arg("--owner")
         .arg(uid.to_string())
-        .arg(format!("{}/{}", dir, file))
+        .arg(format!("{dir}/{file}"))
         .run();
 
     if is_ci() && result.stderr_str().contains("no such user:") {
@@ -337,7 +368,7 @@ fn test_install_target_new_file_with_owner() {
 
     result.success();
     assert!(at.file_exists(file));
-    assert!(at.file_exists(&format!("{}/{}", dir, file)));
+    assert!(at.file_exists(format!("{dir}/{file}")));
 }
 
 #[test]
@@ -350,9 +381,9 @@ fn test_install_target_new_file_failing_nonexistent_parent() {
     at.touch(file1);
 
     ucmd.arg(file1)
-        .arg(format!("{}/{}", dir, file2))
+        .arg(format!("{dir}/{file2}"))
         .fails()
-        .stderr_contains(&"No such file or directory");
+        .stderr_contains("No such file or directory");
 }
 
 #[test]
@@ -416,29 +447,53 @@ fn test_install_nested_paths_copy_file() {
 
     at.mkdir(dir1);
     at.mkdir(dir2);
-    at.touch(&format!("{}/{}", dir1, file1));
+    at.touch(format!("{dir1}/{file1}"));
 
-    ucmd.arg(format!("{}/{}", dir1, file1))
+    ucmd.arg(format!("{dir1}/{file1}"))
         .arg(dir2)
         .succeeds()
         .no_stderr();
-    assert!(at.file_exists(&format!("{}/{}", dir2, file1)));
+    assert!(at.file_exists(format!("{dir2}/{file1}")));
 }
 
 #[test]
 fn test_install_failing_omitting_directory() {
-    let (at, mut ucmd) = at_and_ucmd!();
-    let file1 = "source_file";
-    let dir1 = "source_dir";
-    let dir2 = "target_dir";
+    let scene = TestScenario::new(util_name!());
+    let at = &scene.fixtures;
+    let file1 = "file1";
+    let dir1 = "dir1";
+    let no_dir2 = "no-dir2";
+    let dir3 = "dir3";
 
     at.mkdir(dir1);
-    at.mkdir(dir2);
+    at.mkdir(dir3);
     at.touch(file1);
 
-    ucmd.arg(dir1)
+    // GNU install checks for existing target dir first before checking on source params
+    scene
+        .ucmd()
         .arg(file1)
-        .arg(dir2)
+        .arg(dir1)
+        .arg(no_dir2)
+        .fails()
+        .stderr_contains("is not a directory");
+
+    // file1 will be copied before install fails on dir1
+    scene
+        .ucmd()
+        .arg(file1)
+        .arg(dir1)
+        .arg(dir3)
+        .fails()
+        .code_is(1)
+        .stderr_contains("omitting directory");
+    assert!(at.file_exists(format!("{dir3}/{file1}")));
+
+    // install also fails, when only one source param is given
+    scene
+        .ucmd()
+        .arg(dir1)
+        .arg(dir3)
         .fails()
         .code_is(1)
         .stderr_contains("omitting directory");
@@ -668,6 +723,106 @@ fn test_install_creating_leading_dirs() {
         .arg(at.plus(target))
         .succeeds()
         .no_stderr();
+
+    assert!(at.file_exists(target));
+}
+
+#[test]
+fn test_install_creating_leading_dirs_verbose() {
+    let scene = TestScenario::new(util_name!());
+    let at = &scene.fixtures;
+
+    let source = "create_leading_test_file";
+    let target = "dir1/no-dir2/no-dir3/test_file";
+
+    at.touch(source);
+    at.mkdir("dir1");
+
+    let creating_dir1 = regex::Regex::new("(?m)^install: creating directory.*dir1'$").unwrap();
+    let creating_nodir23 =
+        regex::Regex::new(r"(?m)^install: creating directory.*no-dir[23]'$").unwrap();
+
+    scene
+        .ucmd()
+        .arg("-Dv")
+        .arg(source)
+        .arg(at.plus(target))
+        .succeeds()
+        .stdout_matches(&creating_nodir23)
+        .stdout_does_not_match(&creating_dir1)
+        .no_stderr();
+
+    assert!(at.file_exists(target));
+}
+
+#[test]
+fn test_install_creating_leading_dirs_with_single_source_and_target_dir() {
+    let scene = TestScenario::new(util_name!());
+    let at = &scene.fixtures;
+
+    let source1 = "source_file_1";
+    let target_dir = "missing_target_dir/";
+
+    at.touch(source1);
+
+    // installing a single file into a missing directory will fail, when -D is used w/o -t parameter
+    scene
+        .ucmd()
+        .arg("-D")
+        .arg(source1)
+        .arg(at.plus(target_dir))
+        .fails()
+        .stderr_contains("missing_target_dir/' is not a directory");
+
+    assert!(!at.dir_exists(target_dir));
+
+    scene
+        .ucmd()
+        .arg("-D")
+        .arg(source1)
+        .arg("-t")
+        .arg(at.plus(target_dir))
+        .succeeds()
+        .no_stderr();
+
+    assert!(at.file_exists(format!("{target_dir}/{source1}")));
+}
+
+#[test]
+fn test_install_creating_leading_dirs_with_multiple_sources_and_target_dir() {
+    let scene = TestScenario::new(util_name!());
+    let at = &scene.fixtures;
+
+    let source1 = "source_file_1";
+    let source2 = "source_file_2";
+    let target_dir = "missing_target_dir";
+
+    at.touch(source1);
+    at.touch(source2);
+
+    // installing multiple files into a missing directory will fail, when -D is used w/o -t parameter
+    scene
+        .ucmd()
+        .arg("-D")
+        .arg(source1)
+        .arg(source2)
+        .arg(at.plus(target_dir))
+        .fails()
+        .stderr_contains("missing_target_dir' is not a directory");
+
+    assert!(!at.dir_exists(target_dir));
+
+    scene
+        .ucmd()
+        .arg("-D")
+        .arg(source1)
+        .arg(source2)
+        .arg("-t")
+        .arg(at.plus(target_dir))
+        .succeeds()
+        .no_stderr();
+
+    assert!(at.dir_exists(target_dir));
 }
 
 #[test]
@@ -702,14 +857,14 @@ fn test_install_dir() {
     at.mkdir(dir);
     ucmd.arg(file1)
         .arg(file2)
-        .arg(&format!("--target-directory={}", dir))
+        .arg(&format!("--target-directory={dir}"))
         .succeeds()
         .no_stderr();
 
     assert!(at.file_exists(file1));
     assert!(at.file_exists(file2));
-    assert!(at.file_exists(&format!("{}/{}", dir, file1)));
-    assert!(at.file_exists(&format!("{}/{}", dir, file2)));
+    assert!(at.file_exists(format!("{dir}/{file1}")));
+    assert!(at.file_exists(format!("{dir}/{file2}")));
 }
 //
 // test backup functionality
@@ -733,7 +888,7 @@ fn test_install_backup_short_no_args_files() {
 
     assert!(at.file_exists(file_a));
     assert!(at.file_exists(file_b));
-    assert!(at.file_exists(&format!("{}~", file_b)));
+    assert!(at.file_exists(format!("{file_b}~")));
 }
 
 #[test]
@@ -743,7 +898,7 @@ fn test_install_backup_short_no_args_file_to_dir() {
 
     let file = "test_install_simple_backup_file_a";
     let dest_dir = "test_install_dest/";
-    let expect = format!("{}{}", dest_dir, file);
+    let expect = format!("{dest_dir}{file}");
 
     at.touch(file);
     at.mkdir(dest_dir);
@@ -758,7 +913,7 @@ fn test_install_backup_short_no_args_file_to_dir() {
 
     assert!(at.file_exists(file));
     assert!(at.file_exists(&expect));
-    assert!(at.file_exists(&format!("{}~", expect)));
+    assert!(at.file_exists(format!("{expect}~")));
 }
 
 // Long --backup option is tested separately as it requires a slightly different
@@ -783,7 +938,7 @@ fn test_install_backup_long_no_args_files() {
 
     assert!(at.file_exists(file_a));
     assert!(at.file_exists(file_b));
-    assert!(at.file_exists(&format!("{}~", file_b)));
+    assert!(at.file_exists(format!("{file_b}~")));
 }
 
 #[test]
@@ -793,7 +948,7 @@ fn test_install_backup_long_no_args_file_to_dir() {
 
     let file = "test_install_simple_backup_file_a";
     let dest_dir = "test_install_dest/";
-    let expect = format!("{}{}", dest_dir, file);
+    let expect = format!("{dest_dir}{file}");
 
     at.touch(file);
     at.mkdir(dest_dir);
@@ -808,7 +963,7 @@ fn test_install_backup_long_no_args_file_to_dir() {
 
     assert!(at.file_exists(file));
     assert!(at.file_exists(&expect));
-    assert!(at.file_exists(&format!("{}~", expect)));
+    assert!(at.file_exists(format!("{expect}~")));
 }
 
 #[test]
@@ -825,7 +980,7 @@ fn test_install_backup_short_custom_suffix() {
     scene
         .ucmd()
         .arg("-b")
-        .arg(format!("--suffix={}", suffix))
+        .arg(format!("--suffix={suffix}"))
         .arg(file_a)
         .arg(file_b)
         .succeeds()
@@ -833,7 +988,7 @@ fn test_install_backup_short_custom_suffix() {
 
     assert!(at.file_exists(file_a));
     assert!(at.file_exists(file_b));
-    assert!(at.file_exists(&format!("{}{}", file_b, suffix)));
+    assert!(at.file_exists(format!("{file_b}{suffix}")));
 }
 
 #[test]
@@ -850,7 +1005,7 @@ fn test_install_backup_short_custom_suffix_hyphen_value() {
     scene
         .ucmd()
         .arg("-b")
-        .arg(format!("--suffix={}", suffix))
+        .arg(format!("--suffix={suffix}"))
         .arg(file_a)
         .arg(file_b)
         .succeeds()
@@ -858,7 +1013,7 @@ fn test_install_backup_short_custom_suffix_hyphen_value() {
 
     assert!(at.file_exists(file_a));
     assert!(at.file_exists(file_b));
-    assert!(at.file_exists(&format!("{}{}", file_b, suffix)));
+    assert!(at.file_exists(format!("{file_b}{suffix}")));
 }
 
 #[test]
@@ -883,7 +1038,7 @@ fn test_install_backup_custom_suffix_via_env() {
 
     assert!(at.file_exists(file_a));
     assert!(at.file_exists(file_b));
-    assert!(at.file_exists(&format!("{}{}", file_b, suffix)));
+    assert!(at.file_exists(format!("{file_b}{suffix}")));
 }
 
 #[test]
@@ -906,7 +1061,7 @@ fn test_install_backup_numbered_with_t() {
 
     assert!(at.file_exists(file_a));
     assert!(at.file_exists(file_b));
-    assert!(at.file_exists(&format!("{}.~1~", file_b)));
+    assert!(at.file_exists(format!("{file_b}.~1~")));
 }
 
 #[test]
@@ -929,7 +1084,7 @@ fn test_install_backup_numbered_with_numbered() {
 
     assert!(at.file_exists(file_a));
     assert!(at.file_exists(file_b));
-    assert!(at.file_exists(&format!("{}.~1~", file_b)));
+    assert!(at.file_exists(format!("{file_b}.~1~")));
 }
 
 #[test]
@@ -952,7 +1107,7 @@ fn test_install_backup_existing() {
 
     assert!(at.file_exists(file_a));
     assert!(at.file_exists(file_b));
-    assert!(at.file_exists(&format!("{}~", file_b)));
+    assert!(at.file_exists(format!("{file_b}~")));
 }
 
 #[test]
@@ -975,7 +1130,7 @@ fn test_install_backup_nil() {
 
     assert!(at.file_exists(file_a));
     assert!(at.file_exists(file_b));
-    assert!(at.file_exists(&format!("{}~", file_b)));
+    assert!(at.file_exists(format!("{file_b}~")));
 }
 
 #[test]
@@ -1001,7 +1156,7 @@ fn test_install_backup_numbered_if_existing_backup_existing() {
     assert!(at.file_exists(file_a));
     assert!(at.file_exists(file_b));
     assert!(at.file_exists(file_b_backup));
-    assert!(at.file_exists(&format!("{}.~2~", file_b)));
+    assert!(at.file_exists(format!("{file_b}.~2~")));
 }
 
 #[test]
@@ -1027,7 +1182,7 @@ fn test_install_backup_numbered_if_existing_backup_nil() {
     assert!(at.file_exists(file_a));
     assert!(at.file_exists(file_b));
     assert!(at.file_exists(file_b_backup));
-    assert!(at.file_exists(&format!("{}.~2~", file_b)));
+    assert!(at.file_exists(format!("{file_b}.~2~")));
 }
 
 #[test]
@@ -1050,7 +1205,7 @@ fn test_install_backup_simple() {
 
     assert!(at.file_exists(file_a));
     assert!(at.file_exists(file_b));
-    assert!(at.file_exists(&format!("{}~", file_b)));
+    assert!(at.file_exists(format!("{file_b}~")));
 }
 
 #[test]
@@ -1073,7 +1228,7 @@ fn test_install_backup_never() {
 
     assert!(at.file_exists(file_a));
     assert!(at.file_exists(file_b));
-    assert!(at.file_exists(&format!("{}~", file_b)));
+    assert!(at.file_exists(format!("{file_b}~")));
 }
 
 #[test]
@@ -1096,7 +1251,7 @@ fn test_install_backup_none() {
 
     assert!(at.file_exists(file_a));
     assert!(at.file_exists(file_b));
-    assert!(!at.file_exists(&format!("{}~", file_b)));
+    assert!(!at.file_exists(format!("{file_b}~")));
 }
 
 #[test]
@@ -1119,17 +1274,29 @@ fn test_install_backup_off() {
 
     assert!(at.file_exists(file_a));
     assert!(at.file_exists(file_b));
-    assert!(!at.file_exists(&format!("{}~", file_b)));
+    assert!(!at.file_exists(format!("{file_b}~")));
 }
 
 #[test]
 fn test_install_missing_arguments() {
     let scene = TestScenario::new(util_name!());
+    let at = &scene.fixtures;
+
+    let no_target_dir = "no-target_dir";
 
     scene
         .ucmd()
         .fails()
-        .stderr_contains("install: missing file operand");
+        .code_is(1)
+        .usage_error("missing file operand");
+
+    scene
+        .ucmd()
+        .arg("-D")
+        .arg(format!("-t {no_target_dir}"))
+        .fails()
+        .usage_error("missing file operand");
+    assert!(!at.dir_exists(no_target_dir));
 }
 
 #[test]
@@ -1138,12 +1305,25 @@ fn test_install_missing_destination() {
     let at = &scene.fixtures;
 
     let file_1 = "source_file1";
+    let dir_1 = "source_dir1";
 
     at.touch(file_1);
-    scene.ucmd().arg(file_1).fails().stderr_contains(format!(
-        "install: missing destination file operand after '{}'",
-        file_1
-    ));
+    at.mkdir(dir_1);
+
+    // will fail and also print some info on correct usage
+    scene
+        .ucmd()
+        .arg(file_1)
+        .fails()
+        .usage_error(format!("missing destination file operand after '{file_1}'"));
+
+    // GNU's install will check for correct num of arguments and then fail
+    // and it does not recognize, that the source is not a file but a directory.
+    scene
+        .ucmd()
+        .arg(dir_1)
+        .fails()
+        .usage_error(format!("missing destination file operand after '{dir_1}'"));
 }
 
 #[test]
@@ -1220,6 +1400,100 @@ fn test_install_dir_req_verbose() {
         .stdout_contains("install: creating directory 'sub5/a'\ninstall: creating directory 'sub5/a/b'\ninstall: creating directory 'sub5/a/b/c'\n'source_file1' -> 'sub5/a/b/c/file'");
 }
 
+#[test]
+fn test_install_chown_file_invalid() {
+    let scene = TestScenario::new(util_name!());
+    let at = &scene.fixtures;
+
+    let file_1 = "source_file1";
+    at.touch(file_1);
+
+    scene
+        .ucmd()
+        .arg("-o")
+        .arg("test_invalid_user")
+        .arg(file_1)
+        .arg("target_file1")
+        .fails()
+        .stderr_contains("install: invalid user: 'test_invalid_user'");
+
+    scene
+        .ucmd()
+        .arg("-g")
+        .arg("test_invalid_group")
+        .arg(file_1)
+        .arg("target_file1")
+        .fails()
+        .stderr_contains("install: invalid group: 'test_invalid_group'");
+
+    scene
+        .ucmd()
+        .arg("-o")
+        .arg("test_invalid_user")
+        .arg("-g")
+        .arg("test_invalid_group")
+        .arg(file_1)
+        .arg("target_file1")
+        .fails()
+        .stderr_contains("install: invalid user: 'test_invalid_user'");
+
+    scene
+        .ucmd()
+        .arg("-g")
+        .arg("test_invalid_group")
+        .arg("-o")
+        .arg("test_invalid_user")
+        .arg(file_1)
+        .arg("target_file1")
+        .fails()
+        .stderr_contains("install: invalid user: 'test_invalid_user'");
+}
+
+#[test]
+fn test_install_chown_directory_invalid() {
+    let scene = TestScenario::new(util_name!());
+
+    scene
+        .ucmd()
+        .arg("-o")
+        .arg("test_invalid_user")
+        .arg("-d")
+        .arg("dir1/dir2")
+        .fails()
+        .stderr_contains("install: invalid user: 'test_invalid_user'");
+
+    scene
+        .ucmd()
+        .arg("-g")
+        .arg("test_invalid_group")
+        .arg("-d")
+        .arg("dir1/dir2")
+        .fails()
+        .stderr_contains("install: invalid group: 'test_invalid_group'");
+
+    scene
+        .ucmd()
+        .arg("-o")
+        .arg("test_invalid_user")
+        .arg("-g")
+        .arg("test_invalid_group")
+        .arg("-d")
+        .arg("dir1/dir2")
+        .fails()
+        .stderr_contains("install: invalid user: 'test_invalid_user'");
+
+    scene
+        .ucmd()
+        .arg("-g")
+        .arg("test_invalid_group")
+        .arg("-o")
+        .arg("test_invalid_user")
+        .arg("-d")
+        .arg("dir1/dir2")
+        .fails()
+        .stderr_contains("install: invalid user: 'test_invalid_user'");
+}
+
 #[test]
 fn test_install_compare_option() {
     let scene = TestScenario::new(util_name!());
@@ -1231,7 +1505,7 @@ fn test_install_compare_option() {
         .ucmd()
         .args(&["-Cv", first, second])
         .succeeds()
-        .stdout_contains(format!("'{}' -> '{}'", first, second));
+        .stdout_contains(format!("'{first}' -> '{second}'"));
     scene
         .ucmd()
         .args(&["-Cv", first, second])
@@ -1241,12 +1515,12 @@ fn test_install_compare_option() {
         .ucmd()
         .args(&["-Cv", "-m0644", first, second])
         .succeeds()
-        .stdout_contains(format!("removed '{}'\n'{}' -> '{}'", second, first, second));
+        .stdout_contains(format!("removed '{second}'\n'{first}' -> '{second}'"));
     scene
         .ucmd()
         .args(&["-Cv", first, second])
         .succeeds()
-        .stdout_contains(format!("removed '{}'\n'{}' -> '{}'", second, first, second));
+        .stdout_contains(format!("removed '{second}'\n'{first}' -> '{second}'"));
     scene
         .ucmd()
         .args(&["-C", "--preserve-timestamps", first, second])
