@@ -1,13 +1,18 @@
-use crate::common::util::*;
+use crate::common::util::{is_ci, TestScenario};
 use std::env;
 
+#[test]
+fn test_invalid_arg() {
+    new_ucmd!().arg("--definitely-invalid").fails().code_is(1);
+}
+
 #[test]
 fn test_normal() {
     let result = new_ucmd!().run();
     println!("env::var(CI).is_ok() = {}", env::var("CI").is_ok());
 
     for (key, value) in env::vars() {
-        println!("{}: {}", key, value);
+        println!("{key}: {value}");
     }
     if (is_ci() || uucore::os::is_wsl_1()) && result.stderr_str().contains("no login name") {
         // ToDO: investigate WSL failure
