@@ -5,11 +5,11 @@ extern crate libc;
 extern crate regex;
 #[cfg(not(windows))]
 extern crate tempfile;
-#[cfg(unix)]
-extern crate unix_socket;
 
 use self::regex::Regex;
-use crate::common::util::*;
+#[cfg(feature = "feat_selinux")]
+use crate::common::util::expected_result;
+use crate::common::util::TestScenario;
 #[cfg(all(unix, feature = "chmod"))]
 use nix::unistd::{close, dup};
 use std::collections::HashMap;
@@ -92,16 +92,16 @@ fn test_ls_allocation_size() {
         // fill empty file with zeros
         scene
             .ccmd("dd")
-            .arg("--if=/dev/zero")
-            .arg("--of=some-dir1/zero-file")
+            .arg("if=/dev/zero")
+            .arg("of=some-dir1/zero-file")
             .arg("bs=1024")
             .arg("count=4096")
             .succeeds();
 
         scene
             .ccmd("dd")
-            .arg("--if=/dev/zero")
-            .arg("--of=irregular-file")
+            .arg("if=/dev/zero")
+            .arg("of=irregular-file")
             .arg("bs=1")
             .arg("count=777")
             .succeeds();
@@ -387,6 +387,7 @@ fn test_ls_io_errors() {
         .arg("-1")
         .arg("some-dir1")
         .fails()
+        .code_is(2)
         .stderr_contains("cannot open directory")
         .stderr_contains("Permission denied");
 
@@ -395,6 +396,7 @@ fn test_ls_io_errors() {
         .arg("-Li")
         .arg("some-dir2")
         .fails()
+        .code_is(1)
         .stderr_contains("cannot access")
         .stderr_contains("No such file or directory")
         .stdout_contains(if cfg!(windows) { "dangle" } else { "? dangle" });
@@ -410,6 +412,7 @@ fn test_ls_io_errors() {
         .arg("-laR")
         .arg("some-dir3")
         .fails()
+        .code_is(1)
         .stderr_contains("some-dir4")
         .stderr_contains("cannot open directory")
         .stderr_contains("Permission denied")
@@ -434,44 +437,43 @@ fn test_ls_io_errors() {
 
         // on the mac and in certain Linux containers bad fds are typed as dirs,
         // however sometimes bad fds are typed as links and directory entry on links won't fail
-        if PathBuf::from(format!("/dev/fd/{fd}", fd = fd2)).is_dir() {
+        if PathBuf::from(format!("/dev/fd/{fd2}")).is_dir() {
             scene
                 .ucmd()
                 .arg("-alR")
-                .arg(format!("/dev/fd/{fd}", fd = fd2))
+                .arg(format!("/dev/fd/{fd2}"))
                 .fails()
                 .stderr_contains(format!(
-                    "cannot open directory '/dev/fd/{fd}': Bad file descriptor",
-                    fd = fd2
+                    "cannot open directory '/dev/fd/{fd2}': Bad file descriptor"
                 ))
-                .stdout_does_not_contain(format!("{fd}:\n", fd = fd2));
+                .stdout_does_not_contain(format!("{fd2}:\n"));
 
             scene
                 .ucmd()
                 .arg("-RiL")
-                .arg(format!("/dev/fd/{fd}", fd = fd2))
+                .arg(format!("/dev/fd/{fd2}"))
                 .fails()
-                .stderr_contains(format!("cannot open directory '/dev/fd/{fd}': Bad file descriptor", fd = fd2))
+                .stderr_contains(format!("cannot open directory '/dev/fd/{fd2}': Bad file descriptor"))
                 // don't double print bad fd errors
-                .stderr_does_not_contain(format!("ls: cannot open directory '/dev/fd/{fd}': Bad file descriptor\nls: cannot open directory '/dev/fd/{fd}': Bad file descriptor", fd = fd2));
+                .stderr_does_not_contain(format!("ls: cannot open directory '/dev/fd/{fd2}': Bad file descriptor\nls: cannot open directory '/dev/fd/{fd2}': Bad file descriptor"));
         } else {
             scene
                 .ucmd()
                 .arg("-alR")
-                .arg(format!("/dev/fd/{fd}", fd = fd2))
+                .arg(format!("/dev/fd/{fd2}"))
                 .succeeds();
 
             scene
                 .ucmd()
                 .arg("-RiL")
-                .arg(format!("/dev/fd/{fd}", fd = fd2))
+                .arg(format!("/dev/fd/{fd2}"))
                 .succeeds();
         }
 
         scene
             .ucmd()
             .arg("-alL")
-            .arg(format!("/dev/fd/{fd}", fd = fd2))
+            .arg(format!("/dev/fd/{fd2}"))
             .succeeds();
 
         let _ = close(fd2);
@@ -560,8 +562,8 @@ fn test_ls_a() {
         .arg("-a")
         .arg("-1")
         .succeeds()
-        .stdout_contains(&".test-1")
-        .stdout_contains(&"..")
+        .stdout_contains(".test-1")
+        .stdout_contains("..")
         .stdout_matches(&re_pwd);
 
     scene
@@ -589,8 +591,8 @@ fn test_ls_a() {
         .arg("-1")
         .arg("some-dir")
         .succeeds()
-        .stdout_contains(&".test-2")
-        .stdout_contains(&"..")
+        .stdout_contains(".test-2")
+        .stdout_contains("..")
         .no_stderr()
         .stdout_matches(&re_pwd);
 
@@ -609,10 +611,10 @@ fn test_ls_a() {
 fn test_ls_width() {
     let scene = TestScenario::new(util_name!());
     let at = &scene.fixtures;
-    at.touch(&at.plus_as_string("test-width-1"));
-    at.touch(&at.plus_as_string("test-width-2"));
-    at.touch(&at.plus_as_string("test-width-3"));
-    at.touch(&at.plus_as_string("test-width-4"));
+    at.touch(at.plus_as_string("test-width-1"));
+    at.touch(at.plus_as_string("test-width-2"));
+    at.touch(at.plus_as_string("test-width-3"));
+    at.touch(at.plus_as_string("test-width-4"));
 
     for option in [
         "-w 100",
@@ -684,7 +686,7 @@ fn test_ls_width() {
             .args(&option.split(' ').collect::<Vec<_>>())
             .arg("-C")
             .fails()
-            .stderr_only("ls: invalid line width: '1a'");
+            .stderr_only("ls: invalid line width: '1a'\n");
     }
 }
 
@@ -692,10 +694,10 @@ fn test_ls_width() {
 fn test_ls_columns() {
     let scene = TestScenario::new(util_name!());
     let at = &scene.fixtures;
-    at.touch(&at.plus_as_string("test-columns-1"));
-    at.touch(&at.plus_as_string("test-columns-2"));
-    at.touch(&at.plus_as_string("test-columns-3"));
-    at.touch(&at.plus_as_string("test-columns-4"));
+    at.touch(at.plus_as_string("test-columns-1"));
+    at.touch(at.plus_as_string("test-columns-2"));
+    at.touch(at.plus_as_string("test-columns-3"));
+    at.touch(at.plus_as_string("test-columns-4"));
 
     // Columns is the default
     let result = scene.ucmd().succeeds();
@@ -735,7 +737,7 @@ fn test_ls_columns() {
             .arg("-C")
             .succeeds()
             .stdout_is("test-columns-1  test-columns-2  test-columns-3  test-columns-4\n")
-            .stderr_is("ls: ignoring invalid width in environment variable COLUMNS: 'garbage'");
+            .stderr_is("ls: ignoring invalid width in environment variable COLUMNS: 'garbage'\n");
     }
     scene
         .ucmd()
@@ -753,10 +755,10 @@ fn test_ls_columns() {
 fn test_ls_across() {
     let scene = TestScenario::new(util_name!());
     let at = &scene.fixtures;
-    at.touch(&at.plus_as_string("test-across-1"));
-    at.touch(&at.plus_as_string("test-across-2"));
-    at.touch(&at.plus_as_string("test-across-3"));
-    at.touch(&at.plus_as_string("test-across-4"));
+    at.touch(at.plus_as_string("test-across-1"));
+    at.touch(at.plus_as_string("test-across-2"));
+    at.touch(at.plus_as_string("test-across-3"));
+    at.touch(at.plus_as_string("test-across-4"));
 
     for option in ACROSS_ARGS {
         let result = scene.ucmd().arg(option).succeeds();
@@ -781,10 +783,10 @@ fn test_ls_across() {
 fn test_ls_commas() {
     let scene = TestScenario::new(util_name!());
     let at = &scene.fixtures;
-    at.touch(&at.plus_as_string("test-commas-1"));
-    at.touch(&at.plus_as_string("test-commas-2"));
-    at.touch(&at.plus_as_string("test-commas-3"));
-    at.touch(&at.plus_as_string("test-commas-4"));
+    at.touch(at.plus_as_string("test-commas-1"));
+    at.touch(at.plus_as_string("test-commas-2"));
+    at.touch(at.plus_as_string("test-commas-3"));
+    at.touch(at.plus_as_string("test-commas-4"));
 
     for option in COMMA_ARGS {
         let result = scene.ucmd().arg(option).succeeds();
@@ -814,8 +816,8 @@ fn test_ls_zero() {
     let scene = TestScenario::new(util_name!());
     let at = &scene.fixtures;
     at.mkdir("0-test-zero");
-    at.touch(&at.plus_as_string("2-test-zero"));
-    at.touch(&at.plus_as_string("3-test-zero"));
+    at.touch(at.plus_as_string("2-test-zero"));
+    at.touch(at.plus_as_string("3-test-zero"));
 
     let ignored_opts = [
         "--quoting-style=c",
@@ -870,7 +872,7 @@ fn test_ls_zero() {
 
     #[cfg(unix)]
     {
-        at.touch(&at.plus_as_string("1\ntest-zero"));
+        at.touch(at.plus_as_string("1\ntest-zero"));
 
         let ignored_opts = [
             "--quoting-style=c",
@@ -933,9 +935,9 @@ fn test_ls_zero() {
 fn test_ls_commas_trailing() {
     let scene = TestScenario::new(util_name!());
     let at = &scene.fixtures;
-    at.touch(&at.plus_as_string("test-commas-trailing-2"));
+    at.touch(at.plus_as_string("test-commas-trailing-2"));
 
-    at.touch(&at.plus_as_string("test-commas-trailing-1"));
+    at.touch(at.plus_as_string("test-commas-trailing-1"));
     at.append(
         "test-commas-trailing-1",
         &(0..2000)
@@ -950,14 +952,14 @@ fn test_ls_commas_trailing() {
         .arg("./test-commas-trailing-1")
         .arg("./test-commas-trailing-2")
         .succeeds()
-        .stdout_matches(&Regex::new(r"\S$").unwrap()); // matches if there is no whitespace at the end of stdout.
+        .stdout_matches(&Regex::new(r"\S\n$").unwrap());
 }
 
 #[test]
 fn test_ls_long() {
     let scene = TestScenario::new(util_name!());
     let at = &scene.fixtures;
-    at.touch(&at.plus_as_string("test-long"));
+    at.touch(at.plus_as_string("test-long"));
 
     for arg in LONG_ARGS {
         let result = scene.ucmd().arg(arg).arg("test-long").succeeds();
@@ -965,7 +967,7 @@ fn test_ls_long() {
         result.stdout_matches(&Regex::new(r"[-bcCdDlMnpPsStTx?]([r-][w-][xt-]){3}.*").unwrap());
 
         #[cfg(windows)]
-        result.stdout_contains("---------- 1 somebody somegroup");
+        result.stdout_matches(&Regex::new(r"[-dl](r[w-]x){3}.*").unwrap());
     }
 }
 
@@ -975,7 +977,7 @@ fn test_ls_long_format() {
     let scene = TestScenario::new(util_name!());
     let at = &scene.fixtures;
     at.mkdir(&at.plus_as_string("test-long-dir"));
-    at.touch(&at.plus_as_string("test-long-dir/test-long-file"));
+    at.touch(at.plus_as_string("test-long-dir/test-long-file"));
     at.mkdir(&at.plus_as_string("test-long-dir/test-long-dir"));
 
     for arg in LONG_ARGS {
@@ -1173,7 +1175,7 @@ fn test_ls_long_symlink_color() {
                 captures.get(1).unwrap().as_str().to_string(),
                 captures.get(2).unwrap().as_str().to_string(),
             ),
-            None => ("".to_string(), input.to_string()),
+            None => (String::new(), input.to_string()),
         }
     }
 
@@ -1231,9 +1233,9 @@ fn test_ls_long_symlink_color() {
 fn test_ls_long_total_size() {
     let scene = TestScenario::new(util_name!());
     let at = &scene.fixtures;
-    at.touch(&at.plus_as_string("test-long"));
+    at.touch(at.plus_as_string("test-long"));
     at.append("test-long", "1");
-    at.touch(&at.plus_as_string("test-long2"));
+    at.touch(at.plus_as_string("test-long2"));
     at.append("test-long2", "2");
 
     let expected_prints: HashMap<_, _> = if cfg!(unix) {
@@ -1275,12 +1277,12 @@ fn test_ls_long_total_size() {
 fn test_ls_long_formats() {
     let scene = TestScenario::new(util_name!());
     let at = &scene.fixtures;
-    at.touch(&at.plus_as_string("test-long-formats"));
+    at.touch(at.plus_as_string("test-long-formats"));
 
     // Zero or one "." for indicating a file with security context
 
     // Regex for three names, so all of author, group and owner
-    let re_three = Regex::new(r"[xrw-]{9}\.? \d ([-0-9_a-z]+ ){3}0").unwrap();
+    let re_three = Regex::new(r"[xrw-]{9}\.? \d ([-0-9_a-z_A-Z]+ ){3}0").unwrap();
 
     #[cfg(unix)]
     let re_three_num = Regex::new(r"[xrw-]{9}\.? \d (\d+ ){3}0").unwrap();
@@ -1289,13 +1291,13 @@ fn test_ls_long_formats() {
     // - group and owner
     // - author and owner
     // - author and group
-    let re_two = Regex::new(r"[xrw-]{9}\.? \d ([-0-9_a-z]+ ){2}0").unwrap();
+    let re_two = Regex::new(r"[xrw-]{9}\.? \d ([-0-9_a-z_A-Z]+ ){2}0").unwrap();
 
     #[cfg(unix)]
     let re_two_num = Regex::new(r"[xrw-]{9}\.? \d (\d+ ){2}0").unwrap();
 
     // Regex for one name: author, group or owner
-    let re_one = Regex::new(r"[xrw-]{9}\.? \d [-0-9_a-z]+ 0").unwrap();
+    let re_one = Regex::new(r"[xrw-]{9}\.? \d [-0-9_a-z_A-Z]+ 0").unwrap();
 
     #[cfg(unix)]
     let re_one_num = Regex::new(r"[xrw-]{9}\.? \d \d+ 0").unwrap();
@@ -1422,8 +1424,8 @@ fn test_ls_long_formats() {
 fn test_ls_oneline() {
     let scene = TestScenario::new(util_name!());
     let at = &scene.fixtures;
-    at.touch(&at.plus_as_string("test-oneline-1"));
-    at.touch(&at.plus_as_string("test-oneline-2"));
+    at.touch(at.plus_as_string("test-oneline-1"));
+    at.touch(at.plus_as_string("test-oneline-2"));
 
     // Bit of a weird situation: in the tests oneline and columns have the same output,
     // except on Windows.
@@ -1443,7 +1445,7 @@ fn test_ls_deref() {
     let path_regexp = r"(.*)test-long.link -> (.*)test-long(.*)";
     let re = Regex::new(path_regexp).unwrap();
 
-    at.touch(&at.plus_as_string("test-long"));
+    at.touch(at.plus_as_string("test-long"));
     at.symlink_file("test-long", "test-long.link");
     assert!(at.is_symlink("test-long.link"));
 
@@ -1652,6 +1654,7 @@ fn test_ls_styles() {
     let re_iso = Regex::new(r"[a-z-]* \d* \w* \w* \d* \d{2}-\d{2} \d{2}:\d{2} test\n").unwrap();
     let re_locale =
         Regex::new(r"[a-z-]* \d* \w* \w* \d* [A-Z][a-z]{2} ( |\d)\d \d{2}:\d{2} test\n").unwrap();
+    let re_custom_format = Regex::new(r"[a-z-]* \d* \w* \w* \d* \d{4}__\d{2} test\n").unwrap();
 
     //full-iso
     let result = scene
@@ -1674,6 +1677,17 @@ fn test_ls_styles() {
     let result = scene.ucmd().arg("-l").arg("--time-style=locale").succeeds();
     assert!(re_locale.is_match(result.stdout_str()));
 
+    //+FORMAT
+    let result = scene
+        .ucmd()
+        .arg("-l")
+        .arg("--time-style=+%Y__%M")
+        .succeeds();
+    assert!(re_custom_format.is_match(result.stdout_str()));
+
+    // Also fails due to not having full clap support for time_styles
+    scene.ucmd().arg("-l").arg("-time-style=invalid").fails();
+
     //Overwrite options tests
     let result = scene
         .ucmd()
@@ -1796,8 +1810,8 @@ fn test_ls_files_dirs() {
     at.mkdir("a/b");
     at.mkdir("a/b/c");
     at.mkdir("z");
-    at.touch(&at.plus_as_string("a/a"));
-    at.touch(&at.plus_as_string("a/b/b"));
+    at.touch(at.plus_as_string("a/a"));
+    at.touch(at.plus_as_string("a/b/b"));
 
     scene.ucmd().arg("a").succeeds();
     scene.ucmd().arg("a/a").succeeds();
@@ -1808,7 +1822,7 @@ fn test_ls_files_dirs() {
         .ucmd()
         .arg("doesntexist")
         .fails()
-        .stderr_contains(&"'doesntexist': No such file or directory");
+        .stderr_contains("'doesntexist': No such file or directory");
 
     // One exists, the other doesn't
     scene
@@ -1816,8 +1830,8 @@ fn test_ls_files_dirs() {
         .arg("a")
         .arg("doesntexist")
         .fails()
-        .stderr_contains(&"'doesntexist': No such file or directory")
-        .stdout_contains(&"a:");
+        .stderr_contains("'doesntexist': No such file or directory")
+        .stdout_contains("a:");
 }
 
 #[test]
@@ -1828,8 +1842,8 @@ fn test_ls_recursive() {
     at.mkdir("a/b");
     at.mkdir("a/b/c");
     at.mkdir("z");
-    at.touch(&at.plus_as_string("a/a"));
-    at.touch(&at.plus_as_string("a/b/b"));
+    at.touch(at.plus_as_string("a/a"));
+    at.touch(at.plus_as_string("a/b/b"));
 
     scene.ucmd().arg("a").succeeds();
     scene.ucmd().arg("a/a").succeeds();
@@ -1838,7 +1852,7 @@ fn test_ls_recursive() {
         .arg("z")
         .arg("-R")
         .succeeds()
-        .stdout_contains(&"z:");
+        .stdout_contains("z:");
     let result = scene
         .ucmd()
         .arg("--color=never")
@@ -1848,9 +1862,9 @@ fn test_ls_recursive() {
         .succeeds();
 
     #[cfg(not(windows))]
-    result.stdout_contains(&"a/b:\nb");
+    result.stdout_contains("a/b:\nb");
     #[cfg(windows)]
-    result.stdout_contains(&"a\\b:\nb");
+    result.stdout_contains("a\\b:\nb");
 }
 
 #[test]
@@ -1868,7 +1882,7 @@ fn test_ls_color() {
         .join("nested_file")
         .to_string_lossy()
         .to_string();
-    at.touch(&nested_file);
+    at.touch(nested_file);
     at.touch("test-color");
 
     let a_with_colors = "\x1b[1;34ma\x1b[0m";
@@ -1919,10 +1933,7 @@ fn test_ls_color() {
         .arg("-w=15")
         .arg("-C")
         .succeeds()
-        .stdout_only(format!(
-            "{}  test-color\nb  {}\n",
-            a_with_colors, z_with_colors
-        ));
+        .stdout_only(format!("{a_with_colors}  test-color\nb  {z_with_colors}\n"));
 }
 
 #[cfg(unix)]
@@ -1976,7 +1987,7 @@ fn test_ls_indicator_style() {
     at.mkdir("directory");
     assert!(at.dir_exists("directory"));
 
-    at.touch(&at.plus_as_string("link-src"));
+    at.touch(at.plus_as_string("link-src"));
     at.symlink_file("link-src", "link-dest.link");
     assert!(at.is_symlink("link-dest.link"));
 
@@ -2001,7 +2012,7 @@ fn test_ls_indicator_style() {
         "-p",
     ] {
         // Verify that classify and file-type both contain indicators for symlinks.
-        scene.ucmd().arg(opt).succeeds().stdout_contains(&"/");
+        scene.ucmd().arg(opt).succeeds().stdout_contains("/");
     }
 
     // Classify, Indicator options should not contain any indicators when value is none.
@@ -2017,9 +2028,9 @@ fn test_ls_indicator_style() {
             .ucmd()
             .arg(opt)
             .succeeds()
-            .stdout_does_not_contain(&"/")
-            .stdout_does_not_contain(&"@")
-            .stdout_does_not_contain(&"|");
+            .stdout_does_not_contain("/")
+            .stdout_does_not_contain("@")
+            .stdout_does_not_contain("|");
     }
 
     // Classify and File-Type all contain indicators for pipes and links.
@@ -2028,23 +2039,23 @@ fn test_ls_indicator_style() {
         // Verify that classify and file-type both contain indicators for symlinks.
         scene
             .ucmd()
-            .arg(format!("--indicator-style={}", opt))
+            .arg(format!("--indicator-style={opt}"))
             .succeeds()
-            .stdout_contains(&"@")
-            .stdout_contains(&"|");
+            .stdout_contains("@")
+            .stdout_contains("|");
     }
 
     // Test sockets. Because the canonical way of making sockets to test is with
     // TempDir, we need a separate test.
     {
-        use self::unix_socket::UnixListener;
+        use std::os::unix::net::UnixListener;
 
         let dir = tempfile::Builder::new()
             .prefix("unix_socket")
             .tempdir()
             .expect("failed to create dir");
         let socket_path = dir.path().join("sock");
-        let _listener = UnixListener::bind(&socket_path).expect("failed to create socket");
+        let _listener = UnixListener::bind(socket_path).expect("failed to create socket");
 
         new_ucmd!()
             .args(&[
@@ -2068,7 +2079,7 @@ fn test_ls_indicator_style() {
     at.mkdir("directory");
     assert!(at.dir_exists("directory"));
 
-    at.touch(&at.plus_as_string("link-src"));
+    at.touch(at.plus_as_string("link-src"));
     at.symlink_file("link-src", "link-dest.link");
     assert!(at.is_symlink("link-dest.link"));
 
@@ -2078,15 +2089,15 @@ fn test_ls_indicator_style() {
         // Verify that classify and file-type both contain indicators for symlinks.
         scene
             .ucmd()
-            .arg(format!("--indicator-style={}", opt))
+            .arg(format!("--indicator-style={opt}"))
             .succeeds()
-            .stdout_contains(&"/");
+            .stdout_contains("/");
     }
 
     // Same test as above, but with the alternate flags.
     let options = vec!["--classify", "--file-type", "-p"];
     for opt in options {
-        scene.ucmd().arg(opt).succeeds().stdout_contains(&"/");
+        scene.ucmd().arg(opt).succeeds().stdout_contains("/");
     }
 
     // Classify and File-Type all contain indicators for pipes and links.
@@ -2095,9 +2106,9 @@ fn test_ls_indicator_style() {
         // Verify that classify and file-type both contain indicators for symlinks.
         scene
             .ucmd()
-            .arg(format!("--indicator-style={}", opt))
+            .arg(format!("--indicator-style={opt}"))
             .succeeds()
-            .stdout_contains(&"@");
+            .stdout_contains("@");
     }
 }
 
@@ -2376,7 +2387,7 @@ fn test_ls_quoting_style() {
                 .arg(arg)
                 .arg("one\ntwo")
                 .succeeds()
-                .stdout_only(format!("{}\n", correct));
+                .stdout_only(format!("{correct}\n"));
         }
 
         for (arg, correct) in [
@@ -2392,7 +2403,7 @@ fn test_ls_quoting_style() {
                 .arg("--show-control-chars")
                 .arg("one\ntwo")
                 .succeeds()
-                .stdout_only(format!("{}\n", correct));
+                .stdout_only(format!("{correct}\n"));
         }
 
         for (arg, correct) in [
@@ -2414,7 +2425,7 @@ fn test_ls_quoting_style() {
                 .arg(arg)
                 .arg("one\\two")
                 .succeeds()
-                .stdout_only(format!("{}\n", correct));
+                .stdout_only(format!("{correct}\n"));
         }
 
         // Tests for a character that forces quotation in shell-style escaping
@@ -2430,7 +2441,7 @@ fn test_ls_quoting_style() {
                 .arg(arg)
                 .arg("one\n&two")
                 .succeeds()
-                .stdout_only(format!("{}\n", correct));
+                .stdout_only(format!("{correct}\n"));
         }
     }
 
@@ -2461,7 +2472,7 @@ fn test_ls_quoting_style() {
             .arg(arg)
             .arg("one two")
             .succeeds()
-            .stdout_only(format!("{}\n", correct));
+            .stdout_only(format!("{correct}\n"));
     }
 
     scene.ucmd().arg("one").succeeds().stdout_only("one\n");
@@ -2485,7 +2496,7 @@ fn test_ls_quoting_style() {
             .arg(arg)
             .arg("one")
             .succeeds()
-            .stdout_only(format!("{}\n", correct));
+            .stdout_only(format!("{correct}\n"));
     }
 }
 
@@ -2618,7 +2629,7 @@ fn test_ls_ignore_hide() {
         .arg("--ignore=READ[ME")
         .arg("-1")
         .succeeds()
-        .stderr_contains(&"Invalid pattern")
+        .stderr_contains("Invalid pattern")
         .stdout_is("CONTRIBUTING.md\nREADME.md\nREADMECAREFULLY.md\nsome_other_file\n");
 
     scene
@@ -2626,7 +2637,7 @@ fn test_ls_ignore_hide() {
         .arg("--hide=READ[ME")
         .arg("-1")
         .succeeds()
-        .stderr_contains(&"Invalid pattern")
+        .stderr_contains("Invalid pattern")
         .stdout_is("CONTRIBUTING.md\nREADME.md\nREADMECAREFULLY.md\nsome_other_file\n");
 }
 
@@ -2668,6 +2679,73 @@ fn test_ls_ignore_backups() {
         .stdout_does_not_contain(".somehiddenbackup~");
 }
 
+// This test fails on windows, see details at #3985
+#[cfg(not(windows))]
+#[test]
+fn test_ls_ignore_explicit_period() {
+    // In ls ignore patterns, leading periods must be explicitly specified
+    let scene = TestScenario::new(util_name!());
+
+    let at = &scene.fixtures;
+    at.touch(".hidden.yml");
+    at.touch("regular.yml");
+
+    scene
+        .ucmd()
+        .arg("-a")
+        .arg("--ignore")
+        .arg("?hidden.yml")
+        .succeeds()
+        .stdout_contains(".hidden.yml")
+        .stdout_contains("regular.yml");
+
+    scene
+        .ucmd()
+        .arg("-a")
+        .arg("--ignore")
+        .arg("*.yml")
+        .succeeds()
+        .stdout_contains(".hidden.yml")
+        .stdout_does_not_contain("regular.yml");
+
+    // Leading period is explicitly specified
+    scene
+        .ucmd()
+        .arg("-a")
+        .arg("--ignore")
+        .arg(".*.yml")
+        .succeeds()
+        .stdout_does_not_contain(".hidden.yml")
+        .stdout_contains("regular.yml");
+}
+
+// This test fails on windows, see details at #3985
+#[cfg(not(windows))]
+#[test]
+fn test_ls_ignore_negation() {
+    let scene = TestScenario::new(util_name!());
+
+    let at = &scene.fixtures;
+    at.touch("apple");
+    at.touch("boy");
+
+    scene
+        .ucmd()
+        .arg("--ignore")
+        .arg("[!a]*")
+        .succeeds()
+        .stdout_contains("apple")
+        .stdout_does_not_contain("boy");
+
+    scene
+        .ucmd()
+        .arg("--ignore")
+        .arg("[^a]*")
+        .succeeds()
+        .stdout_contains("apple")
+        .stdout_does_not_contain("boy");
+}
+
 #[test]
 fn test_ls_directory() {
     let scene = TestScenario::new(util_name!());
@@ -2948,31 +3026,31 @@ fn test_ls_path() {
     let file1 = "file1";
     let file2 = "file2";
     let dir = "dir";
-    let path = &format!("{}/{}", dir, file2);
+    let path = &format!("{dir}/{file2}");
 
     at.mkdir(dir);
     at.touch(file1);
     at.touch(path);
 
-    let expected_stdout = &format!("{}\n", path);
+    let expected_stdout = &format!("{path}\n");
     scene.ucmd().arg(path).run().stdout_is(expected_stdout);
 
-    let expected_stdout = &format!("./{}\n", path);
+    let expected_stdout = &format!("./{path}\n");
     scene
         .ucmd()
-        .arg(format!("./{}", path))
+        .arg(format!("./{path}"))
         .run()
         .stdout_is(expected_stdout);
 
     let abs_path = format!("{}/{}", at.as_string(), path);
     let expected_stdout = if cfg!(windows) {
-        format!("\'{}\'\n", abs_path)
+        format!("\'{abs_path}\'\n")
     } else {
-        format!("{}\n", abs_path)
+        format!("{abs_path}\n")
     };
     scene.ucmd().arg(&abs_path).run().stdout_is(expected_stdout);
 
-    let expected_stdout = format!("{}\n{}\n", path, file1);
+    let expected_stdout = format!("{path}\n{file1}\n");
     scene
         .ucmd()
         .arg(file1)
@@ -2989,8 +3067,18 @@ fn test_ls_dangling_symlinks() {
     at.mkdir("temp_dir");
     at.symlink_file("does_not_exist", "temp_dir/dangle");
 
-    scene.ucmd().arg("-L").arg("temp_dir/dangle").fails();
-    scene.ucmd().arg("-H").arg("temp_dir/dangle").fails();
+    scene
+        .ucmd()
+        .arg("-L")
+        .arg("temp_dir/dangle")
+        .fails()
+        .code_is(2);
+    scene
+        .ucmd()
+        .arg("-H")
+        .arg("temp_dir/dangle")
+        .fails()
+        .code_is(2);
 
     scene
         .ucmd()
@@ -3003,6 +3091,7 @@ fn test_ls_dangling_symlinks() {
         .arg("-Li")
         .arg("temp_dir")
         .fails()
+        .code_is(1)
         .stderr_contains("cannot access")
         .stderr_contains("No such file or directory")
         .stdout_contains(if cfg!(windows) { "dangle" } else { "? dangle" });
@@ -3012,6 +3101,7 @@ fn test_ls_dangling_symlinks() {
         .arg("-Ll")
         .arg("temp_dir")
         .fails()
+        .code_is(1)
         .stdout_contains("l?????????");
 
     #[cfg(unix)]
@@ -3020,6 +3110,7 @@ fn test_ls_dangling_symlinks() {
         at.touch("temp_dir/real_file");
 
         let real_file_res = scene.ucmd().arg("-Li1").arg("temp_dir").fails();
+        real_file_res.code_is(1);
         let real_file_stdout_len = String::from_utf8(real_file_res.stdout().to_owned())
             .ok()
             .unwrap()
@@ -3031,6 +3122,7 @@ fn test_ls_dangling_symlinks() {
             .len();
 
         let dangle_file_res = scene.ucmd().arg("-Li1").arg("temp_dir").fails();
+        dangle_file_res.code_is(1);
         let dangle_stdout_len = String::from_utf8(dangle_file_res.stdout().to_owned())
             .ok()
             .unwrap()
@@ -3101,7 +3193,7 @@ fn test_ls_context_format() {
     ] {
         let format = format!("--format={}", word);
         ts.ucmd()
-            .args(&[&"-Z", &format.as_str(), &"/"])
+            .args(&["-Z", format.as_str(), "/"])
             .succeeds()
             .stdout_only(
                 unwrap_or_return!(expected_result(&ts, &["-Z", format.as_str(), "/"])).stdout_str(),
@@ -3154,6 +3246,7 @@ fn test_ls_multiple_a_A() {
 }
 
 #[test]
+#[cfg(feature = "ln")]
 fn test_ls_quoting() {
     let scene = TestScenario::new(util_name!());
 
@@ -3173,6 +3266,7 @@ fn test_ls_quoting() {
 }
 
 #[test]
+#[cfg(feature = "ln")]
 fn test_ls_quoting_color() {
     let scene = TestScenario::new(util_name!());
 
@@ -3201,6 +3295,7 @@ fn test_ls_dereference_looped_symlinks_recursive() {
 
     ucmd.args(&["-RL", "loop"])
         .fails()
+        .code_is(2)
         .stderr_contains("not listing already-listed directory");
 }
 
