@@ -1,4 +1,9 @@
-use crate::common::util::*;
+use crate::common::util::TestScenario;
+
+#[test]
+fn test_invalid_arg() {
+    new_ucmd!().arg("--definitely-invalid").fails().code_is(1);
+}
 
 #[test]
 fn test_stdin_no_newline() {
