@@ -1,10 +1,15 @@
-use crate::common::util::*;
+use crate::common::util::TestScenario;
 use std::thread::sleep;
 
 // General observation: nohup.out will not be created in tests run by cargo test
 // because stdin/stdout is not attached to a TTY.
 // All that can be tested is the side-effects.
 
+#[test]
+fn test_invalid_arg() {
+    new_ucmd!().arg("--definitely-invalid").fails().code_is(125);
+}
+
 #[test]
 #[cfg(any(
     target_os = "linux",
