@@ -1,10 +1,14 @@
 // spell-checker:ignore (ToDO) Sdivide
+extern crate time;
 
-use crate::common::util::*;
-use chrono::offset::Local;
-use chrono::DateTime;
-use chrono::Duration;
+use crate::common::util::{TestScenario, UCommand};
 use std::fs::metadata;
+use time::macros::format_description;
+use time::Duration;
+use time::OffsetDateTime;
+
+const DATE_TIME_FORMAT: &[time::format_description::FormatItem] =
+    format_description!("[month repr:short] [day] [hour]:[minute] [year]");
 
 fn file_last_modified_time(ucmd: &UCommand, path: &str) -> String {
     let tmp_dir_path = ucmd.get_full_fixture_path(path);
@@ -13,28 +17,28 @@ fn file_last_modified_time(ucmd: &UCommand, path: &str) -> String {
         .map(|i| {
             i.modified()
                 .map(|x| {
-                    let date_time: DateTime<Local> = x.into();
-                    date_time.format("%b %d %H:%M %Y").to_string()
+                    let date_time: OffsetDateTime = x.into();
+                    date_time.format(&DATE_TIME_FORMAT).unwrap()
                 })
                 .unwrap_or_default()
         })
         .unwrap_or_default()
 }
 
-fn all_minutes(from: DateTime<Local>, to: DateTime<Local>) -> Vec<String> {
+fn all_minutes(from: OffsetDateTime, to: OffsetDateTime) -> Vec<String> {
     let to = to + Duration::minutes(1);
-    const FORMAT: &str = "%b %d %H:%M %Y";
+    // const FORMAT: &str = "%b %d %H:%M %Y";
     let mut vec = vec![];
     let mut current = from;
     while current < to {
-        vec.push(current.format(FORMAT).to_string());
+        vec.push(current.format(&DATE_TIME_FORMAT).unwrap());
         current += Duration::minutes(1);
     }
     vec
 }
 
-fn valid_last_modified_template_vars(from: DateTime<Local>) -> Vec<Vec<(String, String)>> {
-    all_minutes(from, Local::now())
+fn valid_last_modified_template_vars(from: OffsetDateTime) -> Vec<Vec<(String, String)>> {
+    all_minutes(from, OffsetDateTime::now_utc())
         .into_iter()
         .map(|time| vec![("{last_modified_time}".to_string(), time)])
         .collect()
@@ -155,7 +159,7 @@ fn test_with_valid_page_ranges() {
     scenario
         .args(&["--pages=20:5", test_file_path])
         .fails()
-        .stderr_is("pr: invalid --pages argument '20:5'")
+        .stderr_is("pr: invalid --pages argument '20:5'\n")
         .stdout_is("");
     new_ucmd!()
         .args(&["--pages=1:5", test_file_path])
@@ -164,17 +168,17 @@ fn test_with_valid_page_ranges() {
     new_ucmd!()
         .args(&["--pages=-1:5", test_file_path])
         .fails()
-        .stderr_is("pr: invalid --pages argument '-1:5'")
+        .stderr_is("pr: invalid --pages argument '-1:5'\n")
         .stdout_is("");
     new_ucmd!()
         .args(&["--pages=1:-5", test_file_path])
         .fails()
-        .stderr_is("pr: invalid --pages argument '1:-5'")
+        .stderr_is("pr: invalid --pages argument '1:-5'\n")
         .stdout_is("");
     new_ucmd!()
         .args(&["--pages=5:1", test_file_path])
         .fails()
-        .stderr_is("pr: invalid --pages argument '5:1'")
+        .stderr_is("pr: invalid --pages argument '5:1'\n")
         .stdout_is("");
 }
 
@@ -250,7 +254,7 @@ fn test_with_suppress_error_option() {
 fn test_with_stdin() {
     let expected_file_path = "stdin.log.expected";
     let mut scenario = new_ucmd!();
-    let start = Local::now();
+    let start = OffsetDateTime::now_utc();
     scenario
         .pipe_in_fixture("stdin.log")
         .args(&["--pages=1:2", "-n", "-"])
@@ -313,7 +317,7 @@ fn test_with_mpr() {
     let expected_test_file_path = "mpr.log.expected";
     let expected_test_file_path1 = "mpr1.log.expected";
     let expected_test_file_path2 = "mpr2.log.expected";
-    let start = Local::now();
+    let start = OffsetDateTime::now_utc();
     new_ucmd!()
         .args(&["--pages=1:2", "-m", "-n", test_file_path, test_file_path1])
         .succeeds()
@@ -322,7 +326,7 @@ fn test_with_mpr() {
             &valid_last_modified_template_vars(start),
         );
 
-    let start = Local::now();
+    let start = OffsetDateTime::now_utc();
     new_ucmd!()
         .args(&["--pages=2:4", "-m", "-n", test_file_path, test_file_path1])
         .succeeds()
@@ -331,7 +335,7 @@ fn test_with_mpr() {
             &valid_last_modified_template_vars(start),
         );
 
-    let start = Local::now();
+    let start = OffsetDateTime::now_utc();
     new_ucmd!()
         .args(&[
             "--pages=1:2",
@@ -356,13 +360,13 @@ fn test_with_mpr_and_column_options() {
     new_ucmd!()
         .args(&["--column=2", "-m", "-n", test_file_path])
         .fails()
-        .stderr_is("pr: cannot specify number of columns when printing in parallel")
+        .stderr_is("pr: cannot specify number of columns when printing in parallel\n")
         .stdout_is("");
 
     new_ucmd!()
         .args(&["-a", "-m", "-n", test_file_path])
         .fails()
-        .stderr_is("pr: cannot specify both printing across and printing in parallel")
+        .stderr_is("pr: cannot specify both printing across and printing in parallel\n")
         .stdout_is("");
 }
 
@@ -409,7 +413,6 @@ fn test_with_pr_core_utils_tests() {
         let value = file_last_modified_time(&scenario, test_file_path);
         let mut arguments: Vec<&str> = flags
             .split(' ')
-            .into_iter()
             .filter(|i| i.trim() != "")
             .collect::<Vec<&str>>();
 
@@ -439,7 +442,7 @@ fn test_with_join_lines_option() {
     let test_file_2 = "test.log";
     let expected_file_path = "joined.log.expected";
     let mut scenario = new_ucmd!();
-    let start = Local::now();
+    let start = OffsetDateTime::now_utc();
     scenario
         .args(&["+1:2", "-J", "-m", test_file_1, test_file_2])
         .run()
@@ -461,3 +464,13 @@ fn test_value_for_number_lines() {
     // Therefore, pr tries to access the file, which does not exist.
     new_ucmd!().args(&["-n", "foo5.txt", "test.log"]).fails();
 }
+
+#[test]
+fn test_help() {
+    new_ucmd!().arg("--help").succeeds();
+}
+
+#[test]
+fn test_version() {
+    new_ucmd!().arg("--version").succeeds();
+}
