@@ -2,7 +2,7 @@
 
 use std::path::PathBuf;
 
-use crate::common::util::*;
+use crate::common::util::{TestScenario, UCommand};
 
 #[test]
 fn test_invalid_arg() {
@@ -60,7 +60,7 @@ fn symlinked_env() -> Env {
     // Note: on Windows this requires admin permissions
     at.symlink_dir("subdir", "symdir");
     let root = PathBuf::from(at.root_dir_resolved());
-    ucmd.raw.current_dir(root.join("symdir"));
+    ucmd.current_dir(root.join("symdir"));
     #[cfg(not(windows))]
     ucmd.env("PWD", root.join("symdir"));
     Env {
@@ -90,6 +90,35 @@ fn test_symlinked_default() {
     env.ucmd.succeeds().stdout_is(env.subdir + "\n");
 }
 
+#[test]
+fn test_symlinked_default_posix() {
+    let mut env = symlinked_env();
+    env.ucmd
+        .env("POSIXLY_CORRECT", "1")
+        .succeeds()
+        .stdout_is(env.symdir.clone() + "\n");
+}
+
+#[test]
+fn test_symlinked_default_posix_l() {
+    let mut env = symlinked_env();
+    env.ucmd
+        .env("POSIXLY_CORRECT", "1")
+        .arg("-L")
+        .succeeds()
+        .stdout_is(env.symdir + "\n");
+}
+
+#[test]
+fn test_symlinked_default_posix_p() {
+    let mut env = symlinked_env();
+    env.ucmd
+        .env("POSIXLY_CORRECT", "1")
+        .arg("-P")
+        .succeeds()
+        .stdout_is(env.symdir + "\n");
+}
+
 #[cfg(not(windows))]
 pub mod untrustworthy_pwd_var {
     use std::path::Path;
