@@ -10,9 +10,9 @@ extern crate touch;
 use self::touch::filetime::{self, FileTime};
 
 extern crate time;
-use time::macros::{datetime, format_description};
+use time::macros::format_description;
 
-use crate::common::util::*;
+use crate::common::util::{AtPath, TestScenario};
 use std::fs::remove_file;
 use std::path::PathBuf;
 
@@ -34,7 +34,7 @@ fn get_symlink_times(at: &AtPath, path: &str) -> (FileTime, FileTime) {
 }
 
 fn set_file_times(at: &AtPath, path: &str, atime: FileTime, mtime: FileTime) {
-    filetime::set_file_times(&at.plus_as_string(path), atime, mtime).unwrap();
+    filetime::set_file_times(at.plus_as_string(path), atime, mtime).unwrap();
 }
 
 // Adjusts for local timezone
@@ -49,12 +49,7 @@ fn str_to_filetime(format: &str, s: &str) -> FileTime {
         _ => panic!("unexpected dt format"),
     };
     let tm = time::PrimitiveDateTime::parse(s, &format_description).unwrap();
-    let d = match time::OffsetDateTime::now_local() {
-        Ok(now) => now,
-        Err(e) => {
-            panic!("Error {} retrieving the OffsetDateTime::now_local", e);
-        }
-    };
+    let d = time::OffsetDateTime::now_utc();
     let offset_dt = tm.assume_offset(d.offset());
     FileTime::from_unix_time(offset_dt.unix_timestamp(), tm.nanosecond())
 }
@@ -108,10 +103,7 @@ fn test_touch_set_mdhm_time() {
 
     let start_of_year = str_to_filetime(
         "%Y%m%d%H%M",
-        &format!(
-            "{}01010000",
-            time::OffsetDateTime::now_local().unwrap().year()
-        ),
+        &format!("{}01010000", time::OffsetDateTime::now_utc().year()),
     );
     let (atime, mtime) = get_file_times(&at, file);
     assert_eq!(atime, mtime);
@@ -425,7 +417,7 @@ fn test_touch_set_date3() {
 
     assert!(at.file_exists(file));
 
-    let expected = FileTime::from_unix_time(1623786360, 0);
+    let expected = FileTime::from_unix_time(1_623_786_360, 0);
     let (atime, mtime) = get_file_times(&at, file);
     assert_eq!(atime, mtime);
     assert_eq!(atime, expected);
@@ -464,9 +456,9 @@ fn test_touch_set_date5() {
     // Slightly different result on Windows for nano seconds
     // TODO: investigate
     #[cfg(windows)]
-    let expected = FileTime::from_unix_time(67413, 23456700);
+    let expected = FileTime::from_unix_time(67413, 23_456_700);
     #[cfg(not(windows))]
-    let expected = FileTime::from_unix_time(67413, 23456789);
+    let expected = FileTime::from_unix_time(67413, 23_456_789);
 
     let (atime, mtime) = get_file_times(&at, file);
     assert_eq!(atime, mtime);
@@ -485,7 +477,7 @@ fn test_touch_set_date6() {
 
     assert!(at.file_exists(file));
 
-    let expected = FileTime::from_unix_time(946684800, 0);
+    let expected = FileTime::from_unix_time(946_684_800, 0);
 
     let (atime, mtime) = get_file_times(&at, file);
     assert_eq!(atime, mtime);
@@ -504,7 +496,7 @@ fn test_touch_set_date7() {
 
     assert!(at.file_exists(file));
 
-    let expected = FileTime::from_unix_time(1074254400, 0);
+    let expected = FileTime::from_unix_time(1_074_254_400, 0);
 
     let (atime, mtime) = get_file_times(&at, file);
     assert_eq!(atime, mtime);
@@ -512,6 +504,98 @@ fn test_touch_set_date7() {
     assert_eq!(mtime, expected);
 }
 
+/// Test for setting the date by a relative time unit.
+#[test]
+fn test_touch_set_date_relative_smoke() {
+    // From the GNU documentation:
+    //
+    // > The unit of time displacement may be selected by the string
+    // > ‘year’ or ‘month’ for moving by whole years or months.  These
+    // > are fuzzy units, as years and months are not all of equal
+    // > duration.  More precise units are ‘fortnight’ which is worth 14
+    // > days, ‘week’ worth 7 days, ‘day’ worth 24 hours, ‘hour’ worth
+    // > 60 minutes, ‘minute’ or ‘min’ worth 60 seconds, and ‘second’ or
+    // > ‘sec’ worth one second.  An ‘s’ suffix on these units is
+    // > accepted and ignored.
+    //
+    let times = [
+        // "-1 year", "+1 year", "-1 years", "+1 years",
+        // "-1 month", "+1 month", "-1 months", "+1 months",
+        "-1 fortnight",
+        "+1 fortnight",
+        "-1 fortnights",
+        "+1 fortnights",
+        "fortnight",
+        "fortnights",
+        "-1 week",
+        "+1 week",
+        "-1 weeks",
+        "+1 weeks",
+        "week",
+        "weeks",
+        "-1 day",
+        "+1 day",
+        "-1 days",
+        "+1 days",
+        "day",
+        "days",
+        "-1 hour",
+        "+1 hour",
+        "-1 hours",
+        "+1 hours",
+        "hour",
+        "hours",
+        "-1 minute",
+        "+1 minute",
+        "-1 minutes",
+        "+1 minutes",
+        "minute",
+        "minutes",
+        "-1 min",
+        "+1 min",
+        "-1 mins",
+        "+1 mins",
+        "min",
+        "mins",
+        "-1 second",
+        "+1 second",
+        "-1 seconds",
+        "+1 seconds",
+        "second",
+        "seconds",
+        "-1 sec",
+        "+1 sec",
+        "-1 secs",
+        "+1 secs",
+        "sec",
+        "secs",
+    ];
+    for time in times {
+        let (at, mut ucmd) = at_and_ucmd!();
+        at.touch("f");
+        ucmd.args(&["-d", time, "f"])
+            .succeeds()
+            .no_stderr()
+            .no_stdout();
+    }
+
+    // From the GNU documentation:
+    //
+    // > The string ‘tomorrow’ is worth one day in the future
+    // > (equivalent to ‘day’), the string ‘yesterday’ is worth one day
+    // > in the past (equivalent to ‘day ago’).
+    //
+    let times = ["yesterday", "tomorrow", "now"];
+    for time in times {
+        let (at, mut ucmd) = at_and_ucmd!();
+        at.touch("f");
+        ucmd.args(&["-d", time, "f"])
+            .succeeds()
+            .no_stderr()
+            .no_stdout();
+    }
+}
+
 #[test]
 fn test_touch_set_date_wrong_format() {
     let (_at, mut ucmd) = at_and_ucmd!();
@@ -538,61 +622,21 @@ fn test_touch_mtime_dst_succeeds() {
     assert_eq!(target_time, mtime);
 }
 
-// // is_dst_switch_hour returns true if timespec ts is just before the switch
-// // to Daylight Saving Time.
-// // For example, in EST (UTC-5), Timespec { sec: 1583647200, nsec: 0 }
-// // for March 8 2020 01:00:00 AM
-// // is just before the switch because on that day clock jumps by 1 hour,
-// // so 1 minute after 01:59:00 is 03:00:00.
-// fn is_dst_switch_hour(ts: time::Timespec) -> bool {
-//     let ts_after = ts + time::Duration::hours(1);
-//     let tm = time::at(ts);
-//     let tm_after = time::at(ts_after);
-//     tm_after.tm_hour == tm.tm_hour + 2
-// }
-
-// get_dst_switch_hour returns date string for which touch -m -t fails.
-// For example, in EST (UTC-5), that will be "202003080200" so
-// touch -m -t 202003080200 file
-// fails (that date/time does not exist).
-// In other locales it will be a different date/time, and in some locales
-// it doesn't exist at all, in which case this function will return None.
-fn get_dst_switch_hour() -> Option<String> {
-    //let now = time::OffsetDateTime::now_local().unwrap();
-    let now = match time::OffsetDateTime::now_local() {
-        Ok(now) => now,
-        Err(e) => {
-            panic!("Error {} retrieving the OffsetDateTime::now_local", e);
-        }
-    };
-
-    // Start from January 1, 2020, 00:00.
-    let tm = datetime!(2020-01-01 00:00 UTC);
-    tm.to_offset(now.offset());
-
-    // let mut ts = tm.to_timespec();
-    // // Loop through all hours in year 2020 until we find the hour just
-    // // before the switch to DST.
-    // for _i in 0..(366 * 24) {
-    //     // if is_dst_switch_hour(ts) {
-    //     //     let mut tm = time::at(ts);
-    //     //     tm.tm_hour += 1;
-    //     //     let s = time::strftime("%Y%m%d%H%M", &tm).unwrap();
-    //     //     return Some(s);
-    //     // }
-    //     ts = ts + time::Duration::hours(1);
-    // }
-    None
-}
-
 #[test]
+#[ignore = "not implemented"]
 fn test_touch_mtime_dst_fails() {
     let (_at, mut ucmd) = at_and_ucmd!();
     let file = "test_touch_set_mtime_dst_fails";
 
-    if let Some(s) = get_dst_switch_hour() {
-        ucmd.args(&["-m", "-t", &s, file]).fails();
-    }
+    // Some timezones use daylight savings time, this leads to problems if the
+    // specified time is within the jump forward. In EST (UTC-5), there is a
+    // jump from 1:59AM to 3:00AM on, March 8 2020, so any thing in-between is
+    // invalid.
+    // See https://www.gnu.org/software/libc/manual/html_node/TZ-Variable.html
+    // for information on the TZ variable, which where the string is copied from.
+    ucmd.env("TZ", "EST+5EDT,M3.2.0/2,M11.1.0/2")
+        .args(&["-m", "-t", "202003080200", file])
+        .fails();
 }
 
 #[test]
@@ -620,8 +664,7 @@ fn test_touch_no_such_file_error_msg() {
     let path_str = path.to_str().unwrap();
 
     new_ucmd!().arg(&path).fails().stderr_only(format!(
-        "touch: cannot touch '{}': No such file or directory",
-        path_str
+        "touch: cannot touch '{path_str}': No such file or directory\n"
     ));
 }
 
@@ -663,7 +706,7 @@ fn test_touch_permission_denied_error_msg() {
 
     let full_path = at.plus_as_string(path_str);
     ucmd.arg(&full_path).fails().stderr_only(format!(
-        "touch: cannot touch '{}': Permission denied",
+        "touch: cannot touch '{}': Permission denied\n",
         &full_path
     ));
 }
@@ -671,10 +714,7 @@ fn test_touch_permission_denied_error_msg() {
 #[test]
 fn test_touch_no_args() {
     let mut ucmd = new_ucmd!();
-    ucmd.fails().stderr_only(
-        r##"touch: missing file operand
-Try 'touch --help' for more information."##,
-    );
+    ucmd.fails().no_stdout().usage_error("missing file operand");
 }
 
 #[test]
