@@ -1,5 +1,9 @@
-use crate::common::util::*;
+use crate::common::util::TestScenario;
 
+#[test]
+fn test_invalid_arg() {
+    new_ucmd!().arg("--definitely-invalid").fails().code_is(1);
+}
 #[test]
 fn test_sort_call_graph() {
     new_ucmd!()
@@ -46,7 +50,5 @@ fn test_multiple_arguments() {
         .arg("call_graph.txt")
         .arg("invalid_file")
         .fails()
-        .stderr_contains(
-            "Found argument 'invalid_file' which wasn't expected, or isn't valid in this context",
-        );
+        .stderr_contains("unexpected argument 'invalid_file' found");
 }
