@@ -1,4 +1,4 @@
-use crate::common::util::*;
+use crate::common::util::{vec_of_size, TestScenario};
 
 // spell-checker:ignore (flags) lwmcL clmwL ; (path) bogusfile emptyfile manyemptylines moby notrailingnewline onelongemptyline onelongword weirdchars
 
@@ -26,12 +26,12 @@ fn test_count_bytes_large_stdin() {
         128 * 1024,
     ] {
         let data = vec_of_size(n);
-        let expected = format!("{}\n", n);
+        let expected = format!("{n}\n");
         new_ucmd!()
             .args(&["-c"])
             .pipe_in(data)
             .succeeds()
-            .stdout_is_bytes(&expected.as_bytes());
+            .stdout_is_bytes(expected.as_bytes());
     }
 }
 
@@ -464,3 +464,59 @@ fn test_files0_from_with_stdin_try_read_from_stdin() {
         .stderr_contains(MSG)
         .stdout_is("");
 }
+
+#[test]
+fn test_total_auto() {
+    new_ucmd!()
+        .args(&["lorem_ipsum.txt", "--total=auto"])
+        .run()
+        .stdout_is(" 13 109 772 lorem_ipsum.txt\n");
+
+    new_ucmd!()
+        .args(&["lorem_ipsum.txt", "moby_dick.txt", "--total=auto"])
+        .run()
+        .stdout_is(
+            "  13  109  772 lorem_ipsum.txt\n  18  204 1115 moby_dick.txt\n  31  313 1887 total\n",
+        );
+}
+
+#[test]
+fn test_total_always() {
+    new_ucmd!()
+        .args(&["lorem_ipsum.txt", "--total=always"])
+        .run()
+        .stdout_is(" 13 109 772 lorem_ipsum.txt\n 13 109 772 total\n");
+
+    new_ucmd!()
+        .args(&["lorem_ipsum.txt", "moby_dick.txt", "--total=always"])
+        .run()
+        .stdout_is(
+            "  13  109  772 lorem_ipsum.txt\n  18  204 1115 moby_dick.txt\n  31  313 1887 total\n",
+        );
+}
+
+#[test]
+fn test_total_never() {
+    new_ucmd!()
+        .args(&["lorem_ipsum.txt", "--total=never"])
+        .run()
+        .stdout_is(" 13 109 772 lorem_ipsum.txt\n");
+
+    new_ucmd!()
+        .args(&["lorem_ipsum.txt", "moby_dick.txt", "--total=never"])
+        .run()
+        .stdout_is("  13  109  772 lorem_ipsum.txt\n  18  204 1115 moby_dick.txt\n");
+}
+
+#[test]
+fn test_total_only() {
+    new_ucmd!()
+        .args(&["lorem_ipsum.txt", "--total=only"])
+        .run()
+        .stdout_is("13 109 772\n");
+
+    new_ucmd!()
+        .args(&["lorem_ipsum.txt", "moby_dick.txt", "--total=only"])
+        .run()
+        .stdout_is("31 313 1887\n");
+}
