@@ -3,7 +3,14 @@
 //  * For the full copyright and license information, please view the LICENSE
 //  * file that was distributed with this source code.
 
-use crate::common::util::*;
+#[cfg(unix)]
+use crate::common::util::expected_result;
+use crate::common::util::{is_ci, whoami, TestScenario};
+
+#[test]
+fn test_invalid_arg() {
+    new_ucmd!().arg("--definitely-invalid").fails().code_is(1);
+}
 
 #[test]
 #[cfg(unix)]
@@ -38,7 +45,7 @@ fn test_normal_compare_env() {
     if whoami == "nobody" {
         println!("test skipped:");
     } else if !is_ci() {
-        new_ucmd!().succeeds().stdout_is(format!("{}\n", whoami));
+        new_ucmd!().succeeds().stdout_is(format!("{whoami}\n"));
     } else {
         println!("test skipped:");
     }
